package nemi;

import cytoscape.CyNetwork;
import giny.model.Edge;
import giny.model.Node;
import nemi.apriori.Apriori;
import nemi.apriori.GraphMotif;
import nemi.apriori.AprioriInstance;
import nemi.cluster.Cluster;
import nemi.compare.ProteinCompare;
import nemi.gui.NeMiFinalView;

import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.util.*;

/**
 * Created by IntelliJ IDEA.
 * User: kristost
 * Date: 19.mai.2006
 * Time: 16:12:52
 * Testing the MotifFinder, gSpan and Apriori algorithms, by timing.
 */
public class Tester {

    Vector<Vector<Long>> aprioriTimes = new Vector<Vector<Long>>();
    Vector<Vector<Long>> motifTimes = new Vector<Vector<Long>>();
    Vector<Vector<Long>> gspanTimes = new Vector<Vector<Long>>();
    Vector<Integer> motifSizes = new Vector<Integer>();
    NeMiFinalView.StatusInfo status;
    CyNetwork network, motif;

    /**
     * @param network the network to look for the motif in
     * @param motif motif to be found in network
     * @param clusters clustered nodes, used by apriori & gspan
     * @param comp a node to node comparer
     * @param status a status window to report progress
     */
    public Tester(CyNetwork network, CyNetwork motif,
                  Collection<Cluster> clusters, ProteinCompare comp, NeMiFinalView.StatusInfo status) {

        this.network = network;
        this.motif = motif;
        this.status = status;
        System.out.println("\n\nCreating new tester.");
        System.out.println(", comparer " + comp + ", #clusters " + clusters.size());

        long testTime = System.currentTimeMillis();
        int prevS = 0;
        for (int i = 0; i < 12; i++) {
            System.out.println("motif size " + motif.getNodeCount()
                    + " and network size " + network.getNodeCount());

            aprioriTimes.add(new Vector<Long>());
            motifTimes.add(new Vector<Long>());
            gspanTimes.add(new Vector<Long>());

            // test
            if (motif.getNodeCount() > prevS) {
                for (int run = 0; run < 1; run ++) {
                    run(network, motif, clusters, comp, i);
                }
                prevS = motif.getNodeCount();
            }


            // enlarge the motif
            for (int j = 0; j < 1; j++) {
                if (!expandMotif(network, motif)) {
                    System.out.println("Unable to expand motif");
                }
            }
        }
        testTime = System.currentTimeMillis()-testTime;
        String t = (System.currentTimeMillis() % 10000) + "";
        writeResults("s:\\temp\\runtimeComparisons" + t + ".txt", testTime);
    }


    private void run(CyNetwork network, CyNetwork motif,
                     Collection<Cluster> clusters, ProteinCompare comp, int i) {
        int motifSize = motif.getNodeCount();

        motifSizes.add(motifSize);

        nemi.Toolkit.networkNeigMap = new HashMap<CyNetwork, Map<Node, Set<Node>>>();
        nemi.Toolkit.id2neighs = new HashMap<String, Vector<Node>>();
        int suppTresh = 2;
        long time = System.currentTimeMillis();

        // APRIORI
        if (motifSize <= 8) {

            System.out.println("\n\ntesting apriori at motif size " + motifSize);

            Apriori a = new Apriori(clusters, network, 1);
            Map<GraphMotif, Vector<AprioriInstance>> aprRes = a.apply(motifSize, suppTresh);
            aprioriTimes.get(i).add(System.currentTimeMillis() - time);
        }
         /*
        // MOTIFFINDER
        time = System.currentTimeMillis();
        System.out.println("\n\ntesting motiffinder....");
        double branchFactor = .5;
        int maxResults = 100;
        AssignmentResults assRes = new AssignmentResults(maxResults,
                new MessagePane());
        MotifFinder mf = new MotifFinder(network, motif, comp,
                branchFactor, new MotifFinderPanel(status), assRes);
        mf.run();
        motifTimes.get(i).add(System.currentTimeMillis() - time);

        // GSPAN
        time = System.currentTimeMillis();
        System.out.println("\n\ntesting gspan....");
        GSpanMining gs = new GSpanMining(NeMiFinalView.statusObject);
        Map<String, Map<CyNetwork, Node>> str2origNet2origNode =
                new HashMap<String, Map<CyNetwork, Node>>();
        HashMap<CyNetwork, CyNetwork> netmap = new HashMap<CyNetwork, CyNetwork>();
        Set<CyNetwork> nets = new HashSet<CyNetwork>();
        nets.add(network);
        nets.add(motif);
        List<DfsResult> gspanres = gs.subGraphMiningMulti(clusters, nets, suppTresh, netmap, str2origNet2origNode);
        gspanTimes.get(i).add(System.currentTimeMillis() - time);
        */

    }

    Random r = new Random();

    private boolean expandMotif(CyNetwork net, CyNetwork motif) {
        // int[] motifNodes = motif.getNodeIndicesArray();
        // for (int ix : motifNodes) {
        int size = motif.getNodeCount();
        for (Iterator<Node> nodeIter = motif.nodesIterator(); nodeIter.hasNext();) {
            nemi.Toolkit.id2neighs = new HashMap<String, Vector<Node>>();
            Node motifNode = nodeIter.next();
            Collection<Node> mNeighs = nemi.Toolkit.getNodeNeighborMap(motif).get(motifNode); 
            Collection<Node> nNeighs = nemi.Toolkit.getNeighbors(new NetNode(network, motifNode));
            if (!mNeighs.containsAll(nNeighs)) {
                Vector<Node> rest = new Vector<Node>(nNeighs);
                rest.removeAll(mNeighs);

                Edge newEdge = null;
                for (Node node : rest) {
                    newEdge = getEdge(motifNode, node, net);
                    if (newEdge != null) break;
                }

                if (newEdge != null) {
                    // System.out.println("Error, couldn't find edge");
                    if (newEdge.getSource() == motifNode) {
                        motif.addNode(newEdge.getTarget());
                    } else {
                        motif.addNode(newEdge.getSource());
                    }
                    motif.addEdge(newEdge);

                    // if (size == motif.getNodeCount()) return false;
                    return true;
                }
            }
        }
        return false;
    }

    private Edge getEdge(Node fromNode, Node toNode, CyNetwork network) {
        int[] edges = network.getEdgeIndicesArray();
        for (int ix : edges) {
            Edge e = network.getEdge(ix);
            if (e.getSource() == fromNode && e.getTarget() == toNode) {
                return e;
            }
        }
        return null;
    }

    private void writeResults(String filename, long testTime) {
        File writeTo = new File(filename);

        try {
            FileWriter fw = new FileWriter(writeTo, false);
            fw.write("#network: " + network.getTitle() + ", motif: " + motif.getTitle() + " took " + (testTime/1000) + " sec.\n");
            fw.write("#motifsizes\n");
            for (Integer integer : motifSizes)
                fw.write(integer + "\n");
            fw.write("#apriori\n");
            for (Vector<Long> aLong : aprioriTimes)
                if (avg(aLong) > 0)
                    fw.write(avg(aLong) + "\n");
            fw.write("#motiffinder\n");
            for (Vector<Long> aLong : motifTimes)
                fw.write(avg(aLong) + "\n");
            fw.write("#gspan\n");
            for (Vector<Long> aLong : gspanTimes)
                fw.write(avg(aLong) + "\n");
            fw.close();
            System.out.println("Wrote to " + writeTo);
        } catch (IOException e) {
            e.printStackTrace();  //To change body of catch statement use File | Settings | File Templates.
        }
    }

    private long avg(Collection<Long> numbers) {
        long sum = 0;
        for (Long aLong : numbers) {
            sum += aLong;
        }
        return (numbers.size() == 0) ? 0 : (sum / numbers.size());
    }

    private long avgi(Collection<Integer> numbers) {
        long sum = 0;
        for (Integer aLong : numbers) {
            sum += aLong;
        }
        return (sum / numbers.size());

    }

}
