package nemi;

import cytoscape.CyEdge;
import cytoscape.CyNetwork;
import cytoscape.CyNode;
import edu.uci.ics.jung.graph.impl.DirectedSparseEdge;
import edu.uci.ics.jung.graph.impl.DirectedSparseGraph;
import edu.uci.ics.jung.graph.impl.SparseVertex;
import edu.uci.ics.jung.graph.impl.UndirectedSparseEdge;
import giny.model.Edge;
import giny.model.GraphPerspective;
import giny.model.Node;
import nemi.apriori.GraphDistance;
import nemi.compare.ProteinCompare;
import nemi.protein.Csa;

import javax.swing.*;
import java.io.*;
import java.util.*;

/**
 * A collection of methods commonly used by the NeMi plugin. All methods are static.
 */
public class Toolkit {

    public static Map<CyNetwork, Map<Node, Set<Node>>> networkNeigMap = new HashMap<CyNetwork, Map<Node, Set<Node>>>();
    public static HashMap<CyNetwork, GraphDistance> networkDistanceMap = new HashMap<CyNetwork, GraphDistance>();
    public static String NetPrefix = "NeMitemp";
    public final static double tfProbLimit = 0.01;
    public static Map<String, Map<String, Double>> id2fac2p = null;


    public static Map<String, Map<String, Double>>  readBindings(){
     // read file
    if(id2fac2p!=null ) return id2fac2p;
           id2fac2p = new HashMap<String, Map<String, Double>>();
        try {
            BufferedReader reader = new BufferedReader(new FileReader("plugins/bindings.txt"));
            while (reader.ready()) {
                StringTokenizer tokens = new StringTokenizer(reader.readLine());
                if (tokens.countTokens() < 3) break;
                String prot = tokens.nextToken();
                String fac = tokens.nextToken();
                Double pvalue = Double.valueOf(tokens.nextToken());
                if (!id2fac2p.containsKey(prot))
                {
                    id2fac2p.put(prot, new HashMap<String, Double>());
                }
                id2fac2p.get(prot).put(fac, pvalue);
                //allProts.add(prot);
            }
        } catch (FileNotFoundException e1) {
            e1.printStackTrace();  //To change body of catch statement use File | Settings | File Templates.
        } catch (java.io.IOException e) {
            e.printStackTrace();
        }
        return id2fac2p;
}

    /**
     * Creates a histogram of similarities for all nodes in all networks.
     * @param comp
     * @param nets
     * @return Histogram
     */
    public static Map<Double, Integer> comparerStats(ProteinCompare comp, Collection<CyNetwork> nets) {
        double value = 20;
        Vector<NetNode> nodes = new Vector<NetNode>();
        for (CyNetwork network : nets) {
            for (Iterator<Node> iterator = network.nodesIterator(); iterator.hasNext();) {
                Node node = iterator.next();
                nodes.add(new NetNode(network, node));
            }
        }
        HashMap<Double, Integer> count = new HashMap<Double, Integer>();
        for (int i = 0; i <= value; i++) {
            count.put(i / value, 0);
        }
        for (int i = 0; i < nodes.size(); i++) {
            NetNode n1 = nodes.elementAt(i);
            for (int j = i + 1; j < nodes.size(); j++) {
                NetNode n2 = nodes.elementAt(j);
                double sim = comp.similarity(n1, n2);
                double near = ((int) Math.round(sim * value)) / value;
                if (!count.containsKey(near)) {
                    count.put(near, 0);
                }
                int s = count.get(near);
                count.put(near, s + 1);
            }
        }
        return count;

    }

    public static void printTable(Object table, String text) {
        System.out.println(text + ":");

        if (table instanceof int[][]) {
            for (int[] rows : (int[][]) table) {
                for (int val : rows) {
                    System.out.print(val + " ");
                }
                System.out.println("");
            }
        } else if (table instanceof byte[]) {
            for (byte b : (byte[]) table) {
                System.out.print(b + " ");
            }
        } else if (table instanceof byte[][]) {
            for (byte[] rows : (byte[][]) table) {
                for (byte val : rows) {
                    System.out.print(val + " ");
                }
                System.out.println("");
            }
        } else if (table instanceof double[][]) {
            for (double[] rows : (double[][]) table) {
                for (double val : rows) {
                    System.out.print(val + " ");
                }
                System.out.println("");
            }
        } else if (table instanceof int[]) {
            for (int val : (int[]) table) {
                System.out.print(val + " ");
            }
            System.out.println("");
        } else if (table instanceof double[]) {
            for (double val : (double[]) table) {
                System.out.print(val + " ");
            }
            System.out.println("");
        } else if (table != null) {
            System.out.println("Unprintable object " + table.getClass().toString());
        } else {
            System.out.println("null");
        }
    }


    /* static {
            props = new Properties();

            try {
                InputStream configStream = new FileInputStream("basic.properties");
                props.load(configStream);
                configStream.close();
            } catch (IOException e) {
                // TODO Auto-generated catch block
                e.printStackTrace();
            } catch (Exception e) {
                e.printStackTrace();
            }
        }
    */

    public static GraphDistance getDistance(CyNetwork network) {
        GraphDistance gd = networkDistanceMap.get(network);
        if (gd == null) {
            gd = new GraphDistance(network);
            networkDistanceMap.put(network, gd);
        }
        return gd;
    }

    public static void clearDistanceMemory() {
        networkDistanceMap.clear();
    }

    private static Map<CyNetwork, Map<Node, Map<Node, Edge>>> nodeNode2Edge =
            new HashMap<CyNetwork, Map<Node, Map<Node, Edge>>>();

    /**
     * Finds an edge between two nodes in a given network.
     * @param net
     * @param fromNode
     * @param toNode
     * @return Edge between the given  nodes
     */
    public static Edge getEdge(CyNetwork net, Node fromNode, Node toNode) {
        Map<Node, Map<Node, Edge>> nn2E = nodeNode2Edge.get(net);

        if (nn2E == null) {
            nodeNode2Edge.put(net, new HashMap<Node, Map<Node, Edge>>());
            nn2E = nodeNode2Edge.get(net);

            Iterator<Edge> ei = net.edgesIterator();

            while (ei.hasNext()) {
                Edge edge = ei.next();

                Node s = (edge.getSource().getRootGraphIndex() < edge.getTarget().getRootGraphIndex()) ?
                        edge.getSource() : edge.getTarget();
                Node t = (s == edge.getTarget()) ? edge.getSource() : edge.getTarget();
                // mapping s,t -> edge
                if (nn2E.containsKey(s)) {
                    nn2E.get(s).put(t, edge);
                } else {
                    HashMap<Node, Edge> newMap = new HashMap<Node, Edge>();
                    newMap.put(t, edge);
                    nn2E.put(s, newMap);
                }
            }
        }
        // System.out.println("Toolkit getEdge: " + fromNode.getRootGraphIndex() + " to: " + toNode.getRootGraphIndex());

        if (fromNode.getRootGraphIndex() > toNode.getRootGraphIndex()) {
            return nn2E.get(toNode).get(fromNode);
        } else {
            return nn2E.get(fromNode).get(toNode);
        }
    }

    /**
     * @param perspective
     * @return A map from all nodes to a set of edges connecting the node.
     */
    public static HashMap<Node, Set<Edge>> createEdgeMaps(GraphPerspective perspective) {
        HashMap<Node, Set<Edge>> node2Edges = new HashMap<Node, Set<Edge>>();
        Iterator<Edge> ei = perspective.edgesIterator();
        while (ei.hasNext()) {
            Edge edge = ei.next();
            Node s = edge.getSource();
            Node t = edge.getTarget();

            // node2Edges map
            if (node2Edges.containsKey(s)) {
                node2Edges.get(s).add(edge);
            } else {
                Set<Edge> edges = new HashSet<Edge>();
                edges.add(edge);
                node2Edges.put(s, edges);
            }

            // incoming edges
            /*
            if (node2Edges.containsKey(t)) {
                node2Edges.get(t).add(edge);
            } else {
                Set<Edge> edges = new HashSet<Edge>();
                edges.add(edge);
                node2Edges.put(t, edges);
            }
            */
        }
        return node2Edges;
    }

    /**
     * Compare two nodes neighborhoods.
     * @param node1
     * @param node2
     * @param comp Node comparer
     * @return Score, sum score for every node in the optimal matching.
     */
    public static int compareNeighbours(NetNode node1, NetNode node2,
                                        ProteinCompare comp) {

        Vector<Node> sourceNeighbors = new Vector<Node>(getNodeNeighborMap(node1.net).get(node1.node));
        Vector<Node> targetNeighbors = new Vector<Node>(getNodeNeighborMap(node2.net).get(node2.node));


        int size = Math.max(targetNeighbors.size(), sourceNeighbors.size());
        int[][] weights = new int[size][size];
        for (int y = 0; y < sourceNeighbors.size(); y++) {
            for (int x = 0; x < targetNeighbors.size(); x++) {
                weights[x][y] = (int) (-comp.similarity(new NetNode(null, sourceNeighbors.get(y)),
                        new NetNode(null, targetNeighbors.get(x))) * 10000) + 5000;

            }
            for (int x = targetNeighbors.size(); x < size; x++) {
                weights[x][y] = 0;
            }
        }
        for (int y = sourceNeighbors.size(); y < size; y++) {
            for (int x = 0; x < size; x++) {
                weights[x][y] = 0;
            }
        }
        Csa matching = new Csa(weights);
        int[] cost = matching.getMinCost();
        int sum = 0;
        for (int i = 0; i < cost.length; i++) {
            sum += (-weights[i][cost[i]]);
        }

        System.out.println("From " + node1.node.getIdentifier() + " to node " + node2.node.getIdentifier() +
                " score was " + sum);
        return sum;
    }

    // memo
    public static HashMap<String, Vector<Node>> id2neighs = new HashMap<String, Vector<Node>>();
      public static void clearNeighborMemory() {
        id2neighs.clear();
    }

    /**
     * Find a node's neighbors.
     * @param network
     * @return All nodes connected to the given node in the given network.
     */
    public static Map<Node, Set<Node>> getNodeNeighborMap(
            CyNetwork network) {
        //Set<Edge> motifEdges = new HashSet<Edge>();
        if (networkNeigMap.containsKey(network))
            if (networkNeigMap.get(network) != null)
                return networkNeigMap.get(network);
        Map<Node, Set<Node>> neighborMap = new HashMap<Node, Set<Node>>();
        for (Iterator<Edge> edgeIter = network.edgesIterator(); edgeIter.hasNext();) {

            Edge e = edgeIter.next();
            if (!neighborMap.containsKey(e.getTarget())) {
                neighborMap.put(e.getTarget(), new HashSet<Node>());
            }
            if (!neighborMap.containsKey(e.getSource())) {
                neighborMap.put(e.getSource(), new HashSet<Node>());
            }
            Set<Node> sourceNodeNeihMap = neighborMap.get(e.getSource());
            sourceNodeNeihMap.add(e.getTarget());
            Set<Node> targetNodeNeihMap = neighborMap.get(e.getTarget());
            targetNodeNeihMap.add(e.getSource());
        }
        for (Iterator<Node> nodeIter = network.nodesIterator();
             nodeIter.hasNext();) {
            Node n = nodeIter.next();
            if (!neighborMap.containsKey(n)) neighborMap.put(n, new HashSet<Node>());
        }
        networkNeigMap.put(network, neighborMap);
        return neighborMap;
    }
    public static Collection<Node> getNeighbors(NetNode nodeNet){
        return getNodeNeighborMap(nodeNet.net).get(nodeNet.node);

    }

    public static void clearNeighborSetMemory() {
        networkNeigMap.clear();
    }

    public static String pathNamesToString(String[][] pathNamesArray) {
        StringBuffer pathNames = new StringBuffer();
        for (int i = 0; i < pathNamesArray.length; i++) {
            pathNames.append(i + ": ");
            for (int j = 0; j < pathNamesArray[i].length; j++) {
                pathNames.append(pathNamesArray[i][j] + ",");
            }
            pathNames.append("\n");
        }
        return pathNames.toString();
    }
/*

    public static String getProp(String key) {
        return props.getProperty(key);
    }
*/

    public static DirectedSparseGraph cyToJungGraph(CyNetwork cygraph) {
        DirectedSparseGraph jungGraph = new DirectedSparseGraph();
        HashMap<String, SparseVertex> idToVertexMap = new HashMap<String, SparseVertex>();
        for (Iterator i = cygraph.nodesIterator(); i.hasNext();) {
            CyNode n = (CyNode) i.next();
            SparseVertex v = new SparseVertex();
            //v.addUserDatum("identifier", n.getIdentifier(), new UserDataContainer.CopyAction.Clone() );
            idToVertexMap.put(n.getIdentifier(), v);
            jungGraph.addVertex(v);
        }

        for (Iterator i = cygraph.edgesIterator(); i.hasNext();) {
            CyEdge e = (CyEdge) i.next();
            Node cynode1 = e.getSource();
            Node cynode2 = e.getTarget();
            SparseVertex jungNode1 = idToVertexMap.get(cynode1.getIdentifier());
            SparseVertex jungNode2 = idToVertexMap.get(cynode2.getIdentifier());
            edu.uci.ics.jung.graph.Edge jungEdge;
            if (e.isDirected()) {
                jungEdge = new DirectedSparseEdge(jungNode1, jungNode2);
            } else {
                jungEdge = new UndirectedSparseEdge(jungNode1, jungNode2);
            }

            //jungEdge.addUserDatum("identifier", e.getIdentifier(), new UserDataContainer.CopyAction.Clone() );
            jungGraph.addEdge(jungEdge);
        }

        return jungGraph;
    }

    public static Map<String, List<List<Integer>>> permutationsMemo = new HashMap<String, List<List<Integer>>>();

    /**
     * A list of resultSize-permutation indexes in a setSize-sized set.
     * @param resultSize
     * @param setSize
     * @return All permutations.
     */
    public static List<List<Integer>> getPermutations(int resultSize, int setSize) {
        String id = resultSize + ":" + setSize;
        if (!permutationsMemo.containsKey(id)){
            List<List<Integer>> inclist = new ArrayList<List<Integer>>();
            for (int i = 0; i < setSize; i++) {
                Vector<Integer> v = new Vector<Integer>();
                v.add(i);
                inclist.add(v);
            }

            int lastElmSize = 1;

            while (lastElmSize < resultSize) {
                List<List<Integer>> newList = new ArrayList<List<Integer>>();
                for (List<Integer> currElm : inclist) {
                    for (int i = 0; i < setSize; i++) {
                        if (!currElm.contains(i)) {
                            ArrayList<Integer> newElm = new ArrayList<Integer>(
                                    currElm);
                            newElm.add(i);
                            newList.add(newElm);
                        }
                    }
                }

                inclist = newList;
                lastElmSize++;
            }
            permutationsMemo.put(id, inclist);
        }
        return permutationsMemo.get(id);
    }

    /* unused
    public static List<List<Integer>> getSubsetSize(int subsetsize, int totalsize) {
        if (subsetsize > totalsize || subsetsize < 1) return null;
        List<List<Integer>> list = new ArrayList<List<Integer>>(subsetsize);

        if (subsetsize == 1) {
            for (int i = 0; i < totalsize; i++) {
                List<Integer> l = new ArrayList<Integer>();
                l.add(i);
                list.add(l);
            }
            return list;
        }

        Vector<Integer> newList = new Vector<Integer>();
        for (int i = 0; i < subsetsize; i++) {
            newList.add(-1);
        }

        int j = 0;
        boolean goUp = true;
        boolean goDown = false;
        while (newList.get(0) <= (totalsize - subsetsize)) {
            if (j == 0) {
                newList.set(0, newList.get(0) + 1);
                j++;
                goUp = true;
                goDown = false;
            } else
            if (goUp == false && j == subsetsize - 1 && newList.get(subsetsize - 1) > totalsize - 1)
            {
                j--;
                goUp = false;
                goDown = true;
            } else if (j == subsetsize - 1) {
                if (goUp) {
                    newList.set(j, newList.get(j - 1) + 1);
                    goUp = false;
                } else {
                    list.add(new Vector<Integer>(newList));
                    newList.set(j, newList.get(j) + 1);
                }
            } else if (goUp & !goDown) {
                newList.set(j, newList.get(j - 1) + 1);
                j++;
                goUp = true;
            } else if (!goUp & goDown) {
                if (newList.get(j) + 1 == newList.get(j + 1)) {
                    j--;
                    goDown = true;
                    goUp = false;
                } else {
                    newList.set(j, newList.get(j) + 1);
                    j++;
                    goUp = true;
                    goDown = false;
                }
            } else if ((!goUp && !goDown) || (goUp && goDown)) {
                System.err.println("Error in Toolkit.getSubsetSize(int, int) ");

            }


        }

        return list;

    }
    */

    /**
     * Builds a string representing the network.
     * @param net
     * @return Network presented as string.
     */
    public static String networkToString(CyNetwork net) {
        String name = net.getTitle();
        int lastSlash = -1;
        if (name.contains("/"))
            lastSlash = name.lastIndexOf('/');
        else if (name.contains("\\"))
            lastSlash = name.lastIndexOf("\\");
        name = name.substring(lastSlash + 1, name.length());
        return name;
    }

    /**
     * Opens a save-file dialog and writes the given buffer to the specified file to disk. 
     * @param s Buffer to be written.
     */
    public static void saveToFile(StringBuffer s) {
        JFileChooser fc = new JFileChooser();
        fc.setDialogTitle("Choose file to save");
        fc.setCurrentDirectory(new File("."));
        JFrame saveFrame = new JFrame();
        saveFrame.setDefaultCloseOperation(JFrame.DISPOSE_ON_CLOSE);
        int result = fc.showSaveDialog(saveFrame);
        saveFrame.setVisible(false);
        if (result == JFileChooser.APPROVE_OPTION) {
            File saveto = fc.getSelectedFile();
            try {
                FileWriter out = new FileWriter(saveto);
                out.write(s.toString());
                out.close();
                System.out.println("Saved results to " + saveto);
            } catch (IOException e) {
                System.out.println("Couldn't write to " + saveto);
            }
        }
    }
}
