package nemi.apriori;

import cytoscape.CyNetwork;
import giny.model.Node;
import nemi.cluster.Cluster;

import java.util.*;

/**
 * Created by IntelliJ IDEA. User: kristost Date: 15.feb.2006 Time: 15:50:51 To
 * change this template use File | Settings | File Templates.
 *
 * An Apriori subgraph-mining implementation, built on Complete Mining of Frequent Patterns from Graphs:
 * Mining Graph Dataon  by Akihiro Inokuchi and Takashi Washio and Hiroshi Motoda.
 */
public class Apriori {
    private boolean DEBUG = false;

    private boolean running = false;

    int level = 0;

    int numSubs = 0;
    int lastLevelCount = 0;
    Vector<AprioriInstance> graphs = new Vector<AprioriInstance>();
    private Collection<Cluster> clusters;
    private CyNetwork network;
    public Apriori(Collection<Cluster> clusters, CyNetwork network, int overlapcount) {
        this.overlapCount = overlapcount;


        this.clusters = clusters;
        this.network = network;


    }

    public boolean isrunning() {
        return running;
    }


    /**
     * Stop all calculations.
     */
    public void stopRunning() {
        running = false;
    }

    /**
     * @return Returns the current search depth.
     */
    public int getLevel() {
        return level;
    }

    /**
     * @return The number of motifs found.
     */
    public int getMotifFoundSoFar() {
        return numSubs;
    }

    /**
     * @return The number of motifs found during the last level.
     */
    public int getMotifsPrevious() {
        return lastLevelCount;
    }

    /**
     * Start searching for motifs.
     * @param maxSize Maximal motif size to be returned.
     * @param suppTresh Minimal support for the motifs returned.
     * @return A map from the motif to a set of instances.
     */
    public Map<GraphMotif, Vector<AprioriInstance>> apply(int maxSize, int suppTresh) {
        numSubs = 0;
        running = true;
        int i = 0;
        System.out.println(graphs.size() + ": " + graphs);
        for (Cluster cluster : clusters) {
            if (cluster.size()<suppTresh) continue;
            for (Iterator<Node> it = cluster.getRepNodes().iterator(); it.hasNext();) {
                i++;
                Node n = it.next();
                AprioriInstance sg = new AprioriInstance(n, cluster.id, network);
                graphs.add(sg);
            }
        }
        System.out.println("Nodecount " + i);
        return startMine(graphs, suppTresh, maxSize);
    }

    /**
     * Start mining.
     * @param startSubGraphs
     * @param supportThresh
     * @param graphSize
     * @return A map from all motifs found to their corresponding instances.
     */
    private Map<GraphMotif, Vector<AprioriInstance>> startMine(Vector<AprioriInstance> startSubGraphs, int supportThresh, int graphSize) {
        if (startSubGraphs.isEmpty()) {
            return null;
        }

        // Container for frequent subgraphs
        HashMap<GraphMotif, Vector<AprioriInstance>> listOfFrequents =
                new HashMap<GraphMotif, Vector<AprioriInstance>>();
        //make sure the startSubGraphs are sorted
        Collections.sort(startSubGraphs);
        //newcands store result
        Vector<AprioriInstance> newCands = new Vector<AprioriInstance>();

        level = 1;

        for (int i = 0; i < startSubGraphs.size(); i++) {
            AprioriInstance c1 = startSubGraphs.get(i);
            for (int j = i + 1; j < startSubGraphs.size(); j++) {
                if (!running)
                    return null;
                // Join all two startSubGraphs in lexicographical order
                AprioriInstance c2 = startSubGraphs.get(j);
                AprioriInstance newC = c1.join(c2, graphSize);
                if (newC != null) {
                    GraphMotif m = SubGraphView.findNormalForm(newC);
                    //make sure thie new combindes startSubGraph is in frequent list
                    Vector<AprioriInstance> cands = listOfFrequents.get(m);
                    if (cands == null) {
                        cands = new Vector<AprioriInstance>();
                        listOfFrequents.put(m, cands);

                    }
                    if (m.labels.charAt(0) == m.labels.charAt(1)) {
                        AprioriInstance newC2 = c2.join(c1, graphSize);
                        if (newC2 != null) cands.add(newC2);
                    }
                    cands.add(newC);

                }
            }
        }
        // prune new subgraph instances for support
        for (Iterator<Map.Entry<GraphMotif, Vector<AprioriInstance>>> i = listOfFrequents.entrySet().iterator(); i.hasNext();)
        {
            Map.Entry<GraphMotif, Vector<AprioriInstance>> e = i.next();
            if (e.getValue().size() < supportThresh) {
                i.remove();
            }
        }
        removeNonUnique(listOfFrequents, supportThresh);   //no function
        for (Map.Entry<GraphMotif, Vector<AprioriInstance>> entry : listOfFrequents.entrySet()) {
            newCands.addAll(entry.getValue());
        }
        List<GraphMotif> motifs = new Vector<GraphMotif>(listOfFrequents.keySet());
        Collections.sort(motifs);

        // generate size 3 motifs
        /* Set<GraphMotif> newMotifSupport = new HashSet<GraphMotif>();
        for (int i = 0; i < motifs.size(); i++) {
            GraphMotif m1 = motifs.get(i);
            for (int j = i + 1; j < motifs.size(); j++) {
                GraphMotif m2 = motifs.get(j);
                GraphMotif[] newMotifs = m1.join(m2);
                if (newMotifs != null) {
                    for (GraphMotif graphMotif : newMotifs) {
                        newMotifSupport.add(SubGraphView.makeCanonical(
                                graphMotif, new HashSet<GraphMotifHelper>()));
                    }
                }
            }
        }*/

        //open for garbage collection
        motifs.clear();
        motifs = null;
        System.gc();
        //start mining
        Map<GraphMotif, Vector<AprioriInstance>> res = null;
        // if (graphSize > 2 && newMotifSupport.size() > 0) {
        if (graphSize > 2) {
            res = mine(newCands, supportThresh, graphSize);
        }
        if (res != null && res.size() > 0) {

            removeUninteresting(res, supportThresh);
            if (res.size() > 0) return res;
        }
        removeUninteresting(listOfFrequents, supportThresh);
        System.out.println("");

        return listOfFrequents;
    }

    private Map<GraphMotif, Vector<AprioriInstance>> mine
            (Vector<AprioriInstance> startSubGraphs, int supportThresh,
             int graphsize) {
        //are there any candidates to start mining?
        if (startSubGraphs.isEmpty()) {
            return null;
        }

        HashMap<GraphMotif, Vector<AprioriInstance>> listOfFrequents =
                new HashMap<GraphMotif, Vector<AprioriInstance>>(); //container for frequent subgraphs
        Collections.sort(startSubGraphs); //make sure the subgraphs are sorted
        Map<GraphMotif, Vector<AprioriInstance>> resultMap =
                new HashMap<GraphMotif, Vector<AprioriInstance>>(); //container for frequent subgraphs

        level = startSubGraphs.get(0).nodeCount(); // update for gui to read
        //normal form -> canonical map
        HashMap<GraphMotif, GraphMotif> toCanonical = new HashMap<GraphMotif, GraphMotif>();
        // the labels excluding the last:
        String leftStartsWith = startSubGraphs.get(0).getLabels();
        String rightStartsWith = leftStartsWith.substring(0, leftStartsWith.length() - 1);
        int rightLimit = -1;
        int leftLimit = startSubGraphs.size();
        for (int k = 0; k < startSubGraphs.size(); k++) {
            if (!startSubGraphs.get(k).getLabels().startsWith(leftStartsWith)) {
                leftLimit = k;
                break;
            }
        }
        boolean foundRightLimit = false;
        int prunecount = 0;
        for (int i = 0; i < startSubGraphs.size() + 1; i++) {
            // check wether the second to last label is the working label
            // start DFS hybrid (or if we're at the widthBreakoff of this startsubgraphlist
            if (i == leftLimit) {
                // START DFS HYBRID
                int currentSize = startSubGraphs.get(0).nodeCount();
                System.out.print("\rGoing depth with labels " +
                        "(" + leftStartsWith + "*). Progress: " + i + " / "
                        + startSubGraphs.size() + "                ");//+ " pruned: " + prunecount);

                System.out.flush();
                if (!running) {
                    return null;
                }
                // Prune for support
                Vector<AprioriInstance> tempCands = new Vector<AprioriInstance>();
                for (Iterator<Map.Entry<GraphMotif, Vector<AprioriInstance>>> entryIter =
                        listOfFrequents.entrySet().iterator(); entryIter.hasNext();) {
                    Map.Entry<GraphMotif, Vector<AprioriInstance>> entry = entryIter.next();
                    if (entry.getValue().size() < supportThresh) {
                        entryIter.remove();
                    }
                }
                removeNonUnique(listOfFrequents, supportThresh);
                //Vector<AprioriInstance> newCands = new Vector<AprioriInstance>(); //result vector
                for (Map.Entry<GraphMotif, Vector<AprioriInstance>> entry : listOfFrequents.entrySet()) {
                    tempCands.addAll(entry.getValue());
                }
                toCanonical.clear();    // we do not need these normal->canonical mapping

                //are we on the last level ?
                if (startSubGraphs.get(0).nodeCount() + 1 >= graphsize) {
                    removeUninteresting(listOfFrequents, supportThresh);
                    resultMap.putAll(listOfFrequents);


                } else {
                    // generate new graph motif candidates
                    //Set<GraphMotif> motifCandidates = new HashSet<GraphMotif>();
                    // Vector<GraphMotif> confirmedMotifs = new Vector<GraphMotif>(listOfFrequents.keySet());
                    /*Collections.sort(confirmedMotifs);
                    for (int k = 0; k < confirmedMotifs.size(); k++) {
                        GraphMotif m1 = confirmedMotifs.get(k);
                        for (int l = k + 1; l < confirmedMotifs.size(); l++) {
                            GraphMotif m2 = confirmedMotifs.get(l);
                            GraphMotif[] newMotifs = m1.join(m2);
                            if (newMotifs != null) {
                                for (GraphMotif graphMotif : newMotifs) {
                                    motifCandidates.add(SubGraphView.makeCanonical(graphMotif,
                                            new HashSet<GraphMotifHelper>()));
                                }
                            }

                        }
                    }*/

                    Map<GraphMotif, Vector<AprioriInstance>> res = mine(tempCands, supportThresh, graphsize);

                    if (res != null && res.size() > 0) {

                        if (resultMap.size() > 0) {
                            GraphMotif resgm = res.entrySet().iterator().next().getKey();
                            GraphMotif already = resultMap.entrySet().iterator().next().getKey();
                            // we have some new interesting candidates!
                            if (already.nodeCount() < resgm.nodeCount()) {
                                resultMap.clear();
                            }
                        }
                        resultMap.putAll(res);
                    } else {
                        if (resultMap.size() > 0 && listOfFrequents.size() > 0) {
                            GraphMotif freqgm = listOfFrequents.entrySet().iterator().next().getKey();
                            GraphMotif already = resultMap.entrySet().iterator().next().getKey();
                            if (already.nodeCount() <= freqgm.nodeCount()) {
                                removeUninteresting(listOfFrequents, supportThresh);
                                resultMap.putAll(listOfFrequents);
                            }
                        } else {
                            removeUninteresting(listOfFrequents, supportThresh);
                            resultMap.putAll(listOfFrequents);

                        }
                        //make sure newCands aren't empty
                    }

                }
                listOfFrequents.clear(); // frequents are not interesting any more
                if (i == startSubGraphs.size()) break;
                leftLimit = startSubGraphs.size();
                leftStartsWith = startSubGraphs.get(i).getLabels();
                for (int k = i + 1; k < startSubGraphs.size(); k++) {
                    if (!startSubGraphs.get(k).getLabels().startsWith(leftStartsWith)) {
                        leftLimit = k;
                        break;
                    }
                }
            }
            AprioriInstance c1 = startSubGraphs.get(i);
            int j = i;
            if (foundRightLimit == false && rightLimit == startSubGraphs.size()) {
                foundRightLimit = true;  // this is just the case for the last round
            }
            if (rightLimit <= i) {
                rightLimit = startSubGraphs.size();
                foundRightLimit = false;
                rightStartsWith = leftStartsWith.substring(0, startSubGraphs.get(0).nodeCount() - 1);
            }
            while (++j < rightLimit) {
                if (!foundRightLimit) {
                    String startsWith2 = startSubGraphs.get(j).getLabels().substring(0, startSubGraphs.get(0).nodeCount() - 1);
                    if (!startsWith2.equals(rightStartsWith)) {
                        rightLimit = j;
                        foundRightLimit = true;
                        break;
                    }
                }
                //start joining to get subgraph candidates for k+1
                AprioriInstance c2 = startSubGraphs.get(j);
                AprioriInstance newC = c1.join(c2, graphsize);
                if (newC != null) {
                    GraphMotif normal = GraphMotif.findNormalForm(newC); //fast find normal form
                    if (!toCanonical.containsKey(normal)) {
                        //make sure we have normal-> canonical mapping
                        HashSet<GraphMotifHelper> normals = new HashSet<GraphMotifHelper>();
                        GraphMotif canonical = SubGraphView.makeCanonical(newC, normals);
                        for (GraphMotif m : normals) {
                            toCanonical.put(m, canonical);
                        }
                    }
                    GraphMotif canonical = toCanonical.get(normal);
                    //if (candidateMotifs.contains(canonical)) {
                    //only count candiate motifs
                    if (!listOfFrequents.containsKey(canonical)) {
                        listOfFrequents.put(canonical, new Vector<AprioriInstance>());
                    }
                    Vector<AprioriInstance> frequents = listOfFrequents.get(canonical);
                    frequents.add(newC);
                    //} else {
                    //    prunecount++;
                    //System.out.println("EXCITING! Not a motif!");
                    //}
                }
            }
        }
        System.gc(); //TEST: can the garbagecollector do any good on stack space?
        return resultMap;
    }


    public static void main
            (String[] args) {

        new Apriori(0);
    }

    public void printTable
            (Object
                    table, String
                    text) {
        out(text + ":");

        if (table instanceof int[][]) {
            for (int[] rows : (int[][]) table) {
                for (int val : rows) {
                    outc(val + " ");
                }
                out("");
            }
        } else if (table instanceof byte[]) {
            for (byte b : (byte[]) table) {
                outc(b + " ");
            }
        } else if (table instanceof double[][]) {
            for (double[] rows : (double[][]) table) {
                for (double val : rows) {
                    outc(val + " ");
                }
                out("");
            }
        } else if (table instanceof int[]) {
            for (int val : (int[]) table) {
                outc(val + " ");
            }
            out("");
        } else if (table instanceof double[]) {
            for (double val : (double[]) table) {
                outc(val + " ");
            }
            out("");
        } else if (table != null) {
            out("Unprintable object " + table.getClass().toString());
        } else {
            out("null");
        }
    }

    void out
            (Object
                    o) {
        if (DEBUG)
            System.out.println(o);
    }

    private void outc
            (Object
                    o) {
        if (DEBUG)
            System.out.print(o);
    }
    int overlapCount = 0;
    public Apriori(int overlapcount) {
        this.overlapCount = overlapcount;
        Vector<Integer> pointers = new Vector<Integer>();
        pointers.add(0);
        pointers.add(1);
        pointers.add(2);
        pointers.add(3);

        AprioriGraph transaction = new AprioriGraph("110",
                new AprioriGraphLabels("AAA", pointers), false);

        System.out.println(transaction.getCanonicalForm());

    }

    /**
     * Remove unconnected motifs.
     * @param res
     * @param supplimit
     */
    private void removeUninteresting(Map<GraphMotif, Vector<AprioriInstance>> res, int supplimit) {
        for (Iterator<Map.Entry<GraphMotif, Vector<AprioriInstance>>> resultIter = res.entrySet().iterator(); resultIter.hasNext();)
        {

            Map.Entry<GraphMotif, Vector<AprioriInstance>> entry = resultIter.next();
            if(!entry.getKey().isConnected()){
                resultIter.remove();
            }
            /*for (Iterator<SubGraph2> iter = entry.getValue().iterator(); iter.hasNext();) {
                SubGraph2 sg = iter.next();

                if (!sg.isConnected()) {
                    iter.remove();
                }
            }
            if (entry.getValue().size() < supplimit) {
                resultIter.remove();
            } */
        }
        removeNonUnique(res, supplimit);
    }

    public void removeNonUnique
            (Map<GraphMotif, Vector<AprioriInstance>> frequentset, int supportLimit) {
        for (Iterator<Map.Entry<GraphMotif, Vector<AprioriInstance>>> frequentSetIterator = frequentset.entrySet().iterator(); frequentSetIterator.hasNext();)
        {
            Map.Entry<GraphMotif, Vector<AprioriInstance>> frequentEntry = frequentSetIterator.next();
            Set<Node> seen = new HashSet<Node>();
            int uniqueCount = 0;
            for (AprioriInstance sg : frequentEntry.getValue()) {
                int newNodes = 0;

                for (Node n : sg.subGraph) {
                    if (!seen.contains(n)) {
                        newNodes++;
                    }
                }
                if (newNodes >= (sg.nodeCount() -overlapCount)) {
                    uniqueCount++;
                    seen.addAll(sg.subGraph);
                }
            }
            if (uniqueCount < supportLimit) frequentSetIterator.remove();
        }
    }
}
