package nemi.apriori;

import cytoscape.CyNetwork;
import giny.model.Node;
import nemi.Toolkit;

import java.util.*;

/**
 * Apriori subgraph instance imlementation.
 */
public class AprioriInstance extends SubGraphView {
    public Vector<Node> subGraph;
    GraphDistance distmap;
    public String labels;
    public CyNetwork parentNet;

    /**
     * Initialize a single node instance with label 'id' in network.
     * @param n
     * @param id
     * @param network
     */
    public AprioriInstance(Node n, char id, CyNetwork network) {
        subGraph = new Vector<Node>();
        subGraph.add(n);
        labels = String.valueOf(id);
        distmap = Toolkit.getDistance(network);
        parentNet = network;
    }

    private AprioriInstance() {

    }

    public boolean getEdge(int i, int j) {
        Node n1 = subGraph.get(i);
        Node n2 = subGraph.get(j);
        return distmap.hasEdge(n1, n2);
    }

    public String getLabels() {
        return labels;
    }

    public String getLabelOf(int index) {
        return labels.substring(index, index + 1);
    }

    public int nodeCount() {
        return subGraph.size();
    }

    @Override
    public String toString() {
        return this.getLabels() + this.getStringCode();
    }

    public void setEdge(int i, int j, boolean value) {

    }


    public boolean isCorrectLength() {
        return subGraph.size() == labels.length();
    }

    public boolean isSorted() {
        char temp = labels.charAt(0);
        char temp2 = 0;
        for (int i = 1; i < labels.length(); i++) {
            temp2 = labels.charAt(i);
            if (temp > temp2) return false;
            temp = temp2;
        }
        return true;
    }

    public int compareTo(SubGraphView sg) {

        int val = this.nodeCount() - sg.nodeCount();
        if (val != 0) return val;
        val = this.getLabels().compareTo(sg.getLabels());
        if (val != 0) return val;
        return this.getStringCode().compareTo(sg.getStringCode());

    }

    public Node getNode(int i) {
        return subGraph.get(i);
    }

    public AprioriInstance join(AprioriInstance c2, int graphSize) {
        if (this.nodeCount() != c2.nodeCount()) {
            //System.out.println("Wrong nodecount");
            return null;
        }
        for (int i = 0; i < this.nodeCount() - 1; i++) {
            if (!subGraph.get(i).equals(c2.getNode(i))) {
                //	System.out.println("nodes doesn't match");
                return null;
            }
        }
        String l1 = this.getLabelOf(this.nodeCount() - 1);
        String l2 = c2.getLabelOf(this.nodeCount() - 1);
        if (l1.compareTo(l2) > 0) {
            //System.out.println("Label too low");
            return null;
        }
        Node othernode = c2.getNode(c2.nodeCount() - 1);
        for (Node n : this.subGraph) {
            double d = distmap.distance(n, othernode) ;
            if (d>= graphSize || d<=0) {
                //System.out.println("Graph distance too high");
                return null;
            }
        }
        AprioriInstance s2 = new AprioriInstance();
        s2.labels = new String(this.getLabels());
        s2.labels += l2;
        s2.distmap = this.distmap;
        s2.parentNet = this.parentNet;
        s2.subGraph = new Vector<Node>(subGraph);
        s2.subGraph.add(c2.getNode(this.nodeCount() - 1));

        return s2;
    }

    /**
     * @return Whether this motif is connected.
     */
    public boolean isConnected() {
        if (subGraph.size() <= 1) return true;
        //HashSet<Node> nodes = new HashSet<Node>(subGraph);
        HashSet<Node> seen = new HashSet<Node>(subGraph.size());
        LinkedList<Node> que = new LinkedList<Node>();
        que.offer(subGraph.get(0));
        seen.add(que.peek());
        Map<Node, Set<Node>> neighmap = Toolkit.getNodeNeighborMap(parentNet);
        while (que.size() > 0) {
            Node node = que.poll();
            for (Node n : neighmap.get(node)) {
                if (!subGraph.contains(n)) continue;
                if (!seen.contains(n)) {
                    seen.add(n);
                    que.offer(n);
                }
            }
        }
        return seen.size() == subGraph.size();
    }

    @Override
    public String getStringCode() {
        StringBuilder sb = new StringBuilder();
        for (int i = 0; i < labels.length(); i++) {
            for (int j = i + 1; j < labels.length(); j++) {
                Node n1 = subGraph.get(i);
                Node n2 = subGraph.get(j);

                boolean b = distmap.hasEdge(n1, n2);
                sb.append(b ? "1" : "0");
            }
        }
        return sb.toString();
    }

    public int hashCode() {
        return subGraph.hashCode() + getLabels().hashCode();
    }

    public boolean equals(Object o) {
        if (o instanceof AprioriInstance) {
            AprioriInstance aprioriInstance = (AprioriInstance) o;
            return aprioriInstance.subGraph.equals(this.subGraph) && this.getLabels().equals(aprioriInstance.getLabels());
        } else {
            throw new ClassCastException();
        }

    }

    public void flagNodes() {
        
        this.parentNet.setFlaggedNodes(subGraph, true);
    }


}
