package nemi.apriori;

import cytoscape.CyNetwork;
import giny.model.Node;
import nemi.cluster.*;
import nemi.compare.ProteinCompare;
import nemi.compare.gui.ProteinComparerTable;
import nemi.compare.gui.ProteinTableModel;
import nemi.cytoscape.NetworkList;
import nemi.gui.*;

import javax.swing.*;
import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.text.DecimalFormat;
import java.util.*;

/**
 * Created by IntelliJ IDEA.
 * User: Administrator
 * Date: 02.mai.2006
 * Time: 19:04:39
 * GUI for the Apriori graphmining implementation. Input configuration at the top, results are
 * presented in the bottom. 
 */
public class AprioriPanel extends JSplitPane {
    private GridBagConstraints gridBagConstraints;
    private JLabel networkLabel = new JLabel();
    private JPanel springPanel = new JPanel();
    private JLabel supportLabel;
    private JSpinner supportSpinner;
    //ClusterNodeDistancePanel clusterNodeDistPanel = new ClusterNodeDistancePanel();
    //JLabel clusterNodeDistLabel = new JLabel();
    private NetworkList netListModel;
    private ClustererList clusterListModel;
    JPanel miningControlPanel = new JPanel();
    private boolean running;
    private Apriori apriori;
    private int clusterCount;
    private NeMiFinalView nemiView;
    private JButton statisticsButton;
    private Vector<CyNetwork> oldNetList;
    private ProteinCompare oldComp;
    private double oldLimit;
    private Collection<Cluster> oldClusters;
    private JLabel clusteringInfoLabel = new JLabel();
    private JCheckBox joinSingleCluseters = new JCheckBox();
    private Map<GraphMotif, Vector<AprioriInstance>> oldresults;
    private JButton saveButton;
    private ViewableResults oldresulttablemodel;

    public AprioriPanel(NeMiFinalView nemiView) {
        super();
        this.nemiView = nemiView;

        miningControlPanel.setLayout(new java.awt.BorderLayout());


        initComponents();
        this.setBorder(null);
        this.setOrientation(javax.swing.JSplitPane.VERTICAL_SPLIT);
        this.setTopComponent(miningSetupScroller);
        this.setBottomComponent(miningResultScroller);


    }

    void initComponents() {
        netListModel = new NetworkList();
        clusterListModel = new ClustererList();
        miningControlPanel = new JPanel();
        miningPanel = new javax.swing.JPanel();

        //miningSplit = new javax.swing.JSplitPane();
        miningSetupScroller = new javax.swing.JScrollPane();
        //miningSetupPanel = new javax.swing.JPanel();
        networkLabel = new javax.swing.JLabel();
        limitLabel = new javax.swing.JLabel();
        clusteringList = new javax.swing.JComboBox();
        springPanel = new javax.swing.JPanel();
        proteinComparerLabel = new javax.swing.JLabel();
        choseClustererLabel = new javax.swing.JLabel();
        networkchoser = new javax.swing.JComboBox();
        buttonPanel = new javax.swing.JPanel();
        cancelButton = new javax.swing.JButton();
        startButton = new javax.swing.JButton();
        selectMotifSizeLabel = new javax.swing.JLabel();
        maxMotifSizeSpinner = new javax.swing.JSpinner();
        supportSpinner = new JSpinner();
        supportLabel = new JLabel();

        clusteringLimitSlider = new SliderWrapper(0, 100, 100, true);

        proteinComparerPanel = new javax.swing.JPanel();
        proteinComparerTableScroller = new javax.swing.JScrollPane();
        proteinComparerTable = new ProteinComparerTable();//new javax.swing.JTable();

        miningResultPanel = new javax.swing.JPanel();
        miningResultScroller = new javax.swing.JScrollPane();
        //miningResultTextArea = new javax.swing.JTextArea();

        miningControlPanel.setLayout(new java.awt.GridBagLayout());

        miningControlPanel.setAutoscrolls(true);
        networkLabel.setText("Network");
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 0;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
        miningControlPanel.add(networkLabel, gridBagConstraints);
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 1;
        gridBagConstraints.gridy = 0;
        gridBagConstraints.fill = java.awt.GridBagConstraints.BOTH;
        gridBagConstraints.ipadx = 7;
        miningControlPanel.add(springPanel, gridBagConstraints);

        networkchoser.setModel(new javax.swing.DefaultComboBoxModel(
                new String[]{"Network 1", "Network 2"}));
        networkchoser.setMinimumSize(new java.awt.Dimension(100, 20));
        networkchoser.setVerifyInputWhenFocusTarget(false);
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 2;
        gridBagConstraints.gridy = 0;
        //gridBagConstraints.gridwidth = 3;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.NORTHWEST;
        gridBagConstraints.insets = new java.awt.Insets(1, 0, 1, 0);
        miningControlPanel.add(networkchoser, gridBagConstraints);

        proteinComparerLabel.setText("Protein comparer(s)");
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 1;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
        miningControlPanel.add(proteinComparerLabel, gridBagConstraints);

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 2;
        gridBagConstraints.gridy = 1;
        gridBagConstraints.gridwidth = 3;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
        gridBagConstraints.insets = new java.awt.Insets(1, 0, 1, 0);
        miningControlPanel.add(proteinComparerPanel, gridBagConstraints);

        choseClustererLabel.setText("Clustering method");
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 2;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
        miningControlPanel.add(choseClustererLabel, gridBagConstraints);

        clusteringList
                .setModel(new javax.swing.DefaultComboBoxModel(new String[]{
                        "Iterative Clusterer", "Transcription factor clusterer"}));
        clusteringList.setEnabled(true);
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 2;
        gridBagConstraints.gridy = 2;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.NORTHWEST;
        gridBagConstraints.insets = new java.awt.Insets(1, 0, 1, 0);
        miningControlPanel.add(clusteringList, gridBagConstraints);

        joinSingleCluseters.setText("Join single clusters");
        joinSingleCluseters.setSelected(true);
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 3;
        gridBagConstraints.gridy = 2;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
        miningControlPanel.add(joinSingleCluseters, gridBagConstraints);


        limitLabel.setText("Clustering tolerance");
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 3;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
        miningControlPanel.add(limitLabel, gridBagConstraints);
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 2;
        gridBagConstraints.gridy = 3;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
        miningControlPanel.add(clusteringLimitSlider, gridBagConstraints);

        /*
        gridBagConstraints = new GridBagConstraints();
        gridBagConstraints.gridy = 4;
        gridBagConstraints.gridx = 0;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
        clusterNodeDistLabel.setText("Diameter type");
        miningControlPanel.add(clusterNodeDistLabel, gridBagConstraints);
        */
        /*gridBagConstraints = new GridBagConstraints();
        gridBagConstraints.gridy = 4;
        gridBagConstraints.gridx = 2;
        miningControlPanel.add(clusterNodeDistPanel, gridBagConstraints);
          */
        selectMotifSizeLabel.setText("Max motifsize");
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 5;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
        miningControlPanel.add(selectMotifSizeLabel, gridBagConstraints);

        maxMotifSizeSpinner.setAutoscrolls(true);
        maxMotifSizeSpinner.setInheritsPopupMenu(true);
        maxMotifSizeSpinner.setName("max motifsize");
        maxMotifSizeSpinner.setPreferredSize(new java.awt.Dimension(40, 20));
        maxMotifSizeSpinner.setValue(4);
        SpinnerNumberModel maxMotifmodel = new SpinnerNumberModel(2, 2, 100, 1);
        maxMotifSizeSpinner.setModel(maxMotifmodel);
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 2;
        gridBagConstraints.gridy = 5;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
        gridBagConstraints.insets = new java.awt.Insets(1, 0, 1, 0);
        miningControlPanel.add(maxMotifSizeSpinner, gridBagConstraints);


        supportLabel.setText("Support threshold");
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 3;
        gridBagConstraints.gridy = 5;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
        miningControlPanel.add(supportLabel, gridBagConstraints);

        supportSpinner.setAutoscrolls(true);
        supportSpinner.setPreferredSize(new java.awt.Dimension(40, 20));
        supportSpinner.setValue(2);
        SpinnerNumberModel supportModel = new SpinnerNumberModel(2, 2, 100, 1);
        supportSpinner.setModel(supportModel);
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 4;
        gridBagConstraints.gridy = 5;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
        miningControlPanel.add(supportSpinner, gridBagConstraints);

        buttonPanel
                .setLayout(new java.awt.FlowLayout(java.awt.FlowLayout.RIGHT));

        cancelButton.setText("Cancel");
        cancelButton.setEnabled(false);
        cancelButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                cancelButtonActionPerformed(evt);
            }
        });
        saveButton = new JButton("Save results");
        saveButton.setEnabled(false);
        saveButton.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                if (oldresults != null) {
                    StringBuffer sb = new StringBuffer();
                    sb.append("######## Apriori results ########\n");

                    for (Map.Entry<GraphMotif, Vector<AprioriInstance>> entry : oldresults.entrySet())
                    {
                        sb.append("Motif: " + entry.getKey().toString() +"\n");

                        int i = 0;
                        for (AprioriInstance sg : entry.getValue()) {
                            sb.append("Instance " + i++ + ": ");
                            for (Node n : sg.subGraph) {
                                sb.append(n.getIdentifier() + ", ");
                            }
                            sb.delete(sb.length()-2, sb.length()-1);
                            sb.append("\n");
                        }
                        sb.append("\n");

                    }
                    sb.append("\n");
                    sb.append("####### Clustering info ########\n");

                    for(Cluster c: oldClusters){
                        sb.append("Cluster " + c.id + ":");

                            for (Node n : c.getRepNodes()) {
                                sb.append(n.getIdentifier() + ", ");
                            }
                            sb.delete(sb.length()-2, sb.length()-1);
                       sb.append("\n");

                    }
                    nemi.Toolkit.saveToFile(sb);
                }
            }
        });


        startButton.setText("Start");
        startButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                startButtonActionPerformed(evt);
            }
        });

        buttonPanel.add(startButton);
        buttonPanel.add(cancelButton);
        buttonPanel.add(saveButton);

        statisticsButton = new JButton("Statsistics");
        statisticsButton.setEnabled(false);
        statisticsButton.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                statButtonPressed(e);
            }
        });
        buttonPanel.add(statisticsButton);
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 7;
        gridBagConstraints.gridwidth = 3;
        gridBagConstraints.fill = java.awt.GridBagConstraints.BOTH;
        gridBagConstraints.insets = new java.awt.Insets(1, 0, 1, 0);
        miningControlPanel.add(buttonPanel, gridBagConstraints);


        proteinComparerPanel.setMinimumSize(new java.awt.Dimension(168, 80));
        proteinComparerTableScroller
                .setBorder(new javax.swing.border.EtchedBorder());
        proteinComparerTableScroller.setPreferredSize(new java.awt.Dimension(
                400, 100));
        proteinComparerTable.setBorder(new javax.swing.border.EtchedBorder());
//		proteinComparerTable.setModel(new javax.swing.table.DefaultTableModel(
//				new Object[][] { { "Gene Ontology distance", "etc", null } },
//				new String[] { "Comparator", "Properties", "Weight" }) {
//			boolean[] canEdit = new boolean[] { false, false, false };
//
//			public boolean isCellEditable(int rowIndex, int columnIndex) {
//				return canEdit[columnIndex];
//			}
//		});
        proteinComparerTable.setMinimumSize(new java.awt.Dimension(100, 16));
        // proteinComparerTable.setTableHeader(null);
        /*proteinComparerTable
        .addMouseListener(new java.awt.event.MouseAdapter() {
            public void mouseClicked(java.awt.event.MouseEvent evt) {
                proteinComparerTableMouseClicked(evt);
            }
        });
        */
        //proteinComparerTable.setTableHeader()

        proteinComparerTableScroller.setViewportView(proteinComparerTable);

        proteinComparerPanel.add(proteinComparerTableScroller);

        /*addProteinComparerButton.setText("Add");
                addProteinComparerButton
                        .addActionListener(new java.awt.event.ActionListener() {
                            public void goButtonPressed(java.awt.event.ActionEvent evt) {
                                addProteinComparerButtonActionPerformed(evt);
                            }
                        });

                proteinComparerPanel.add(addProteinComparerButton);
        */

        clusteringList.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                clusteringListActionPerformed(evt);
            }
        });
        proteinComparerTable.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                startButton.setEnabled(needsProtComp ? proteinComparerTable.isReady() : true);
            }
        });


        miningSetupScroller.setViewportView(miningControlPanel);

        //miningSplit.setTopComponent(miningSetupScroller);

        miningResultPanel.setLayout(new java.awt.BorderLayout());
        resultTable = new JTable();
        //miningResultScroller.setViewportView(miningResultTextArea);
        miningResultScroller.setViewportView(resultTable);
        miningResultPanel.add(miningResultScroller,
                java.awt.BorderLayout.CENTER);

        //miningSplit.setBottomComponent(miningResultPanel);
        this.setDividerLocation(250);
        //miningPanel.add(miningSplit, java.awt.BorderLayout.CENTER);
        networkchoser.setModel(netListModel);
        clusteringList.setModel(clusterListModel);
        startButton.setEnabled(needsProtComp ? proteinComparerTable.isReady() : true);
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 2;
        gridBagConstraints.gridy = 8;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
        miningControlPanel.add(clusteringInfoLabel, gridBagConstraints);
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 8;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
        miningControlPanel.add(new JLabel("Clustering info; "), gridBagConstraints);
    }

    boolean needsProtComp = true;


    private void clusteringListActionPerformed(java.awt.event.ActionEvent evt) {
        if (clusterListModel.getSelectedItem().equals(ClustererList.itClust)) {
            proteinComparerTable.setEnabled(true);
            clusteringLimitSlider.setEnabled(true);
            startButton.setEnabled(proteinComparerTable.isReady());
            needsProtComp = true;
        } else if (clusterListModel.getSelectedItem().equals(ClustererList.tfCluster)) {
            proteinComparerTable.setEnabled(false);
            clusteringLimitSlider.setEnabled(false);
            startButton.setEnabled(true);
            needsProtComp = false;
        }
    }

    private void startButtonActionPerformed(java.awt.event.ActionEvent evt) {

        cancelButton.setEnabled(true);
        running = true;
        startButton.setEnabled(false);
        //statusLabel.setText("Status: Starting motif discovery");
        //statusLabel.repaint();
        StringBuffer sb = new StringBuffer();
        sb.append("Selected network: ");
        sb.append(netListModel.getSelectedItem());
        sb.append("\n");
        sb.append("Clustering method: ");
        sb.append(clusterListModel.getSelectedItem());
        sb.append("\n");
        sb.append("Clustering limit: ");
        sb.append(clusteringLimitSlider.getValue());
        sb.append("\n");
        sb.append("Max motifsize ");
        sb.append(maxMotifSizeSpinner.getValue());
        System.out.println(sb);
        Thread runThread = new Thread(new Runnable() {
            public void run() {
                //statusProgressbar.setString("");
                //statusProgressbar.setStringPainted(true);
                apriori = null;
                //statusProgressbar.setIndeterminate(true);
                CyNetwork network = netListModel.getSelectedNetwork();
                Vector<CyNetwork> netlist = new Vector<CyNetwork>();
                oldNetList = netlist;
                netlist.add(network);
                Vector<ProteinCompare> comps = new Vector<ProteinCompare>();

                Collection<Cluster> clusters = null;
                ProteinTableModel model = (ProteinTableModel) proteinComparerTable.getModel();
                Vector<Integer> weights = new Vector<Integer>(model.weight);
                if (running) {
                    System.out.println("making comparer");
                    //statusLabel.setText("Status: Making comparer");
                    //statusLabel.repaint();

                    ProteinCompare comp = proteinComparerTable.getComparer(netlist);
                    oldComp = comp;
                    System.out.println("Clustering...");
                    // statusLabel.setText("Status: Clustering");
                    //statusLabel.repaint();
                    Collection<CyNetwork> nets = new Vector<CyNetwork>();
                    nets.add(network);
                    double limit =1-  clusteringLimitSlider.getValue();

                    oldLimit = limit;
                    clusters = clusterListModel.cluster(nets, comp, QualityAssesser.CompareClusterNodeType.MIN_REP, QualityAssesser.CompareClusterType.AVERAGE_REP, limit, joinSingleCluseters.isSelected());
                    if(clusters==null){
                        System.out.println("No clusters");
                        return;
                    }
                    int size = 0;

                Set<Node> nodeset = new HashSet<Node>();
                for (Cluster cluster : clusters) {
                    size += cluster.size();
                    nodeset.addAll(cluster.getRepNodes());


                    }
                    int nodecount = nodeset.size();
                    nodeset = null;
                    DecimalFormat df = new DecimalFormat("#.#");
                    String clusterReusult = clusters.size() + " clusters, " + nodecount + " nodes on average in " + df.format(((double) size) / (double) nodecount) + " clusters.";
                    clusteringInfoLabel.setText(clusterReusult);
                    clusteringInfoLabel.repaint();
                    oldClusters = clusters;
                    ;
                    statisticsButton.setEnabled(true);
                    clusterCount = clusters.size();
                }
                if (running) {

                    System.out.println("Starting Apriori");
                    long start = System.currentTimeMillis();
                    // statusLabel.setText("Status: Starting Apriori");
                    // todo: should the number of overlaps be set in gui?
                    apriori = new Apriori(clusters, network, 0);


                    // todo: apriori should update a gui status bas
                    int motifSize = (Integer) maxMotifSizeSpinner.getValue();
                    int supportThresh = (Integer) supportSpinner.getValue();

                    System.out.println("Running Apriori");
                    // statusLabel.setText("Status: Running Apriori");
                    // statusLabel.repaint();
                    //Runtime.getRuntime().runFinalization();
                    //Runtime.getRuntime().gc();

                    //AprioriResults apr = new AprioriResults(apriori.applyMerge(motifSize,supportThresh ));
                    Map<GraphMotif, Vector<AprioriInstance>> results = apriori.apply(motifSize, supportThresh);
                    oldresults = results;
                    saveButton.setEnabled(true);
                    System.out.println("res " + results);
                    System.out.println("Run time: " + ((double) (System.currentTimeMillis() - start)) / 1000.0);
                    //SubgraphTreeModel s = new SubgraphTreeModel(results);
                    AprioriResults res = new AprioriResults(results);
                    //System.out.println(apr.toString());
                    //System.out.println("Showing");

                    resultTableModel = res;
                    updateTable();
                }
                // statusProgressbar.setIndeterminate(false);
                // if (running) statusLabel.setText("Status: done");
                // else statusLabel.setText("Status: Interrupted");
                running = false;
                // statusLabel.repaint();
                cancelButton.setEnabled(false);
                startButton.setEnabled(needsProtComp ? proteinComparerTable.isReady() : true);
                // statusProgressbar.setStringPainted(false);
                //  statusProgressbar.repaint();
                apriori = null;
                System.gc();


                clusterCount = 0;
                System.out.println("done");


            }

        });
        runThread.start();
        final java.util.Timer timer = new java.util.Timer();
        timer.schedule(new TimerTask() {
            public void run() {
                if (running) {
                    if (apriori != null) {
                        //statusProgressbar.setString("clusters:" + clusterCount + " working on size: " + apriori.getLevel() + " found:" + apriori.getMotifFoundSoFar());
                        //statusProgressbar.repaint();

                    }
                } else {
                    timer.cancel();
                }
            }
        }, 0, 1 * 1000);  //subsequent rate


    }

    private void statButtonPressed(ActionEvent e) {
        StatFrame stats = new StatFrame(oldClusters, oldComp, oldNetList, oldLimit);
        stats.setVisible(true);

    }

    private void updateTable() {
        javax.swing.SwingUtilities.invokeLater(new Runnable() {
            public void run() {

                resultTable.getSelectionModel().removeListSelectionListener(oldresulttablemodel);
                TableSorter sorter = new TableSorter(resultTableModel);
                resultTable.setModel(sorter);
                sorter.setTableHeader(resultTable.getTableHeader());
                resultTable.setTableHeader(sorter.getTableHeader());
                resultTable.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
                resultTable.getSelectionModel().addListSelectionListener(resultTableModel);
                //resultTree.setAutoCreateColumnsFromModel(true);
                oldresulttablemodel = resultTableModel;
                // jPanelResults.removeAll();
                //jPanelResults.add(table);

                //table.setVisible(true);

                resultTableModel.fireTableDataChanged();
            }
        });

    }

    private void cancelButtonActionPerformed(java.awt.event.ActionEvent evt) {
        System.out.println("Cancel");
        running = false;
        if (apriori != null) apriori.stopRunning();

    }

    private javax.swing.JPanel buttonPanel;
    private javax.swing.JButton cancelButton;
    private javax.swing.JLabel choseClustererLabel;
    private javax.swing.JComboBox clusteringList;
    private javax.swing.JLabel limitLabel;
    SliderWrapper clusteringLimitSlider;

    private javax.swing.JSpinner maxMotifSizeSpinner;
    //private javax.swing.JPanel miningSetupPanel;
    private javax.swing.JPanel miningPanel;
    private javax.swing.JPanel miningResultPanel;
    private javax.swing.JScrollPane miningResultScroller;
    //private javax.swing.JTextArea miningResultTextArea;
    private JTable resultTable;
    private javax.swing.JScrollPane miningSetupScroller;

    private javax.swing.JComboBox networkchoser;
    private javax.swing.JLabel proteinComparerLabel;
    private javax.swing.JPanel proteinComparerPanel;
    private ProteinComparerTable proteinComparerTable;
    JScrollPane proteinComparerTableScroller;
    private javax.swing.JLabel selectMotifSizeLabel;

    private javax.swing.JButton startButton;
    private ViewableResults resultTableModel;
}
