package nemi.apriori;

import cytoscape.CyNetwork;
import giny.model.Node;
import nemi.gui.ViewableResults;

import javax.swing.*;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.TableModelListener;
import java.util.HashSet;
import java.util.Map;
import java.util.Set;
import java.util.Vector;

/**
 * Created by IntelliJ IDEA.
 * User: kristost
 * Date: 24.feb.2006
 * Time: 15:20:15
 * A table model implementation presenting the Apriori graphmining's results. 
 */
public class AprioriResults extends ViewableResults {
    private Map<GraphMotif, Vector<AprioriInstance>> graphs;
    private Vector<GraphMotif> motifs;
    private CyNetwork network;
    int level = 0;

    public AprioriResults(Map<GraphMotif, Vector<AprioriInstance>> graphs) {
        super();
        this.graphs = graphs;
        motifs = new Vector<GraphMotif>();
        motifs.addAll(graphs.keySet());
        if (motifs.size() > 0) {
            network = graphs.entrySet().iterator().next().getValue().get(0).parentNet;
        }
    }

    // from AbstractTableModel
    public int getRowCount() {
        if (graphs == null) {
            return 0;
        }
        return motifs.size();
    }

    public int getColumnCount() {
        return 3;  //To change body of implemented methods use File | Settings | File Templates.
    }

    public Object getValueAt(int rowIndex, int columnIndex) {
        switch (columnIndex) {
            case 0:
                return motifs.get(rowIndex);
            case 1: return motifs.get(rowIndex).nodeCount();
            case 2:
                return graphs.get(motifs.get(rowIndex)).size();
        }
        return null;  //To change body of implemented methods use File | Settings | File Templates.
    }

    public String getColumnName(int columnIndex) {
        switch (columnIndex) {
            case 0:
                return "Labels and code";
            case 1: return "Node count"   ;
            case 2:
                return "Pseudo-frequency";

        }

        return null;
    }

    public Class<?> getColumnClass(int columnIndex) {
        return String.class;
    }

    public boolean isCellEditable(int rowIndex, int columnIndex) {
        return false;  //To change body of implemented methods use File | Settings | File Templates.
    }

    public void setValueAt(Object aValue, int rowIndex, int columnIndex) {
        System.out.println("AprioriResults: Apriori table not editable");
    }

    Vector<TableModelListener> listeners = new Vector<TableModelListener>();

    public void addTableModelListener(TableModelListener l) {
        listeners.add(l);
    }

    public void removeTableModelListener(TableModelListener l) {
        listeners.remove(l);
    }

    // selection changed, flag subgraph's nodes
    public void valueChanged(ListSelectionEvent e) {
        //Ignore extra messages.
        if (e.getValueIsAdjusting()) return;

        ListSelectionModel lsm =
                (ListSelectionModel) e.getSource();
        if (!lsm.isSelectionEmpty()) {

            int selectedRow = lsm.getMinSelectionIndex();
            if(lsm.isSelectedIndex(selectedRow)) selectedRow= lsm.getMaxSelectionIndex();
            network = graphs.get(motifs.get(selectedRow)).iterator().next().parentNet;
            Set<Node> flagged = new HashSet<Node>(network.getFlaggedNodes());
            if (flagged.size() > 0)
                network.setFlaggedNodes(flagged, false);
            if (selectedRow >= 0 && selectedRow < graphs.size()) {
                for(AprioriInstance sg: graphs.get(motifs.get(selectedRow))){
                    sg.flagNodes();
                }
            }
        }
    }

    public void stopRunning() {

        // todo: kill apriori
        //To change body of implemented methods use File | Settings | File Templates.
    }

    public String getToolTipText(int rowIndex) {
        return graphs.get(rowIndex).toString();//.toHtml();  //To change body of implemented methods use File | Settings | File Templates.
    }

    public String toString() {
        if (graphs == null) return "AprioriResults: No graph";
        StringBuilder sb = new StringBuilder();
        sb.append("AprioriResults: \n");
        //for (AprioriInstance g : graphs) {
        //    sb.append(g.toString());
        //    sb.append("\n");
        //}
        return sb.toString();
    }
}
