package nemi.apriori;

import cytoscape.CyNetwork;
import giny.model.Node;
import nemi.Toolkit;

import java.util.*;

/**
 * Calculates and memoizes distance between any two nodes in the given network.
 */
public class GraphDistance {
    HashMap<Node, HashMap<Node, Integer>> distance = new HashMap<Node, HashMap<Node, Integer>>();
    CyNetwork network;

    public GraphDistance(CyNetwork network) {
        this.network = network;
    }

    public int distance(Node source, Node target) {
        if (source.equals(target)) return 0;
        HashMap<Node, Integer> sourceMap = distance.get(source);
        HashMap<Node, Integer> targetMap = distance.get(target);
        if (sourceMap != null) {
            if (sourceMap.containsKey(target)) return sourceMap.get(target);
            return -1;
        }
        if (targetMap != null) {
            if (targetMap.containsKey(source)) return targetMap.get(source);
            return -1;
        }
        Map<Node, Set<Node>> neighmap = Toolkit.getNodeNeighborMap(network);
        Queue<Node> que = new LinkedList<Node>();
        HashSet<Node> seen = new HashSet<Node>();
        que.offer(source);
        seen.add(source);

        if (sourceMap == null) {
            sourceMap = new HashMap<Node, Integer>();
            distance.put(source, sourceMap);
        }
        sourceMap.put(source, 0);
        while (!que.isEmpty()) {
            Node n = que.poll();
            int dist = sourceMap.get(n);
            if (neighmap.get(n) == null) {
                System.out.print(" dist error ");
                continue;
            }
            for (Node neigh : neighmap.get(n)) {
                if (!seen.contains(neigh)) {
                    seen.add(neigh);
                    sourceMap.put(neigh, dist + 1);
                    que.offer(neigh);
                }

            }
        }
        if (sourceMap.containsKey(target)) return sourceMap.get(target);
        return -1;
    }

    public boolean hasEdge(Node n1, Node n2) {
        Map<Node, Set<Node>> neighmap = Toolkit.getNodeNeighborMap(network);
        return neighmap.get(n1).contains(n2);
    }

}
