package nemi.apriori;

import java.util.Collections;
import java.util.HashSet;
import java.util.Set;
import java.util.Vector;

public abstract  class SubGraphView  implements Comparable<SubGraphView> {
	public abstract void setEdge(int i, int j, boolean value) ;

	public abstract boolean getEdge(int i, int j) ;
	public abstract  String getLabels();

	public abstract  String getLabelOf(int index) ;
	public abstract int nodeCount() ;

	public abstract boolean isCorrectLength();
    public abstract String getStringCode();
	public abstract boolean isSorted();
    public abstract int hashCode();
    public abstract boolean equals(Object o);
    public static GraphMotif findNormalForm(SubGraphView motif){
		String labels = motif.getLabels();
		if (!(motif.isSorted() && motif.isCorrectLength())) {
			System.out.println("Not sorted or correct length");
			return null;
		}
		// Generate a vector of graphmotifs with only one node
		// the permutation vector keeps track of the mapping to the start motif
		Vector<GraphMotifHelper> liste = new Vector<GraphMotifHelper>();
		for (int i = 0; i < motif.nodeCount(); i++) {
			Vector<Integer> perm = new Vector<Integer>();
			perm.add(i);
			GraphMotifHelper m = new GraphMotifHelper("", motif.getLabelOf(i));
			m.permutation = perm;
			liste.add(m);

		}
		// starting, all simple graphmotifs are least.
		Set<GraphMotifHelper> result = new HashSet<GraphMotifHelper>();

		recurseNormalForm(motif, liste, 0, result, false);
		for (GraphMotifHelper m : result) {
			m.permutation = null;
			return m;
		}
		return null;
	}
	public static GraphMotif makeCanonical(SubGraphView motif, Set<GraphMotifHelper> returnNormalforms) {
		String labels = motif.getLabels();
		if (!(motif.isSorted() && motif.isCorrectLength())) {
			System.out.println("Not sorted or correct length");
			return null;
		} 
		if(returnNormalforms == null){
			System.out.println("Canonicalforms must be stored in returnNormalForms");
			return null;
		}
		// Generate a vector of graphmotifs with only one node
		// the permutation vector keeps track of the mapping to the start motif
		Vector<GraphMotifHelper> liste = new Vector<GraphMotifHelper>();
		for (int i = 0; i < motif.nodeCount(); i++) {
			Vector<Integer> perm = new Vector<Integer>();
			perm.add(i);
			GraphMotifHelper m = new GraphMotifHelper("", motif.getLabelOf(i));
			m.permutation = perm;
			liste.add(m);
		}
		// starting, all simple graphmotifs are least.
		//Set<GraphMotifHelper> result = new HashSet<GraphMotifHelper>();
		for (int i = 0; i < liste.size(); i++) {
			// System.out.println("Recursing with " + liste.get(i) +" as
			// least");
			recurseNormalForm(motif, liste, i, returnNormalforms, true);
		}
		//Vector<GraphMotifHelper> sortlist = new Vector<GraphMotifHelper>(returnNormalforms.size());
		//sortlist.addAll(returnNormalforms);
		//Collections.sort(sortlist);
        //System.out.println("Normals: " + liste);
        GraphMotifHelper canonic = null;
		for (GraphMotifHelper m : returnNormalforms) {
			if (canonic ==null){
				canonic = m;
			} else {
				if(canonic.compareTo(m)>0){
					canonic = m;
				}
			}
			m.permutation = null;
			
		}
		canonic.permutation = null;
		return canonic;

	}
	public static void recurseNormalForm(SubGraphView start,
			Vector<GraphMotifHelper> motifList, int least, Set<GraphMotifHelper> results,
			boolean all) {
		Vector<GraphMotifHelper> finalList = new Vector<GraphMotifHelper>();
		// joint the least motif with all other motifs.
		GraphMotifHelper leastMotif = (GraphMotifHelper) motifList.get(least);
		int lastLeast = leastMotif.permutation.lastElement();
		for (int i = 0; i < motifList.size(); i++) {
			if (i == least)
				continue;
			GraphMotifHelper otherMotif = motifList.get(i);
			boolean joinable = true;
			// check wether leastmotif and othermotif has equal pointers to
			// startmotif
			// up until the last node
			for (int j = 0; j < leastMotif.nodeCount() - 1; j++) {
				if (leastMotif.permutation.get(j) != otherMotif.permutation
						.get(j)) {
					joinable = false;
					break;
				}
			}
			if (!joinable)
				break;
			int lastOther = otherMotif.permutation.lastElement();
			// make sure that the last element is different
			if (lastOther == lastLeast) {
				System.out.println("last element different");
				break;
			}
			// Check that code is less
			// TODO: when we are sure this can be removed
			int c = start.getLabelOf(lastLeast).compareTo(start.getLabelOf(lastOther));
			if(c>0){
				break;
			}
			if (leastMotif.compareTo( otherMotif) > 0) {
				System.out.println("Storre leastmotif.. noe galt");
				break;
			}

			// copy the leastmotif
			GraphMotifHelper newMotif = (GraphMotifHelper) leastMotif.clone();
			// grow it to fit this new node.
			
			newMotif.permutation.add(lastOther);
			String labels = leastMotif.getLabels();
			labels += otherMotif.getLabelOf(otherMotif.nodeCount() - 1);
			newMotif.setLabels(labels);
			newMotif.setSize((labels.length() * (labels.length() - 1)) / 2);
			// fill the edges
			for (int j = 0; j < leastMotif.nodeCount(); j++) {
				int index = leastMotif.permutation.get(j);
				boolean edge = start.getEdge(lastOther, index);
				newMotif.setEdge(j, leastMotif.nodeCount(), edge);
			}
			finalList.add(newMotif);
		}
		if(finalList.size() < 1) return;
		Collections.sort(finalList);
		/*System.out.println("level: " + leastMotif.nodeCount() + " finallist "
		 +finalList );*/
		GraphMotif newleast = finalList.get(0);
		if (newleast.nodeCount() == start.nodeCount()) {
			results.addAll(finalList);
		} else if (all) {

			for (int i = 0; i < finalList.size(); i++) {
				GraphMotif other = finalList.get(i);
				if (newleast.compareTo( other) == 0) {
					recurseNormalForm(start, finalList, i, results, all);
				} else {
					break;
				}

			}
		} else {
			recurseNormalForm(start, finalList, 0, results, all);
		}
	}
}
