//$Id: Cluster.java 509 2006-06-14 09:50:16Z sundsdal $
package nemi.cluster;

import cytoscape.CyNetwork;
import giny.model.Node;
import nemi.NetNode;

import java.util.Collection;
import java.util.List;
import java.util.Random;
import java.util.Vector;


/**
 * Cluster implementation used by Apriori and gSpan. Contains a collection of nodes.
 */
public class Cluster implements Comparable<Cluster> {
    private static char counter = 'A';
    boolean buildPhase = true;
    // WeakHashMap<Cluster, Double> distanceMap;
    java.util.Random r = new Random();
    // is a number above zero
    public char id = counter++;
    List<NetNode> repset = null;

    /**
     * Initialize a cluster with a representative list
     **/
    public Cluster(List<NetNode> nodes) {
        //this.nodes = new Vector<Node>(nodes);
        repset = new Vector<NetNode>(nodes);
    }

    /**
     * Start a cluster with a list of nodes from a network
     * @param nodes
     * @param net
     */
    public Cluster(List<Node> nodes, CyNetwork net){
        repset = new Vector<NetNode>();
        for (Node node : nodes) {
            repset.add(new NetNode(net, node));
        }
    }

    /**
     * initialize a cluster with a single node
     * @param n
     * @param net
     */
    public Cluster(Node n, CyNetwork net) {
        //this.nodes = new Vector<Node>();
        //nodes.add(n);
        repset = new Vector<NetNode>();
        repset.add(new NetNode(net, n));

    }

    /**
     * Copy a cluster, new id
     * @return identical cluster
     */
    public Cluster clone() {
        return new Cluster(repset);
    }


    /**
     * Get the collection of nodes in this cluster.
     * @return This clusters' nodes.
     */
    public Collection<Node> getRepNodes() {
        Vector<Node> ret = new Vector<Node>();
        for (NetNode netNode : repset) {
              ret.add(netNode.node);
        }
        return ret;
    }
        /**
     * Get the collection of netnodes in this cluster.
     * @return This clusters netnodes.
     */
    public Collection<NetNode> getRepNetNodes() {
        return this.repset;
    }
        /**
     * Get the number of clusters represented by this cluster
     * @return This clusters' nodes.
     */
    public int size() {
        return repset.size();
    }


    public void merge(Cluster w) {
        repset.addAll(w.repset);
    }

    public void add(Node n, CyNetwork net) {
        repset.add(new NetNode(net, n));
    }

    public int compareTo(Cluster o) {

            return this.id - o.id;
    }
    public void add(NetNode netNode) {
        repset.add(netNode);
    }

    public boolean equals(Cluster other) {
        if (other == null) return false;
        return (id == other.id && repset.equals(other.getRepNodes()));
    }

    public String toString() {
        StringBuilder sb = new StringBuilder();
        sb.append("Cluster '");
        sb.append(id);
        sb.append("' - ");
        if (repset.size() < 4) {
            for (NetNode netNode : repset) {
                sb.append(netNode.node.getIdentifier() + ",");
            }
            sb.deleteCharAt(sb.length() - 1);
        } else {
            sb.append(repset.size());
            sb.append(" nodes");
        }
        return sb.toString();
    }
}
