//$Id:IterativeClusterer.java 129 2006-02-21 16:33:10Z sundsdal $
package nemi.cluster;

import cytoscape.CyNetwork;
import giny.model.Node;
import nemi.NetNode;
import nemi.compare.ProteinCompare;

import java.util.*;

/**
 * Iterative clusterer. Clusters all nodes in the given networks using the given comparer.
 */
public class IterativeClusterer implements Clusterer {
          Comparator clusterComp = new Comparator<Cluster>() {
                public int compare(Cluster o1, Cluster o2) {
                    return o2.size() - o1.size();
                }
            };
    Comparator<Node> nodeComp = new Comparator<Node>() {
        public int compare(Node o1, Node o2) {
            return o1.getIdentifier().compareTo(o2.getIdentifier());

        }
    };
    /**
     * Returns a list of clusters.
     *
     * @param networks           Networks to be clustered.
     * @param comp               Single node to node comparer
     * @param compType           The comparer type.
     * @param least              Cluster limit. All nodes equal or more equal to least will be placed in the same
     *                           clusters.
     * @param joinSingleClusters When all nodes are clustered, some clusters may contain only one single
     *                           node. This flag joins all single-noded clusters.
     * @return A list of clusters.
     */
    public List<Cluster> cluster(Collection<CyNetwork> networks, ProteinCompare comp,
                                        QualityAssesser.CompareClusterNodeType compType
            , double least, boolean joinSingleClusters) {

        Vector<Node> nodesNotInCluster = new Vector<Node>();

        //System.out.println("making networkset");
        HashSet<Cluster> clusters = new HashSet<Cluster>();
        int mycount = 0;
        for (CyNetwork network : networks) {
            nodesNotInCluster.ensureCapacity(network.getNodeCount() + nodesNotInCluster.size());
            for (Iterator<Node> nodeIter = network.nodesIterator(); nodeIter.hasNext();) {
                nodesNotInCluster.add(nodeIter.next());
                mycount++;
            }
        }

        Collections.sort(nodesNotInCluster, nodeComp);
        Vector<Node> nodesInCluster = new Vector<Node>(mycount);

        //System.out.println(least + " Nodecount " + mycount);
        //nodesInCluster.addAll(nodesNotInCluster);
        Vector<Node> nodesInClusterTemp = new Vector<Node>();
        while (nodesNotInCluster.size() > 0) {
            //System.out.println("rest:" + nodesNotInCluster.size());
            //System.out.flush();
            Node nStart = extract(nodesNotInCluster);
            Cluster c = new Cluster(nStart, null);
            nodesInClusterTemp.add(nStart);
            //try non-clustered nodes first
            for (Iterator<Node> notInClusterIter = nodesNotInCluster.iterator(); notInClusterIter.hasNext();)
            {
                Node n = notInClusterIter.next();
                double similarity = QualityAssesser.similarity(comp, compType, c, n);
                //System.out.println("Similarity between cluster " + c.id + " to node " + n.getIdentifier() + " is " + similarity);
                if (similarity >= least) {
                    //System.out.println("added");
                    c.add(n, null);
                    notInClusterIter.remove();
                    nodesInClusterTemp.add(n);
                }
            }
            for (Node n : nodesInCluster) {
                double similarity = QualityAssesser.similarity(comp, compType, c, n);
                //System.out.println("Similarity between cluster " + c.id + " to node " + n.getIdentifier() + " is " + similarity);
                if (similarity >= least) {
                    //System.out.println("added");
                    c.add(n, null);
                }
            }
            for(Node n: nodesInClusterTemp){
                int i = Collections.binarySearch(nodesInCluster, n, nodeComp);;
                if (i<0) i = -i-1;
                nodesInCluster.add(i, n);

            }
            nodesInClusterTemp.clear();
            clusters.add(c);
        }
        //System.out.println("\nGarbage collect...");
        System.gc();
        //System.out.println("Number of clusters " + clusters.size());
        int sum = 0;
        //char i = 'a';
        int i = 0;
        Cluster single = new Cluster(new Vector<NetNode>());
        single.id = '?';
        List<Cluster> clusterList = new Vector<Cluster>(clusters);
        Collections.sort(clusterList, clusterComp);
        char maxChar = readableCharacters[0];

        for (Iterator<Cluster> clusterIter = clusterList.iterator(); clusterIter.hasNext();) {
            Cluster c = clusterIter.next();
            sum += c.size();
            if (c.size() == 1 && joinSingleClusters) {
                clusterIter.remove();
                single.add(c.getRepNetNodes().iterator().next());
            } else {
                if (i < readableCharacters.length) {
                    c.id = readableCharacters[i++];
                    if (c.id > maxChar) maxChar = c.id;
                } else {
                    c.id = ++maxChar;
                }
            }
            //System.out.println(c);
        }
        if (joinSingleClusters && single.size() > 0) {

            int index = Collections.binarySearch(clusterList, single, clusterComp);

            if (index < 0) {
                clusterList.add((-index) - 1, single);
            } else {
                clusterList.add(index, single);
            }

        }
        //System.out.println("Total number of nodes: " + sum + ", factor: " + (double) sum / (double) nodesInCluster.size());

        return clusterList;
    }

    protected Node extract(Vector<Node> nodesNotInCluster) {
        return nodesNotInCluster.remove(0);
    }


}
