//$Id:QualityAssesser.java 129 2006-02-21 16:33:10Z sundsdal $
package nemi.cluster;

import giny.model.Node;
import nemi.NetNode;
import nemi.compare.ProteinCompare;

import java.util.Vector;
import java.util.WeakHashMap;

/**
 *
 */
public class QualityAssesser {
	public static enum CompareClusterType {
		MIN_REP, MAX_REP, HAUSDORF_REP, AVERAGE_REP
	}

	public static enum CompareClusterNodeType {
		MIN_REP, MAX_REP, AVG_REP;
	}


    static public double similarity(ProteinCompare comp, CompareClusterNodeType clusterNodeCompare,
                                    Cluster cluster, Node n) {
        return similarity(comp, clusterNodeCompare, cluster, new NetNode(null, n));
    }
    static public double similarity(ProteinCompare comp, CompareClusterNodeType clusterNodeCompare,
                                    Cluster cluster, NetNode n) {


		double d = 0;
		if (clusterNodeCompare == CompareClusterNodeType.MIN_REP) {
			d = java.lang.Double.MAX_VALUE;
			for (NetNode cn : cluster.getRepNetNodes()) {
				d = Math.min(comp.similarity(n, cn), d);
			}
		} else if (clusterNodeCompare == CompareClusterNodeType.MAX_REP) {
			d = java.lang.Double.MIN_VALUE;
			for (NetNode cn : cluster.getRepNetNodes()) {
				d = Math.max(comp.similarity(n, cn), d);
			}
		} else if (clusterNodeCompare == CompareClusterNodeType.AVG_REP) {
			d = 0;
			for (NetNode cn : cluster.getRepNetNodes()) {
				d += comp.similarity(n, cn);
			}
			d /= (double) cluster.getRepNodes().size();
		}
		return d;
	}

	public static double distance(ProteinCompare comp,CompareClusterType clusterCompare, Cluster cluster1, Cluster cluster2) {
		Cluster c1, c2;
		if (cluster1.id < cluster2.id) {
			c1 = cluster1;
			c2 = cluster2;
		} else {
			c1 = cluster2;
			c2 = cluster1;
		}
		double d = 0;

		if (clusterCompare == CompareClusterType.MIN_REP) {
			d = Double.MAX_VALUE;
			for (NetNode here : c1.getRepNetNodes()) {
				for (NetNode there : c2.getRepNetNodes()) {
					d = Math.min(d, 1 - comp.similarity(here,there));
				}
			}
		} else if (clusterCompare == CompareClusterType.AVERAGE_REP) {
			int i = 0;
			for (NetNode here : c1.getRepNetNodes()) {
				for (NetNode there : c2.getRepNetNodes()) {
					i++;
					d += 1 - comp.similarity(here, there);
				}
			}
			d = d / (double) i;
		} else if (clusterCompare == CompareClusterType.MAX_REP) {
			d = Double.MIN_VALUE;
			for (NetNode here : c1.getRepNetNodes()) {
				for (NetNode there : c2.getRepNetNodes()) {
					d = Math.max(d, 1 - comp.similarity(here, there));

				}
			}
		} else if (clusterCompare == CompareClusterType.HAUSDORF_REP) {

			double hHT = Double.MIN_VALUE;
			double hTH = Double.MIN_VALUE;
			for (NetNode here : c1.getRepNetNodes()) {
				d = Double.MAX_VALUE;
				for (NetNode there : c2.getRepNetNodes()) {
					d = Math.min(d, 1 - comp.similarity(here,there));

				}
				hHT = Math.max(hHT, d);
			}
			for (NetNode there : c1.getRepNetNodes()) {
				d = Double.MAX_VALUE;
				for (NetNode here : c2.getRepNetNodes()) {
					d = Math.min(d, 1 - comp.similarity(here, there));
				}
				hTH = Math.max(hTH, d);
			}
			d = Math.max(hTH, hHT);
		}
		return d;
	}

	public static double quality(ProteinCompare comp, Cluster cluster) {
		double q = 0;
		Vector<NetNode> nodes = new Vector(cluster.getRepNetNodes());
		NetNode iNode, jNode;
		double temp = 0;
		int count = 0;
		for (int i = 0; i < nodes.size(); i++) {
			iNode = nodes.get(i);
			for (int j = i + 1; j < nodes.size(); j++) {
				jNode = nodes.get(j);
				temp = comp.similarity(iNode, jNode);
				if (temp == 0) {
					return 0;
				}
				q += temp * temp;
				count++;
			}
		}
		q = q / (double) count;
		return q;
	}

}
