package nemi.cluster;

import cytoscape.CyNetwork;
import giny.model.Node;
import nemi.NetNode;

import java.util.*;

/**
 * Clusterer based on transcription factor activation levels. For all transcription factors, all
 * proteins having activation level above nemi.Toolkit.tfProbLimit will be clustered. Bindings
 * are read from a textfile using nemi.Toolkit.readBindings().
 */
public class TFClusterer implements Clusterer {

    /**
     * Cluster all nodes in the given network.
     * @param networks
     * @param joinSingleClusters
     * @return A list of clusters based on the given network(s).
     */
    public static List<Cluster> cluster(Collection<CyNetwork> networks, boolean joinSingleClusters) {

        //Set<Node> nodesNotInCluster = new HashSet<Node>();
        Set<Node> nodeWithoutFac = new HashSet<Node>();

        List<Cluster> clusters = new Vector<Cluster>();
        Map<String, Map<String, Double>> id2fac2prob = nemi.Toolkit.readBindings();
        //Map<String, Map<String, Double>> id2fac2prob = new HashMap<String, Map<String, Double>>();
        Map<String, Set<String>> fac2idset = new HashMap<String, Set<String>>();
        Map<String, Node> id2Node = new HashMap<String, Node>();
        int mycount = 0;
        nemi.Toolkit.id2fac2p = null;
        for (Iterator<Map.Entry<String, Map<String, Double>>> entryIter = id2fac2prob.entrySet().iterator(); entryIter.hasNext();) {
              Map.Entry<String, Map<String, Double>> entry  = entryIter.next();
            for (Iterator<Map.Entry<String, Double>> fac2probEntryIter =
                    entry.getValue().entrySet().iterator(); fac2probEntryIter.hasNext();) {
                Map.Entry<String, Double> fac2probEntry = fac2probEntryIter.next();
                if (fac2probEntry.getValue() > nemi.Toolkit.tfProbLimit) {
                    fac2probEntryIter.remove();
                }


            }
            if(entry.getValue().size()==0) entryIter.remove();

        }

        for (CyNetwork network : networks) {
            //nodesNotInCluster.(nodesNotInCluster.size() + network.getNodeCount());
            for (Iterator<Node> nodeIter = network.nodesIterator(); nodeIter.hasNext();) {
                Node n = nodeIter.next();

                id2Node.put(n.getIdentifier(), n);
                mycount++;
                if (!id2fac2prob.containsKey(n.getIdentifier())) {
                    nodeWithoutFac.add(n);
                } else {
                    //nodesNotInCluster.add(n);
                    for (String fac : id2fac2prob.get(n.getIdentifier()).keySet()) {
                        if (!fac2idset.containsKey(fac)) fac2idset.put(fac, new HashSet<String>());
                        fac2idset.get(fac).add(n.getIdentifier());

                    }
                }
            }
        }
        for(Map.Entry<String, Set<String>> idsetEntry : fac2idset.entrySet()){

            Cluster c = new Cluster(new Vector<NetNode>());
            for(String s: idsetEntry.getValue()){
                c.add(id2Node.get(s), null);

            }
            clusters.add(c);

        }
        Collections.sort(clusters, new Comparator<Cluster>() {
            public int compare(Cluster o1, Cluster o2) {
                return o2.size() - o1.size();
            }
        });
        Set<Cluster> toRemove = new HashSet<Cluster>();
        for (int i = 0; i < clusters.size(); i++) {

            for (int j = i + 1; j < clusters.size(); j++) {
                if (clusters.get(i).getRepNodes().containsAll(clusters.get(j).getRepNodes())) {
                    toRemove.add(clusters.get(j));
                }
            }
        }
        for (Cluster c : toRemove) {
            clusters.remove(c);
        }
        char maxChar = readableCharacters[0];
        int sum = 0;
        int i = 0;
        for (Iterator<Cluster> clusterIter = clusters.iterator(); clusterIter.hasNext();) {
            Cluster c = clusterIter.next();
            sum += c.size();

            if (i < readableCharacters.length) {
                c.id = readableCharacters[i++];
                if (c.id > maxChar) maxChar = c.id;
            } else {
                c.id = ++maxChar;
            }
            //System.out.println(c);
        }
        if (joinSingleClusters) {
            Cluster c = new Cluster(new Vector<Node>(nodeWithoutFac), null);
            int index =Collections.binarySearch(clusters,c, new Comparator<Cluster>() {
            public int compare(Cluster o1, Cluster o2) {
                return o2.id - o1.id;
            }
        });
            c.id = '?';
            if(index<0) {
                clusters.add((-index)-1,c);
            } else {
                clusters.add(index,c);
            }
        } else {
            for (Node n : nodeWithoutFac) {
                Cluster c = new Cluster(n, null);
                if (i < readableCharacters.length) {
                    c.id = readableCharacters[i++];
                    if (c.id > maxChar) maxChar = c.id;
                } else {
                    c.id = ++maxChar;
                }
                clusters.add(c);
            }

        }

        System.out.println("Total number of nodes: " + sum + ", factor: " + (double) sum / (double) clusters.size());


        return clusters;
    }
}
