package nemi.compare;

import nemi.NetNode;

import java.util.HashMap;
import java.util.Vector;

/**
 * A weighted, multiple comparer, using several comparers.
 */
public class CompositeComparer extends ProteinCompare {
       public final static String name="Composite comparer";
    HashMap<String, HashMap<String, Double>> similarityMap;
	Vector<ProteinCompare> comparers;
	Vector<Integer> weights;
	public CompositeComparer(Vector<ProteinCompare> comparers,
	Vector<Integer> weights){
		this.comparers=comparers;
		this.weights = weights;
		similarityMap = new HashMap<String, HashMap<String, Double>>();
	}

    public String getName() {
        return this.name;
    }
    public String toString(){
        StringBuilder sb = new StringBuilder();
        sb.append("CompositeComparer:");
        for(ProteinCompare c : comparers){
            sb.append(c.toString());
            sb.append(", ");

        }
        sb.delete(sb.length()-2, sb.length()-1);
        return sb.toString();
    }

    /**
     * Gets the similarity between two given nodes.
     * @param n1
     * @param n2
     * @return Node similarity
     */
    public double similarity(NetNode n1, NetNode n2) {
        String sourceId = n1.node.getIdentifier();
        String targetId = n2.node.getIdentifier();
        
        String small, large;
        if (sourceId.compareTo(targetId) <= 0) {
            small = targetId;
            large = sourceId;
        } else {
            large = targetId;
            small = sourceId;
        }
        if (similarityMap.containsKey(small)) {
            HashMap<String, Double> map = similarityMap.get(small);
            if (map.containsKey(large)) {
                return map.get(large);
            }

        } else {
            similarityMap.put(small, new HashMap<String, Double>());
        }
        HashMap<String, Double> map = similarityMap.get(small);
        double similarity = 0;
        for(int i = 0; i < comparers.size();i++){
        	ProteinCompare c = comparers.get(i);
        	int weight = weights.get(i);
        	similarity += c.similarity(n1, n2) * weight;
        }
        similarity /= 100.0;
        map.put(large, similarity);
        return similarity;
	}

}
