package nemi.compare;

import giny.model.Node;
import nemi.NetNode;
import nemi.Toolkit;
import nemi.protein.CSAProteinFinder;

import java.util.HashMap;
import java.util.Map;
import java.util.Vector;

/**
 * Created by IntelliJ IDEA.
 * User: kristost
 * Date: 30.mai.2006
 * Time: 09:28:19
 * Using the comparer singleComp, find the best match between the two nodes' neighborhoods, and use
 * this neighborhood score as similariy measure for the two nodes.
 */
public class CsaNeighComparer {
    Map<NetNode, Map<NetNode, Double>> n2n2score = new HashMap<NetNode, Map<NetNode, Double>>();
    ProteinCompare singleComp;
    CSAProteinFinder csa;

    /**
     * @param singleComp Single node comparer used when comparing the nodes in the two neighborhoods.
     */
    public CsaNeighComparer(ProteinCompare singleComp) {
        this.singleComp = singleComp;
        this.csa = new CSAProteinFinder();
    }

    /**
     * Get the similarity between the two provided nodes.
     * @param source
     * @param target
     * @return Similarity score. 
     */
    public double similarity(NetNode source, NetNode target) {
        NetNode n1 = (source.node.getIdentifier().compareTo(target.node.getIdentifier()) < 0) ? source : target;
        NetNode n2 = (n1 == source) ? target : source;
        if (!n2n2score.containsKey(n1)) n2n2score.put(n1, new HashMap<NetNode, Double>());
        if (!n2n2score.get(n1).containsKey(n2)) {
            Vector<Node> sourceNeighs = new Vector<Node>(Toolkit.getNeighbors(source));
            Vector<Node> targetNeighs = new Vector<Node>(Toolkit.getNeighbors(target));
            CSAProteinFinder.CsaMatch match = csa.compareNeighbours(source, sourceNeighs, target, targetNeighs, singleComp);
            double score = match.score/(double)Math.max(sourceNeighs.size(), targetNeighs.size());
            // System.out.println("comparing sizes " + sourceNeighs.size() + " and " + targetNeighs.size() + "" + " gave score: " + score);
            n2n2score.get(n1).put(n2, score);
        }
        return n2n2score.get(n1).get(n2);  //To change body of implemented methods use File | Settings | File Templates.
    }

    public String getName() {
        return "Best neighborhood match";  //To change body of implemented methods use File | Settings | File Templates.
    }
}
