package nemi.compare;

import cytoscape.CyNetwork;
import cytoscape.Cytoscape;
import cytoscape.data.annotation.Annotation;
import cytoscape.data.annotation.AnnotationDescription;
import cytoscape.data.annotation.Ontology;
import cytoscape.data.servers.BioDataServer;
import nemi.NetNode;

import java.util.*;


/**
 * $Id: DeepestCommonGOCategory.java 460 2006-06-10 13:47:24Z sundsdal $
 * Node comparer based on the deepest common GO category. If the deepest common node found between the
 * two nodes to compare are a GO-root node; the similarity is zero.
 */
public class DeepestCommonGOCategory extends ProteinCompare {
    public final static String name = "GO:Deepest common parent";
    BioDataServer server;
    Annotation annotation;
    HashMap<Integer, Integer> ontologyDepth;


    public DeepestCommonGOCategory() {
        new DeepestCommonGOCategory(Cytoscape.getBioDataServer().
                getAnnotation(Cytoscape.getBioDataServer().getAnnotationDescriptions()[0]));
    }

    public DeepestCommonGOCategory(Annotation annotation) {
        BioDataServer server = Cytoscape.getBioDataServer();
        this.annotation = annotation;
        ontologyDepth = new HashMap<Integer, Integer>();
    }
      public String toString(){
        return name + "- " + this.annotation.getOntologyType();
    }
    public String getName() {
        return this.name;
    }

    Object[][] getDescriptions() {
        AnnotationDescription[] descs = server.getAnnotationDescriptions();
        Object[][] list = new Object[descs.length][3];
        for (int i = 0; i < descs.length; i++) {
            list[i] = new Object[]{descs[i].getType(), descs[i].getCurator(), descs[i].getSpecies(), descs[i]};
        }
        return list;
    }

    void setAnnotation(AnnotationDescription desc) {
        this.annotation = server.getAnnotation(desc);
    }


    public void runTotalCompare(CyNetwork network, String source) {


    }

    /**
     * Get the simlarity between the two provided nodes.
     * @param n1
     * @param n2
     * @return Similarity score.
     */
    public double similarity(NetNode n1, NetNode n2) {
        String sourceId = n1.node.getIdentifier();
        String targetId = n2.node.getIdentifier();
        //System.out.println("Comparing " + sourceId + " and " + targetId);
        String small, large;
        //Check wether distance is already computet
        if (sourceId.compareTo(targetId) <= 0) {
            small = targetId;
            large = sourceId;
        } else {
            large = targetId;
            small = sourceId;
        }

        Ontology ont = annotation.getOntology();

        HashMap<String, Set<Integer>> items = new HashMap<String, Set<Integer>>();
        items.put(sourceId, new HashSet<Integer>());
        items.put(targetId, new HashSet<Integer>());
        //If it isn't computed we need to compute ontology depths and find ontology-paths for both nodes.
        int maxdepth = 2;
        for (Map.Entry<String, Set<Integer>> entry : items.entrySet()) {
            String id = entry.getKey();
            Set<Integer> ontologyIDs = entry.getValue();

            Vector<Integer> sourceOnt = annotation.getClassificationsVector(id);
            for (Integer sOnt : sourceOnt) {
                int[][] ontPathIntegers = ont.getAllHierarchyPaths(sOnt);
                for (int[] path : ontPathIntegers) {
                    maxdepth = Math.max(maxdepth, path.length);
                    //System.out.println("Ontology id: " + path[path.length - 1]);
                    for (int i = 0; i < path.length; i++) {

                        int ontId = path[i];
                        ontologyIDs.add(ontId);
                        if (ontologyDepth.containsKey(ontId)) {
                            int depth = ontologyDepth.get(ontId);
                            if (depth > i) {
                                ontologyDepth.put(ontId, i);
                            }
                        } else {
                            ontologyDepth.put(ontId, i);
                        }
                    }
                }
            }
        }

        int max = 0;
        Set<Integer> sourceOntologyIds = items.get(sourceId);
        Set<Integer> targetOntologyIds = items.get(targetId);
        boolean first = true;
        for (Integer sid : sourceOntologyIds) {
            //System.out.println("sourceEntry " + sid + " " + ontologyDepth.get(sid));
            for (Integer tid : targetOntologyIds) {
                //if (first) System.out.println("targetEntry " + tid + " " + ontologyDepth.get(tid));
                if (sid.equals(tid)) {
                    int depth = ontologyDepth.get(sid);
                    max = Math.max(depth, max);
                }
            }
            first = false;
        }
        double similarity = ((double) max) / (double) maxdepth;

        return similarity;
    }
}
