package nemi.compare;

import cytoscape.CyNetwork;
import cytoscape.Cytoscape;
import cytoscape.data.annotation.Annotation;
import cytoscape.data.annotation.Ontology;
import cytoscape.data.servers.BioDataServer;
import giny.model.Node;
import nemi.NetNode;
import nemi.cytoscape.ErrorFrame;

import java.util.*;

/**
 * NeMi - Neighborhood GSpanMining
 * Distributed AS-IS - No Warranty
 * $Id: LeastLikelyGOParent.java 460 2006-06-10 13:47:24Z sundsdal $
 * Similarity measure based on the least likely common GO-parent found.
 */
public class LeastLikelyGOParent extends ProteinCompare {
    public final static String name = "GO:Least likely parent";
    BioDataServer server;
    HashMap<Integer, Double> categoryProbability;

    private double maxCount = Double.MIN_VALUE;
    Annotation annotation;

    public LeastLikelyGOParent(Annotation annotation) {
        this.annotation = annotation;
        init();
    }

    double globMinprob;
       public String toString(){
        return name + "- " + this.annotation.getOntologyType();
    }
    public LeastLikelyGOParent() {
        try {
            this.annotation = Cytoscape.getBioDataServer().
                    getAnnotation(Cytoscape.getBioDataServer().getAnnotationDescriptions()[0]);
        } catch (Exception e) {
            new ErrorFrame("Error", "Couldn't load annotation. " + e.toString());

            return;
        }
        init();
    }

    public String getName() {
        return this.name;
    }

    private void init() {
        //System.out.println("Chosen annotation: " + annotation.getType() + " " + annotation.getCurator() + " " + annotation.getCurator());

        categoryProbability = new HashMap<Integer, Double>();
    }

    public void addNetwork(CyNetwork network) {
        this.populateProbabilityMap(network);
    }

    private void populateProbabilityMap(CyNetwork network) {
        Ontology ont = annotation.getOntology();
        maxCount = Double.MIN_VALUE;

        double min = Double.MAX_VALUE;
        //System.out.println("StatistcalGOCategory comparer: Populating network " + network.getIdentifier());
        for (Iterator<Node> iter = (Iterator<Node>) network.nodesIterator(); iter.hasNext();) {
            Node n = iter.next();
            String id = n.getIdentifier();
            Vector<Integer> sourceOnt = annotation.getClassificationsVector(id);
            for (Integer sOnt : sourceOnt) {
                int[][] ontPathIntegers = ont.getAllHierarchyPaths(sOnt);
                for (int[] path : ontPathIntegers) {
                    for (int ontId : path) {
                        double count = 0;
                        if (!categoryProbability.containsKey(ontId)) {
                            categoryProbability.put(ontId, (double) ++count);
                        } else {
                            count = categoryProbability.get(ontId);
                            categoryProbability.put(ontId, ++count);
                        }
                        if (count > maxCount) {
                            maxCount = count;
                        } else if (count < min) {
                            min = count;
                        }
                    }
                }
            }
        }
        globMinprob = min / maxCount;
    }

    /**
     * Returns a value for the similaritytwo protein.
     * A similarity score of 1.0 means they are equal (or maxCount similarity), and 0.0 means total
     * dissimilarity.
     *
     * @return similarity between 0 and 1.
     */
    public double similarity(NetNode n1, NetNode n2) {
        String sourceId = n1.node.getIdentifier();
        String targetId = n2.node.getIdentifier();
        String small, large;
        Ontology ont = annotation.getOntology();
        if (sourceId.compareTo(targetId) <= 0) {
            small = targetId;
            large = sourceId;
        } else {
            large = targetId;
            small = sourceId;
        }

        HashMap<String, Set<Integer>> items = new HashMap<String, Set<Integer>>();
        items.put(sourceId, new HashSet<Integer>());
        items.put(targetId, new HashSet<Integer>());
        //If it isn't computed we need to compute ontology depths and find ontology-paths for both nodes.
        double thisMIN = Double.MAX_VALUE;
        for (Map.Entry<String, Set<Integer>> entry : items.entrySet()) {
            String id = entry.getKey();
            Set<Integer> ontologyIDs = entry.getValue();
            Vector<Integer> sourceOnt = annotation.getClassificationsVector(id);
            for (Integer sOnt : sourceOnt) {
                int[][] ontPathIntegers = ont.getAllHierarchyPaths(sOnt);
                for (int[] path : ontPathIntegers) {
                    for (int i = 0; i < path.length; i++) {
                        int ontId = path[i];
                        ontologyIDs.add(ontId);
                        if (categoryProbability.get(ontId) < thisMIN) {
                            thisMIN = categoryProbability.get(ontId);
                        }
                    }
                }
            }
        }
        Set<Integer> sourceOntSet = items.get(sourceId);
        Set<Integer> targetOntSet = items.get(targetId);

        sourceOntSet.retainAll(targetOntSet);
        double minProbability = Double.MAX_VALUE;
        int leastProbableOntology = -1;
        for (int ontology : sourceOntSet) {
            double prob = categoryProbability.get(ontology);

            if (prob < minProbability) {
                minProbability = prob;
                leastProbableOntology = ontology;
            }
        }

        double similarity = 0;
        if (thisMIN != maxCount && leastProbableOntology != -1) {
            similarity = thisMIN / minProbability;
        }
        similarity = Math.max(0, similarity);

        return similarity;
    }
}
