package nemi.compare.gui;

import cytoscape.CyNetwork;
import cytoscape.Cytoscape;
import cytoscape.data.annotation.Annotation;
import cytoscape.data.annotation.AnnotationDescription;
import nemi.compare.*;
import nemi.cytoscape.AnnotationList;

import javax.swing.*;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;
import javax.swing.event.TableModelEvent;
import javax.swing.event.TableModelListener;
import javax.swing.table.TableCellEditor;
import javax.swing.table.TableColumn;
import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ItemEvent;
import java.util.Collection;
import java.util.HashSet;
import java.util.Set;
import java.util.Vector;

/**
 * NeMi - Neighborhood Mining
 * Distributed AS-IS - No Warranty
 * $Id: ProteinComparerTable.java 453 2006-06-09 13:53:02Z sundsdal $
 *
 * A table for selecting protein comparer(s).
 */
public class ProteinComparerTable extends JTable {
    public static final Class[] comparers =
            new Class[]{
                    nemi.compare.TransFacCompare.class,
                    nemi.compare.DeepestCommonGOCategory.class,
                    nemi.compare.LeastLikelyGOParent.class,
                    nemi.compare.LogLeastLikelyGOParent.class
            };

    public nemi.compare.ProteinCompare getComparer(Collection<CyNetwork> networks) {

        Vector<ProteinCompare> comps = new Vector<ProteinCompare>();
        System.out.println("Creating comparer:");
        Vector<Integer> weights = new Vector<Integer>();
        for (int i = 0; i < model.comparer.size(); i++) {
            if (model.weight.get(0) == 0) continue;
            if (model.comparer.get(i).equals(LogLeastLikelyGOParent.class)) {
                AnnotationDescription desc = model.annotation.get(i);
                Annotation annotation = Cytoscape.getBioDataServer().getAnnotation(desc);
                LogLeastLikelyGOParent c = new LogLeastLikelyGOParent(annotation);
                for (CyNetwork cyNetwork : networks) {
                    c.addNetwork(cyNetwork);
                }

                comps.add(c);

                System.out.println("Log least likeliy parent in" + desc.toString() + " ,weight:" + model.weight.get(i));
            } else if (model.comparer.get(i).equals(LeastLikelyGOParent.class)) {
                AnnotationDescription desc = model.annotation.get(i);
                Annotation annotation = Cytoscape.getBioDataServer().getAnnotation(desc);
                LeastLikelyGOParent c = new LeastLikelyGOParent(annotation);
                for (CyNetwork cyNetwork : networks) {
                    c.addNetwork(cyNetwork);
                }

                comps.add(c);
                System.out.println("Least likeliy parent of" + desc.toString() + ", weight:" + model.weight.get(i));
            } else if (model.comparer.get(i).equals(DeepestCommonGOCategory.class)) {
                DeepestCommonGOCategory c = new DeepestCommonGOCategory(Cytoscape.getBioDataServer().getAnnotation(model.annotation.get(i)));
                comps.add(c);
                System.out.println("Deepest common parent in " + model.annotation.get(i).toString() + ", weight:" + model.weight.get(i));
            } else if (model.comparer.get(i).equals(TransFacCompare.class)) {
                TransFacCompare c = new TransFacCompare(networks);
                comps.add(c);
                System.out.println("Transcription factor comparer added.");
            }
            weights.add(model.weight.get(i));

        }
        ProteinCompare comp = new CompositeComparer(comps, weights);
        System.out.println("Comparer created.");
        return comp;
    }

    public void setEnabled(boolean b) {
        if(!b){
            model.clear();
            model.fireTableDataChanged();
        }
        super.setEnabled(b);
        setForeground(b ? Color.BLACK : Color.GRAY);
        repaint();


    }

    public static Class<? extends ProteinCompare> getCompareClass(String s) {
        for (Class<? extends ProteinCompare> clas : ProteinComparerTable.comparers) {
            try {
                if (clas.getField("name").get(null).equals(s)) {
                    return clas;
                }

            } catch (IllegalAccessException e) {
                e.printStackTrace();  //To change body of catch statement use File | Settings | File Templates.
            } catch (NoSuchFieldException e) {
                e.printStackTrace();  //To change body of catch statement use File | Settings | File Templates.
            }
        }
        System.out.println("String not found in comparer classes");
        return null;
    }

    public boolean isReady() {
        if (model != null) { // todo: legg til her.
            return model.getRowCount() > 1;
        }
        return false;
    }

    ComparerChoserTableEditor cce;
    AnnotationTableEditor ace;
    ProteinTableModel model;

    private Set<ActionListener> actionListeners = new HashSet<ActionListener>();
    private boolean ready = false; // notify listeners when ready

    public void addActionListener(ActionListener a) {
        actionListeners.add(a);
    }

    WeightProteinTableEditor wpe;
    public ProteinComparerTable() {
        super();
        this.setShowGrid(false);
        cce = new ComparerChoserTableEditor();
        ace = new AnnotationTableEditor();
        model = new ProteinTableModel();
        wpe = new WeightProteinTableEditor(model);
        this.setModel(model);
        this.resizeAndRepaint();
        TableColumn column = null;
        if (this.getColorModel() != null) {
            for (int i = 0; i < this.getColumnModel().getColumnCount(); i++) {
                column = this.getColumnModel().getColumn(i);
                int width = 0;
                switch (i) {
                    case 0:
                        width = (this.getWidth() - 35) / 3;
                        break;
                    case 1:
                        width = (this.getWidth() - 35) / 3;
                        break;
                    case 2:
                        width = 15;
                        break;
                    case 3:
                        width = 20;
                        break;
                    default:
                        break;
                }
                column.setPreferredWidth(width);
            }
            model.addTableModelListener(new TableModelListener() {
                public void tableChanged(TableModelEvent e) {
                    for (ActionListener l : actionListeners) {
                        l.actionPerformed(new ActionEvent(model, 0, "Model changed"));
                    }
                }
            });
        }
    }


    @Override
    protected void resizeAndRepaint() {
        super.resizeAndRepaint();
    }

    @Override
    public TableCellEditor getCellEditor(int row, int column) {
        if (column == 0) {
            return cce;
        } else if (column == 1 && row < this.getRowCount() - 1) {
            return ace;
        } else if ((column == 2 && row < this.getRowCount() - 1)) {
            return wpe;
        }
        return super.getCellEditor(row, column);
    }
}


class WeightProteinTableEditor extends AbstractCellEditor implements
        TableCellEditor, ChangeListener {

    protected static final String EDIT = "edit";

    JSpinner valueChanger;
    ProteinTableModel protmodel;
    int row;

    public WeightProteinTableEditor(ProteinTableModel model) {

        super();
        this.protmodel = model;
        SpinnerNumberModel nmodel = new SpinnerNumberModel(0, 0, 100, 1);
        valueChanger = new JSpinner(nmodel);

        valueChanger.addChangeListener(this);

    }

    // Implement the one method defined by TableCellEditor.
    public Component getTableCellEditorComponent(JTable table, Object value,
                                                 boolean isSelected, int row, int column) {
        this.row = row;
        valueChanger.setValue(value);
        return valueChanger;
    }

    public Object getCellEditorValue() {
        return valueChanger.getValue();
    }

    public void stateChanged(ChangeEvent e) {
        int value = protmodel.newWeight((Integer) valueChanger.getValue(), row);
        valueChanger.setValue(value);
    }
}

class AnnotationTableEditor extends AbstractCellEditor implements
        TableCellEditor, ActionListener {
    protected static final String EDIT = "edit";

    JComboBox goCombo;
    AnnotationList al;
    ComboBoxModel nonelist;
    public AnnotationTableEditor() {
        super();
        goCombo = new JComboBox();
        al = new AnnotationList();
        goCombo.setModel(al);
        nonelist = new DefaultComboBoxModel();

        goCombo.addActionListener(this);
        goCombo.addItemListener(new java.awt.event.ItemListener() {
            public void itemStateChanged(java.awt.event.ItemEvent evt) {
                clusteringListItemStateChanged(evt);
            }
        });
    }

    public void actionPerformed(ActionEvent e) {
        if ((e.getID() & ActionEvent.ACTION_PERFORMED) > 0) {
            // The user has clicked the cell, so
            // bring up the dialog.
            System.out.println("editing done");
            fireEditingStopped(); // Make the renderer reappear.

        } else { // User pressed dialog's "OK" button.
            System.out.println("editing done");
        }
    }

    public void clusteringListItemStateChanged(ItemEvent evt) {
        System.out.println("itemliststatechanged");

    }

    // Implement the one method defined by TableCellEditor.
    public Component getTableCellEditorComponent(JTable table, Object value,
                                                 boolean isSelected, int row, int column) {
        if (ProteinComparerTable.getCompareClass(
                (String) table.getValueAt(row, 0)).equals(TransFacCompare.class)) {
            goCombo.setModel(nonelist);
        } else {
            goCombo.setModel(al);
        }
        return goCombo;
    }

    public Object getCellEditorValue() {
        int index = goCombo.getSelectedIndex();
        if(index<0 || index> goCombo.getItemCount()){
            return "None";
        }
        return al.getIndex(index);
    }
}