package nemi.cytoscape;

import cytoscape.CyNetwork;
import cytoscape.CyNode;
import cytoscape.Cytoscape;
import cytoscape.data.annotation.Annotation;
import cytoscape.data.annotation.Ontology;
import cytoscape.plugin.CytoscapePlugin;
import cytoscape.util.CytoscapeAction;
import cytoscape.view.CyNetworkView;
import giny.view.NodeView;
import nemi.Toolkit;
import nemi.gui.NeMiFinalView;

import javax.swing.*;
import java.awt.event.ActionEvent;
import java.util.HashMap;
import java.util.List;
import java.util.Vector;


/**
 * NeMi - Neighborhood Mining; a Cytoscape plugin.
 *
 * We have implemented methods for the following tasks:<ul>
 * <li>Compare proteins based on neighborhood information</li>
 * <li> Find interaction sub pattern in an interaction network.</li>
 * <li> Find frequent sub patterns in one or several networks.</li></ul>
 *
 * We have constructed similarity measures between proteins that are based on Gene
 * Ontology and Transcription Factor experimental data, but other similarity
 * measures may easily be added.
 *
 * Our main contributions are improvements to graph mining algorithms gSpan and Apriori
 * whose original task was discover frequent sub patterns in a very large set of networks. We have
 * enabled mining a single network and loosened their requirement for
 * exact matches.
 *
 */
@SuppressWarnings( { "unchecked", "StringConcatenationInsideStringBufferAppend" })
public class NeMiPlugin extends CytoscapePlugin {

    /**
     * This constructor creates an action and adds it to the Plugins menu.
     */
    public NeMiPlugin() {
        // create a new action to respond to menu activation
        System.out.println("Running NeMiPlugin");
        SamplePluginAction action = new SamplePluginAction();
        // set the preferred menu
        action.setPreferredMenu("Plugins");
        // and add it to the menus
        Cytoscape.getDesktop().getCyMenus().addAction(action);
    }

    /**
     * Gives a description of this plugin.
     */
    public String describe() {
        StringBuffer sb = new StringBuffer();
        sb.append("For every currently selected node in the graph view, this ");
        sb.append("plugin additionally selects each neighbor of that node if ");
        sb.append("the canonical names of the two nodes have the same last letter.");
        return sb.toString();
    }

    /**
     * This class gets attached to the menu item.
     */
    public class SamplePluginAction extends CytoscapeAction {
        private Ontology ontology;
        private Annotation annotation;

        /**
         * The constructor sets the text that should appear on the menu item.
         */
        public SamplePluginAction() {
            super("NeMi - Neighborhood Mining");
        }

        /**
         * Initializes the NemiFinalView.
         * @param ae
         */
        public void actionPerformed(ActionEvent ae) {
            new NeMiFinalView();
        }

        public void printOntology() {
            // get the network object; this contains the graph
            CyNetwork network = Cytoscape.getCurrentNetwork();
            // get the network view object
            CyNetworkView view = Cytoscape.getCurrentNetworkView();
            // can't continue if either of these is null
            if (network == null || view == null) {
                return;
            }
            // put up a dialog if there are no selected nodes
            if (view.getSelectedNodes().size() == 0) {
                JOptionPane.showMessageDialog(view.getComponent(),
                        "Please select one or more nodes.");
            }
            HashMap map = annotation.getMap();

            // iterate over every node view
            for (NodeView nView : (List<NodeView>) view.getSelectedNodes()) {
                // first get the corresponding node in the network
                CyNode node = (CyNode) nView.getNode();
                String identifier = node.getIdentifier();
                Vector idents = (Vector) map.get(identifier);
                StringBuffer output = new StringBuffer();

                if (idents == null) {
                    output.append("No ontology");
                } else {
                    // noinspection
                    // StringConcatenationInsideStringBufferAppend
                    output.append("Node-ID:" + identifier + "\n");
                    for (Integer ontId : (Vector<Integer>) idents) {
                        output.append("Ontology-ID:" + ontId + "\n");
                        String[][] pathNamesArray = ontology
                                .getAllHierarchyPathsAsNames(ontId);
                        output
                                .append(Toolkit
                                        .pathNamesToString(pathNamesArray));
                    }
                }
                JOptionPane.showMessageDialog(view.getComponent(), output);
            }
        }

        private HashMap getOntologyMap(CyNetwork network) {
            return null;
        }

    }

}
