package nemi.gspan;

/**
 * Created by IntelliJ IDEA.
 * User: kristost
 * Date: 15.mar.2006
 * Time: 13:29:03
 *
 * A gSpan edge. Contains two discovery numbers and two corresponding labels for the two nodes.
 */
class DfsCodeEdge implements Comparable<DfsCodeEdge> {

    final char l1;
    final char l2; // labels
    final short i;
    final short j; // discovery times
    final boolean forward;

    /**
     * Creates a new gSpan dfsCodeEdge.
     * @param i first node's discovery number
     * @param j second node's discovery number
     * @param c1 first node's label
     * @param c2 second node's label
     */
    public DfsCodeEdge(int i, int j, char c1, char c2) {
        this.forward = (i < j);
        if (this.forward && i >= j)
            System.err.println(getClass() + " Warning, edge not forward from " + i + " to " + j);
        this.l1 = c1;
        this.l2 = c2;
        this.i = (short) i;
        this.j = (short) j;
    }

    public final boolean equals(Object obj) {
        if (obj instanceof DfsCodeEdge) {
            DfsCodeEdge e = (DfsCodeEdge) obj;
            return (e.l1 == l1 && e.l2 == l2 && e.i == i && e.j == j);
        }
        return false;
    }

    private int hashCode = Integer.MAX_VALUE;

    public final int hashCode() {
        if (hashCode == Integer.MAX_VALUE) {
            String s = (i + "." + j + "" + (int) l1 + "" + (int) l2) + "";
            hashCode = s.hashCode();
            if (forward) hashCode *= -1;
        }
        return hashCode;
    }

    public final String toString() {
        return i + "," + j + l1 + l2;
    }

    // comparing edges is not completely straightforward.. see of top left of p 289 in the gSpan/closeGraph article. 
    public final int compareTo(DfsCodeEdge other) {
        if (this.equals(other)) return 0;

        // checking the four cases
        if (case1(this, other) < 0) return -1;
        if (case1(other, this) < 0) return 1;
        if (case2(this, other) < 0) return -1;
        if (case2(other, this) < 0) return 1;
        if (case3(this, other) < 0) return -1;
        if (case3(other, this) < 0) return 1;
        if (case4(this, other) < 0) return -1;
        if (case4(other, this) < 0) return 1;

        // vertex label  / clusterId
        boolean firstClusterEqual = (l1 == other.l1);
        if (!firstClusterEqual) {
            return (l1 > other.l1) ? 1 : -1;
        }

        // no more to compare, than the last label
        boolean otherClusterEqual = (l2 == other.l2);
        if (!otherClusterEqual) {
            return (l2 > other.l2) ? 1 : -1;
        }
        System.err.println("DfsCodeEdge got an unexpected comparing");
        return 0;
    }


    private int case1(DfsCodeEdge first, DfsCodeEdge other) {
        if (
                (first.forward && other.forward) &&
                        ((first.j < other.j) || (first.i > other.i) && (first.j == other.j))
                )
            return -1;
        else
            return 0;
    }

    private int case2(DfsCodeEdge first, DfsCodeEdge other) {
        if (
                (!first.forward && !other.forward) && ((first.i < other.i)
                        || (first.i == other.i) && (first.j < other.j))
                )
            return -1;
        else
            return 0;
    }

    private int case3(DfsCodeEdge first, DfsCodeEdge other) {
        if (
                !first.forward && other.forward && first.i < other.j
                )
            return -1;
        else
            return 0;
    }

    private int case4(DfsCodeEdge first, DfsCodeEdge other) {
        if (
                first.forward && !other.forward && first.j <= other.i
                )
            return -1;
        else
            return 0;

    }
}
