package nemi.gspan;

import java.util.*;

/**
 * $Id: DfsMotif.java 441 2006-06-08 15:32:58Z kristost $
 * Created by IntelliJ IDEA.
 * User: kristost
 * Date: 15.mar.2006
 * Time: 13:40:38
 * <p/>
 * gSpan motif implementation. Contains a list of DfsCodeEdges and the nodes label's, sorted in the
 * discovery order of the nodes.
 */
public final class DfsMotif implements Comparable<DfsMotif>, Iterable<DfsCodeEdge> {
    // the treeset keeps the kode sorted (and minimal)
    public List<DfsCodeEdge> code = new Vector<DfsCodeEdge>();
    // list of clusters in discovery order
    public Vector<Character> labels = new Vector<Character>();
    // calculated when called for
    private int hashCode = Integer.MAX_VALUE;

    public DfsMotif(DfsCodeEdge startEdge) {
        if (startEdge.l1 > startEdge.l2) {
            System.out.println(getClass() + " Warning. DfsCode not minimal");
        }
        code.add(startEdge);
        labels.add(startEdge.l1);
        labels.add(startEdge.l2);
    }

    private DfsMotif() {

    }

    public Vector<Integer> getRightmostPathIxs() {
        int[] array = new int[labels.size()];
        for (DfsCodeEdge edge : code) {
            if (edge.forward) {
                array[(int) edge.i] = (int) edge.j;
            }
        }
        Vector<Integer> rmixs = new Vector<Integer>();
        rmixs.add(0);
        for (int i = 0; i < array.length - 1;) {
            rmixs.add(array[i]);
            i = array[i];
        }

        return rmixs;
    }


    @SuppressWarnings({"CloneDoesntCallSuperClone"})
    public DfsMotif clone() {

        DfsMotif nc = new DfsMotif();
        nc.code.addAll(this.code);
        nc.labels = (Vector<Character>) this.labels.clone();


        return nc;
    }


    /**
     * Extension from the right most path to a new nodes, returns a new code
     *
     * @param fromNodeNr The nodes on the righ-most path we're about to extend from
     * @param toCluster  What kind of nodes the new one is
     */
    public DfsMotif addForward(int fromNodeNr, char toCluster) {
        // add edge to code
        int toNodeNr = labels.size();
        DfsMotif newMotif = this.clone();
        newMotif.labels.add(toCluster);
        DfsCodeEdge newEdge = new DfsCodeEdge(fromNodeNr, toNodeNr, labels.get(fromNodeNr), toCluster);
        int ix = Collections.binarySearch(newMotif.code, newEdge);
        //System.out.println(ix);
        if (ix < 0) {
            newMotif.code.add(-ix - 1, newEdge);
        } else {
            System.err.println("############## warning dfsmotif added equal edge.");
        }

        return newMotif;
    }

    /**
     * The graph is extended from the right-most vertex to an already labels nodes
     *
     * @param toNodeNr The nodes to add a new edge from the righmost vertex
     */
    private DfsMotif addBackward(int toNodeNr) {
        int lastNodeNr = (labels.size() - 1);
        // System.out.println("DfsCode adding back from " + lastNodeNr + " to " + toNodeNr);
        char fromCluster = labels.get(lastNodeNr);
        char toCluster = labels.get(toNodeNr);
        DfsCodeEdge newEdge = new DfsCodeEdge(lastNodeNr, toNodeNr, fromCluster, toCluster);
        DfsMotif newMotif = this.clone();
        if (code.contains(newEdge)) {
            System.err.println(getClass() + ": Edge er allerede lagt til!");
            return null;
        } else {
            int ix = Collections.binarySearch(newMotif.code, newEdge);
            if (ix < 0) ix = -ix - 1;
            newMotif.code.add(ix, newEdge);
            return newMotif;
        }
    }

    public int compareTo(DfsMotif other) {
        Iterator<DfsCodeEdge> i1 = code.iterator();
        Iterator<DfsCodeEdge> i2 = other.code.iterator();
        while (i1.hasNext() && i2.hasNext()) {
            // verify this for ik = ik+1 for forward edges: done (correct in gspan article=
            int comp = i1.next().compareTo(i2.next());
            if (comp != 0) return comp;
        }
        // equal so far; shortest code is smallest
        if (i1.hasNext()) return 1;
        if (i2.hasNext()) return -1;
        return 0;
    }

    public String toString() {
        StringBuilder sb = new StringBuilder();
        for (DfsCodeEdge e : code) {
            sb.append(e.toString());
            sb.append(" ");
        }
        return sb.toString().substring(0, sb.length() - 1);
    }

// --Commented out by Inspection START (04.06.06 15:38):
//    private String toSortedString() {
//        DfsMotif sorted = this.clone();
//        Collections.sort(sorted.code);
//        return sorted.toString();
//    }
// --Commented out by Inspection STOP (04.06.06 15:38)

    public int hashCode() {
        if (this.hashCode == Integer.MAX_VALUE) {
            StringBuilder sb = new StringBuilder();
            for (DfsCodeEdge edge : code) {
                sb.append(edge.toString());
            }
            this.hashCode = sb.toString().hashCode();
        }
        return this.hashCode;
    }

    public boolean equals(Object other) {
        return this.hashCode() == other.hashCode();
    }

    /**
     * Returns an iterator over a set of elements of type DfsCodeEdge.
     *
     * @return an Iterator.
     */
    public Iterator<DfsCodeEdge> iterator() {
        return code.iterator();
    }

    /**
     * Searching the motif depth-first looking for smaller representations than this.
     *
     * @return Whether this node has the smallest possible code, or not
     */
    public boolean isSmallest() {
        // edge sets
        Map<Integer, Vector<Integer>> neighs = new HashMap<Integer, Vector<Integer>>();
        //Vector<String> edges = new Vector<String>();
        String min = this.labels.get(this.code.get(0).i) >= this.labels.get(code.get(0).j) ?
                this.labels.get(code.get(0).j) + "" + this.labels.get(code.get(0).i) :
                this.labels.get(code.get(0).i) + "" + this.labels.get(code.get(0).j);
        for (DfsCodeEdge edge : this) {
            if (!neighs.containsKey((int) edge.i)) neighs.put((int) edge.i, new Vector<Integer>());
            neighs.get((int) edge.i).add((int) edge.j);
            if (!neighs.containsKey((int) edge.j)) neighs.put((int) edge.j, new Vector<Integer>());
            neighs.get((int) edge.j).add((int) edge.i);
            String i = this.labels.get(edge.i) >= this.labels.get(edge.j) ?
                    this.labels.get(edge.j) + "" + this.labels.get(edge.i) :
                    this.labels.get(edge.i) + "" + this.labels.get(edge.j);
            if (i.compareTo(min) < 0) min = i;
        }
        for (Map.Entry<Integer, Vector<Integer>> entry : neighs.entrySet()) {
            Collections.sort(entry.getValue(), new Comparator<Integer>() {
                public int compare(Integer o1, Integer o2) {
                    return labels.get(o1) - labels.get(o2);
                }
            });
        }

        //int smallestLblIx = neighs.firstKey();
        char firstLabel = min.charAt(0);
        char secondLabel = min.charAt(1);
        for (Integer firstIx : neighs.keySet()) {
            if (firstLabel != this.labels.get(firstIx)) continue;
            for (int neighIx : neighs.get(firstIx)) {
                if (labels.get(neighIx) != secondLabel) continue;
                Set<Integer> seen = new HashSet<Integer>();
                seen.add(firstIx);
                seen.add(neighIx);
                DfsCodeEdge2 ce2 = new DfsCodeEdge2(0, 1, firstLabel, secondLabel);
                List<DfsCodeEdge2> code = new Vector<DfsCodeEdge2>();
                code.add(ce2);
                ce2.setNodeIxs(firstIx, neighIx);
                List<DfsCodeEdge2> cmin = recurseCanonical(neighs, code, seen, ce2);
                DfsMotif constructed = new DfsMotif();
                constructed.code = new Vector<DfsCodeEdge>();
                for (DfsCodeEdge2 dfsCodeEdge2 : cmin) {
                    constructed.code.add((DfsCodeEdge) dfsCodeEdge2);
                }
                if (constructed.compareTo(this) < 0)
                    return false;

            }
        }
        return true;

    }

    private List<DfsCodeEdge2>  recurseCanonical(Map<Integer, Vector<Integer>> ix2neighixs,
                                                 List<DfsCodeEdge2> code,

                                                 Set<Integer> seenIxs,
                                                 DfsCodeEdge2 expEdge
    ) {

        // complete motif?
        if (seenIxs.size() == this.labels.size()) {
            return code;
        }

        // recurse unseen branches
        Vector<Integer> neighixs = ix2neighixs.get(expEdge.node2ix);

        for (Integer neighix : neighixs) {
            if (seenIxs.contains(neighix)) continue;

            Set<Integer> newSeen = new HashSet<Integer>(seenIxs);
            newSeen.add(neighix);

            DfsCodeEdge2 newEdge = new DfsCodeEdge2((int) (expEdge.j), (maxJ(code) + 1),
                    this.labels.get(expEdge.node2ix),
                    this.labels.get(neighix));
            newEdge.setNodeIxs(expEdge.node2ix, neighix);

            List<DfsCodeEdge2> newCode = new Vector<DfsCodeEdge2>(code);
            newCode.add(newEdge);

            List<DfsCodeEdge2> min = recurseCanonical(ix2neighixs, newCode, newSeen, newEdge);
            if (min != null) return min;
        }

        // step back for unseen branches
        for (int eix = code.size() - 1; eix >= 0; eix--) {
            DfsCodeEdge2 edge = code.get(eix);
            for (Integer neigh : ix2neighixs.get(edge.node1ix)) {
                if (!seenIxs.contains(neigh)) {
                    Set<Integer> newSeen = new HashSet<Integer>(seenIxs);
                    newSeen.add(neigh);
                    DfsCodeEdge2 newEdge = new DfsCodeEdge2((int) edge.i, (maxJ(code) + 1),
                            this.labels.get(edge.node1ix), this.labels.get(neigh));
                    newEdge.setNodeIxs(edge.node1ix, neigh);
                    List<DfsCodeEdge2> newCode = new Vector<DfsCodeEdge2>(code);
                    newCode.add(newEdge);
                    List<DfsCodeEdge2> min = recurseCanonical(ix2neighixs, newCode, newSeen, newEdge);
                    if (min != null) return min;
                }
            }
        }
        return null;
    }
    /*public boolean isSmallest2() {
      // edge sets
      Map<Integer, Set<Integer>> neighs = new HashMap<Integer, Set<Integer>>();
      for (DfsCodeEdge edge : this) {
          if (!neighs.containsKey((int) edge.i)) neighs.put((int) edge.i, new HashSet<Integer>());
          neighs.get((int) edge.i).add((int) edge.j);
          if (!neighs.containsKey((int) edge.j)) neighs.put((int) edge.j, new HashSet<Integer>());
          neighs.get((int) edge.j).add((int) edge.i);
      }
       char smallestChar = this.labels.get(0);
       char firstEdgeChar = this.labels.get(0);
       for(int i =0; i < this.labels.size();i++){
           if(labels.get(i).equals(smallestChar)){
               List<Integer> thisneighs = new Vector<Integer>(neighs.get(i).size());
               for(int j: neighs.get(i))

           }
       }

       return true;
   } */

    /**
     * Helper class for the isSmallest() method.
     */
    final class DfsCodeEdge2 extends DfsCodeEdge {
        private int node1ix;
        private int node2ix;

        public DfsCodeEdge2(int i, int j, char c1, char c2) {
            super(i, j, c1, c2);
        }

        public final void setNodeIxs(int node1ix, int node2ix) {
            this.node1ix = node1ix;
            this.node2ix = node2ix;
        }
    }

    private static int maxJ(List<DfsCodeEdge2> c) {
        int max = 0;
        for (DfsCodeEdge2 dfsCodeEdge : c) {
            if (dfsCodeEdge.j > max) max = dfsCodeEdge.j;
        }
        return max;
    }

    /**
     * Testing the DsfMotif implementation
     *
     * @param args
     */
    public static void main(String[] args) {
        DfsCodeEdge startEdge = new DfsCodeEdge(0, 1, 'a', 'b');
        DfsMotif m2 = new DfsMotif(startEdge);
        m2 = m2.addForward(1, 'a');
        m2 = m2.addForward(2, 'b');
        m2 = m2.addForward(2, 'b');
        m2 = m2.addForward(2, 'b');
        m2 = m2.addForward(2, 'b');
        m2 = m2.addForward(2, 'b');
        m2 = m2.addForward(2, 'b');
        m2 = m2.addForward(2, 'a');

        System.out.println("smallest? " + m2.isSmallest());

    }
}
