package nemi.gspan;

import cytoscape.CyNetwork;
import cytoscape.Cytoscape;

import java.util.*;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;

import giny.model.Node;
import nemi.gui.ViewableResults;
import nemi.compare.ProteinCompare;

import javax.swing.event.ListSelectionEvent;
import javax.swing.event.TreeSelectionEvent;
import javax.swing.*;

/**
 * NeMi - Neighborhood Mining
 * Distributed AS-IS - No Warranty
 * $Id: DfsTableModel.java 530 2006-06-14 17:30:50Z sundsdal $
 */

public class DfsTableModel extends ViewableResults {
    private List<ActionListener> actionListeners = new Vector<ActionListener>();



    private Map<CyNetwork, CyNetwork> expl2orignet;
    private List<gSpanMotif> results;
    private boolean multi;
    private Map<String, Map<CyNetwork, Node>> str2origNet2origNode;
    private int minsup;

    public DfsTableModel(List<DfsResult> freqs, Map<CyNetwork, CyNetwork> netmap, int minsup,
                         boolean multi, Map<String, Map<CyNetwork, Node>> str2origNet2origNode,
                         ProteinCompare comp) {

        this.expl2orignet = netmap;
        this.minsup = minsup;
        this.multi = multi;
        this.str2origNet2origNode = str2origNet2origNode;
        System.out.println("Creating tree...");

        this.results = new Vector<gSpanMotif>();

        for (DfsResult dfsnode : freqs) {
            Map<CyNetwork, Set<Node>> orignet2nodeset = new HashMap<CyNetwork, Set<Node>>();
            Map<CyNetwork, Set<List<Node>>> net2setofnodelist = new HashMap<CyNetwork, Set<List<Node>>>();
            List<OrigInstance> origInstances = new Vector<OrigInstance>();
            //if(dfsnode.motif.toString().startsWith("0,100 1,202 2,320")){
            //    System.out.println("test2");
            //}
            int instcount = 0;
            if (dfsnode.motif.labels.size() < 2) continue;
            for (ResultInstance instance : dfsnode.expFreqs) {
                CyNetwork orignet = expl2orignet.get(instance.net);
                if (!orignet2nodeset.containsKey(orignet))
                    orignet2nodeset.put(orignet, new HashSet<Node>());
                Set<Node> discoveredSet = orignet2nodeset.get(orignet);
                Set<Node> instanceSet = new HashSet<Node>();

                int uniqueCount = 0;
                // ResultInstance f = sub2freq.get(instance);
                for (Node expnode : instance) {
                    CyNetwork origNet = expl2orignet.get(instance.net);
                    String origNetId = expnode.getIdentifier().substring(2);
                    Node orignode = str2origNet2origNode.get(origNetId).get(origNet);
                    if (!discoveredSet.contains(orignode))
                        uniqueCount++;
                    instanceSet.add(orignode);
                }


                if (uniqueCount > instanceSet.size() - 1) {
                    discoveredSet.addAll(instanceSet);
                    if (!net2setofnodelist.containsKey(orignet))
                        net2setofnodelist.put(orignet, new HashSet<List<Node>>());
                    net2setofnodelist.get(orignet).add(new Vector<Node>(instanceSet));
                    origInstances.add(new OrigInstance(orignet, new Vector(instanceSet), instance));
                    instcount++;
                }
            }
            if ((!multi && instcount >= minsup) || (multi && net2setofnodelist.size() >= minsup)) {
                results.add(new gSpanMotif(net2setofnodelist, dfsnode, origInstances, str2origNet2origNode, comp));
            }
        }

        for (CyNetwork expnet : expl2orignet.keySet()) {
            Cytoscape.createNetworkView(expnet);
        }
    }

    public DfsTableModel() {
        results = null;
        //To change body of created methods use File | Settings | File Templates.
    }

    // from AbstractTableModel
    public int getRowCount() {
        if (results == null) return 0;
        return results.size();  //To change body of implemented methods use File | Settings | File Templates.
    }

    public int getColumnCount() {
        return 5;  //To change body of implemented methods use File | Settings | File Templates.
    }

    public Object getValueAt(int rowIndex, int columnIndex) {
        if (results == null) return null;
        if (rowIndex >= results.size()) return null;
        gSpanMotif gm = results.get(rowIndex);
        switch (columnIndex) {
            case 0: //return "Motif";
                return gm.node.motif.toString();
            case 1://              return "Nodes";
                return gm.node.motif.labels.size();
            case 2:  //         return "Matches";
                return gm.countMatches();
            case 3:
                return gm.matchset.size();
            case 4:
                return gm.score; //"Score";
        }
        return "";
        //return null;  //To change body of implemented methods use File | Settings | File Templates.
    }

    public String getColumnName(int columnIndex) {
        switch (columnIndex) {
            case 0:
                return "Motif";
            case 1:
                return "Nodes";
            case 2:
                return "Matches";
            case 3:
                return "Nets";
            case 4:
                return "Score";
        }
        return "";
    }

    public Class<?> getColumnClass(int columnIndex) {
        switch (columnIndex) {
            case 0:
                return String.class;
            case 1:
                return Integer.class;
            case 2:
                return Integer.class;
            case 3:
                return Integer.class;
            case 4:
                return Double.class;
        }
        return null;  //To change body of implemented methods use File | Settings | File Templates.
    }

    // selection changed
    public void valueChanged(ListSelectionEvent e) {
        int index = e.getFirstIndex();
        //if (!lsm.isSelectedIndex(index)) index = lsm.getMaxSelectionIndex();

        if(results == null || index < 0 || index >= results.size()) return;
        final gSpanMotif gm = results.get(index);
        if (gm == null) return;
        javax.swing.SwingUtilities.invokeLater(new Runnable() {
            public void run() {

                // notify listeners
                for (ActionListener actionListener : actionListeners) {
                    actionListener.actionPerformed(new ActionEvent(gm, ActionEvent.ACTION_PERFORMED, "Selection changed"));
                }

                // remove flags for all networks
                //noinspection unchecked
                for (CyNetwork net : (Collection<CyNetwork>) Cytoscape.getNetworkSet()) {
                    //noinspection unchecked
                    Vector<Node> flaggedNodes = new Vector<Node>(net.getFlaggedNodes());
                    net.setFlaggedNodes(flaggedNodes, false);
                }

                try {
                    // flag instance nodes
                    for (Map.Entry<CyNetwork, Set<List<Node>>> entry : gm.matchset.entrySet()) {
                        for (List<Node> nodelist : entry.getValue()) {
                            entry.getKey().setFlaggedNodes(nodelist, true);
                        }
                    }

                    // flag expnet nodes
                    for (ResultInstance instance : gm.node.expFreqs) {
                        //ResultInstance f = new ResultInstance(instance);
                        instance.net.setFlaggedNodes(instance.nodes, true);
                    }

                } catch (Exception e1) {
                    e1.printStackTrace();  //To change body of catch statement use File | Settings | File Templates.
                }
            }
        }

        );

    }

    public void addActionListener(ActionListener a) {
        actionListeners.add(a);
    }

    public void stopRunning() {
        //To change body of implemented methods use File | Settings | File Templates.
    }

    public String getToolTipText(int rowIndex) {
        return null;  //To change body of implemented methods use File | Settings | File Templates.
    }
}
