package nemi.gspan;

import cytoscape.CyNetwork;
import cytoscape.Cytoscape;
import giny.model.Node;

import javax.swing.event.TreeSelectionEvent;
import javax.swing.event.TreeSelectionListener;
import javax.swing.tree.DefaultMutableTreeNode;
import javax.swing.tree.DefaultTreeModel;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.*;

/**
 * Created by IntelliJ IDEA.
 * User: kristost
 * Date: 26.apr.2006
 * Time: 14:34:19
 *
 * TreeMOdel implementation for the DfsResults. Planned to display the results as a tree, but this
 * now works as a list. 
 */
final class DfsTreeModel extends DefaultTreeModel implements TreeSelectionListener, Iterable<DfsTreeNode> {

    //final Collection<DfsTreeNode> rootNodes = new Vector<DfsTreeNode>();
    private final Set<ActionListener> actionListeners = new HashSet<ActionListener>();

    //Map<Node, Map<CyNetwork, Node>> str2expnet2orignod;

    public DfsTreeModel(Collection<DfsResult> freqs, Map<CyNetwork, CyNetwork> expl2orignet, int minsup,
                        boolean multi, Map<String, Map<CyNetwork, Node>> str2origNet2origNode) {
        super(null);
        System.out.println("Creating tree...");

        DfsTreeNode rootNode = new DfsTreeNode(null, null, null, null);
        setRoot(rootNode);

        for (DfsResult dfsnode : freqs) {
            Map<CyNetwork, Set<Node>> orignet2nodeset = new HashMap<CyNetwork, Set<Node>>();
            Map<CyNetwork, Set<List<Node>>> net2setofnodelist = new HashMap<CyNetwork, Set<List<Node>>>();
            List<OrigInstance> origInstances = new Vector<OrigInstance>();
            //if(dfsnode.motif.toString().startsWith("0,100 1,202 2,320")){
            //    System.out.println("test2");
            //}
            int instcount = 0;
            if (dfsnode.motif.labels.size() < 2) continue;
            for (ResultInstance instance : dfsnode.expFreqs) {
                CyNetwork orignet = expl2orignet.get(instance.net);
                if (!orignet2nodeset.containsKey(orignet)) orignet2nodeset.put(orignet, new HashSet<Node>());
                Set<Node> discoveredSet = orignet2nodeset.get(orignet);
                Set<Node> instanceSet = new HashSet<Node>();

                int uniqueCount = 0;
               // ResultInstance f = sub2freq.get(instance);
                for (Node expnode : instance) {
                    CyNetwork origNet = expl2orignet.get(instance.net);
                    String origNetId = expnode.getIdentifier().substring(2);
                    Node orignode = str2origNet2origNode.get(origNetId).get(origNet);
                    if (!discoveredSet.contains(orignode))
                        uniqueCount++;
                    instanceSet.add(orignode);
                }


                if (uniqueCount > instanceSet.size() - 1) {
                    discoveredSet.addAll(instanceSet);
                    if (!net2setofnodelist.containsKey(orignet))
                         net2setofnodelist.put(orignet, new HashSet<List<Node>>());
                    net2setofnodelist.get(orignet).add(new Vector<Node>(instanceSet));
                    origInstances.add(new OrigInstance(orignet, new Vector(instanceSet), instance));
                    instcount++;
                }
            }
            if ((!multi && instcount >= minsup) || (multi && net2setofnodelist.size() >= minsup)) {
                new DfsTreeNode(net2setofnodelist, dfsnode, rootNode, origInstances);
            }
        }
        Collections.sort(rootNode.children);
        Collections.reverse(rootNode.children);
        System.out.println("Done making tree " + (-1 % 3));

        for (CyNetwork expnet : expl2orignet.keySet()) {
            Cytoscape.createNetworkView(expnet);
        }
    }

 
    public DfsTreeModel(){
        super(null);
        root = new DefaultMutableTreeNode();
    }

    public void valueChanged(final TreeSelectionEvent e) {

        javax.swing.SwingUtilities.invokeLater(new Runnable() {
            public void run() {
                     DfsTreeNode treeNode;
                try{
                     treeNode = (DfsTreeNode) e.getPath().getLastPathComponent();
                } catch( ClassCastException e) {
                    return;

                }

                if (treeNode != null) {
                    // notify listeners
                    for (ActionListener actionListener : actionListeners) {
                        actionListener.actionPerformed(new ActionEvent(treeNode, 0, "Selection changed"));
                    }

                    // remove flags for all networks
                    //noinspection unchecked
                    for (CyNetwork net : (Collection<CyNetwork>) Cytoscape.getNetworkSet()) {
                        //noinspection unchecked
                        Vector<Node> flaggedNodes = new Vector<Node>(net.getFlaggedNodes());
                        net.setFlaggedNodes(flaggedNodes, false);
                    }

                    try {
                        // flag instance nodes
                        for (Map.Entry<CyNetwork, Set<List<Node>>> entry : treeNode.matchset.entrySet()) {
                            for (List<Node> nodelist : entry.getValue()) {
                                entry.getKey().setFlaggedNodes(nodelist, true);
                            }
                        }

                        // flag expnet nodes
                        for (ResultInstance instance : treeNode.node.expFreqs) {
                            //ResultInstance f = new ResultInstance(instance);
                            instance.net.setFlaggedNodes(instance.nodes, true);
                        }

                    } catch (Exception e1) {
                        e1.printStackTrace();  //To change body of catch statement use File | Settings | File Templates.
                    }
                }
            }
        });
    }

    public Object getRoot() {

        return root;  //To change body of implemented methods use File | Settings | File Templates.
    }

    public Object getChild(Object parent, int index) {
        if (parent instanceof DfsTreeNode) {
            DfsTreeNode node = (DfsTreeNode) parent;
            return node.children.get(index);
        }
        System.out.println("bad1");
        return null;  //To change body of implemented methods use File | Settings | File Templates.
    }

    public int getChildCount(Object parent) {
        if (parent instanceof DfsTreeNode) {
            DfsTreeNode node = (DfsTreeNode) parent;
            return node.children.size();
        }
        return -1;  //To change body of implemented methods use File | Settings | File Templates.
    }

    public boolean isLeaf(Object node) {
        if (node instanceof DfsTreeNode) {
            DfsTreeNode n = (DfsTreeNode) node;
            return n.isLeaf();
        }
        return false;  //To change body of implemented methods use File | Settings | File Templates.
    }


    public int getIndexOfChild(Object parent, Object child) {
        if (parent instanceof DfsTreeNode && child instanceof DfsTreeNode) {
            DfsTreeNode dfsTreeNode = (DfsTreeNode) parent;
            DfsTreeNode c = (DfsTreeNode) child;
            return dfsTreeNode.getIndex(c);
        }
        return -1;  //To change body of implemented methods use File | Settings | File Templates.
    }

    /**
     * Returns an iterator over a set of elements of type T.
     *
     * @return an Iterator.
     */
    public Iterator<DfsTreeNode> iterator() {
        DfsTreeNode rootNode = (DfsTreeNode)root;
        return rootNode.children.iterator();
        //return null;  //To change body of implemented methods use File | Settings | File Templates.
    }
}
