package nemi.gspan;

import cytoscape.CyNetwork;
import giny.model.Node;
import nemi.cluster.*;
import nemi.compare.ProteinCompare;
import nemi.compare.gui.ProteinComparerTable;
import nemi.gui.*;
import nemi.Toolkit;
import nemi.apriori.GraphMotif;
import nemi.apriori.AprioriInstance;
import nemi.apriori.Apriori;

import javax.swing.*;
import javax.swing.table.TableColumn;
import javax.swing.event.TableModelEvent;
import javax.swing.event.TableModelListener;
import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.text.DecimalFormat;
import java.util.*;
import java.util.List;
import java.io.*;
import java.io.IOException;
import java.io.File;

/**
 * Created by IntelliJ IDEA.
 * User: kristost
 * Date: 03.apr.2006
 * Time: 14:47:52
 * <p/>
 * GUI for the NeMi gSpan implementation.
 */
public final class GspanPanel extends JSplitPane {
                   TableSorter oldts;
    //JSplitPane splitter = new JSplitPane();
    private List<DfsResult> results = null;
    private final NetworkSelector networkSel = new NetworkSelector();
    private final JLabel networkLabel = new JLabel();
    private final ProteinComparerTable protTable = new ProteinComparerTable();
    private final JButton startMineButton = new JButton();
    private final JButton cancelButton = new JButton();
    private final JLabel proteinCompLabel = new JLabel();
    private final JPanel controlPanel = new JPanel();
    private final JScrollPane allScroller = new JScrollPane();
    private final JScrollPane resultScroller = new JScrollPane();
    private final ResultDetailPanel resultDetails = new ResultDetailPanel();
    private final JSplitPane bottomSplitPanel = new JSplitPane();
    private SliderWrapper clusteringLimitSlider;
    private SliderWrapper minsupSlider;
    private final JLabel minsupLabel = new JLabel();
    private final JScrollPane networkScroller = new JScrollPane();
    private final JLabel clusteringLimitLabel = new JLabel();
    private final JTable resultTable = new JTable();
    //private final ClusterNodeDistancePanel clusterNodeDistPanel = new ClusterNodeDistancePanel();
    //private final JLabel clusterNodeDistLabel = new JLabel();
    private JButton saveButton;
    private JButton statisticsButton;
    private final NeMiFinalView.StatusInfo status;
    private double oldLimit;
    private final JCheckBox joinSingleClustersCheckbox = new JCheckBox();
    private JComboBox clusteringList;
    private ClustererList clusterListModel;
    private final JLabel clusteringResultLabel = new JLabel();
    private JComboBox maxsizeCombo;
    private JComboBox overlapCombo;

    private class MySpinner extends AbstractSpinnerModel {
        int i;
        int[] array;

        public MySpinner(int[] array) {
            i = 0;
            this.array = array;

        }

        public Object getValue() {
            return array[i];  //To change body of implemented methods use File | Settings | File Templates.
        }

        public void setValue(Object value) {
            for (int j = 0; j < array.length; j++) {
                if (value.equals(array[j])) {
                    i = j;
                    break;
                }
            }

            fireStateChanged();
            if (i == array.length - 1) allScroller.repaint();
        }

        public Object getNextValue() {
            if (i == array.length - 1) return null;
            return array[i + 1];  //To change body of implemented methods use File | Settings | File Templates.
        }

        public Object getPreviousValue() {
            if (i == 0) return null;
            return array[i - 1];  //To change body of implemented methods use File | Settings | File Templates.
        }
    }

    public GspanPanel(NeMiFinalView.StatusInfo status) {
        this.status = status;
        javax.swing.SwingUtilities.invokeLater(new Runnable() {
            public void run() {
                initComponents();
            }
        });

    }

    private void initComponents() {
        this.setOrientation(JSplitPane.VERTICAL_SPLIT);
        this.setTopComponent(allScroller);
        this.setBottomComponent(bottomSplitPanel);
        this.setDividerLocation(270);

        bottomSplitPanel.setOrientation(JSplitPane.HORIZONTAL_SPLIT);
        bottomSplitPanel.setLeftComponent(resultScroller);
        bottomSplitPanel.setRightComponent(resultDetails);
        bottomSplitPanel.setDividerLocation(250);

        allScroller.setViewportView(controlPanel);
        GridBagLayout gbl = new GridBagLayout();

        controlPanel.setLayout(gbl);

        GridBagConstraints gridBagConstraints;
        networkLabel.setText("Select network");
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 0;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
        controlPanel.add(networkLabel, gridBagConstraints);

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 1;
        gridBagConstraints.gridy = 0;
        gridBagConstraints.gridwidth = 2;
        //gridBagConstraints.fill = GridBagConstraints.BOTH;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
        networkScroller.setViewportView(networkSel);

        networkScroller.setPreferredSize(new Dimension(400, 80));

        controlPanel.add(networkScroller, gridBagConstraints);

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 1;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
        proteinCompLabel.setText("Compare method");
        controlPanel.add(proteinCompLabel, gridBagConstraints);

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 1;
        gridBagConstraints.gridy = 1;
        gridBagConstraints.gridwidth = 3;
        //gridBagConstraints.insets = new Insets(2, 10, 2, 2);
        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
        JScrollPane protcompscroll = new JScrollPane();
        protcompscroll.setViewportView(protTable);
        protcompscroll.setPreferredSize(new Dimension(400, 80));
        controlPanel.add(protcompscroll, gridBagConstraints);

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 2;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
        clusteringLimitLabel.setText("Cluster tolerance");
        controlPanel.add(clusteringLimitLabel, gridBagConstraints);

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 1;
        gridBagConstraints.gridy = 2;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
        clusteringLimitSlider = new SliderWrapper(0, 100, 100, false);
        controlPanel.add(clusteringLimitSlider, gridBagConstraints);
        joinSingleClustersCheckbox.setSelected(true);
        gridBagConstraints = new GridBagConstraints();
        gridBagConstraints.gridx = 2;
        gridBagConstraints.gridy = 2;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.EAST;
        controlPanel.add(joinSingleClustersCheckbox, gridBagConstraints);
        joinSingleClustersCheckbox.setText("Join single clusters");

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 2;
        gridBagConstraints.gridy = 3;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
        controlPanel.add(new JLabel("Max size"), gridBagConstraints);
        //maxsizeSpinner = new JSpinner(new MySpinner(new int[]{2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20, Integer.MAX_VALUE}));
        maxsizeCombo = new JComboBox(new Object[]{2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20, "Max"});
        maxsizeCombo.setSelectedIndex(maxsizeCombo.getItemCount() - 1);
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 3;
        gridBagConstraints.gridy = 3;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
        controlPanel.add(maxsizeCombo, gridBagConstraints);


        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 2;
        gridBagConstraints.gridy = 4;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
        controlPanel.add(new JLabel("Overlaps"), gridBagConstraints);
        overlapCombo = new JComboBox(new Object[]{0, 1, 2, 3, 4, 5, "Max"});
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 3;
        gridBagConstraints.gridy = 4;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
        controlPanel.add(overlapCombo, gridBagConstraints);


        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 4;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
        minsupLabel.setText("Minimum support");
        controlPanel.add(minsupLabel, gridBagConstraints);

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 1;
        gridBagConstraints.gridy = 4;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
        minsupSlider = new SliderWrapper(2, 50, 1, true);
        minsupSlider.setValue(2);
        controlPanel.add(minsupSlider, gridBagConstraints);
        /*clusterNodeDistLabel.setText("Tolerance type");
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 3;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
        controlPanel.add(clusterNodeDistLabel, gridBagConstraints);
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 1;
        gridBagConstraints.gridy = 3;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
        controlPanel.add(clusterNodeDistPanel, gridBagConstraints);
         */

        JPanel buttonPanel = new JPanel(new FlowLayout(FlowLayout.RIGHT, 10, 0));
        //buttonPanel.setPreferredSize(new Dimension(200, 25));

        cancelButton.setText("Cancel");
        cancelButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                cancelButtonActionPerformed(evt);
            }
        });
        cancelButton.setEnabled(false);

        startMineButton.setText("Start");

        startMineButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                startGspanButton(evt);
            }
        });
        startMineButton.setEnabled(false);
        buttonPanel.add(startMineButton);
        buttonPanel.add(cancelButton);
        statisticsButton = new JButton("Statistics");
        statisticsButton.setEnabled(false);
        statisticsButton.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                statButtonPressed(e);
            }

        });
        buttonPanel.add(statisticsButton);

        saveButton = new JButton("Save results");
        saveButton.setEnabled(false);
        saveButton.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                if (results != null) {
                    StringBuffer sb = new StringBuffer();
                    for (DfsResult resultNode : results) {
                        sb.append(resultNode.toStringDetailed()).append("\n");
                    }
                    nemi.Toolkit.saveToFile(sb);
                }
            }
        });
        buttonPanel.add(saveButton);
        protTable.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                updateStartButton();
            }
        });
        networkSel.getModel().addTableModelListener(new TableModelListener() {
            public void tableChanged(TableModelEvent e) {
                int count = networkSel.getSelected().size();
                if (count < 2) {
                    minsupSlider.setMax(50);
                } else {
                    minsupSlider.setMax(count);
                    //minsupSlider.setValue(count);
                }
                minsupSlider.repaint();
                updateStartButton();
            }
        });

        resultScroller.setViewportView(resultTable);
        //resultScroller.setHorizontalScrollBarPolicy(JScrollPane.HORIZONTAL_SCROLLBAR_ALWAYS);
        resultTable.setShowGrid(false);
        resultTable.setShowHorizontalLines(true);
        resultTable.setAutoResizeMode(JTable.AUTO_RESIZE_OFF);
        this.setDividerLocation(270);

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 1;
        gridBagConstraints.gridy = 6;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
        controlPanel.add(buttonPanel, gridBagConstraints);

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 3;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
        controlPanel.add(new JLabel("Clustering method"), gridBagConstraints);
        clusteringList = new JComboBox();
        clusterListModel = new ClustererList();
        clusteringList.setModel(clusterListModel);
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 1;
        gridBagConstraints.gridy = 3;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
        controlPanel.add(clusteringList, gridBagConstraints);
        clusteringList.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                clusteringListActionPerformed(e);
            }
        });
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 7;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
        controlPanel.add(new JLabel("Clustering info:"), gridBagConstraints);
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 1;
        gridBagConstraints.gridy = 7;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
        controlPanel.add(clusteringResultLabel, gridBagConstraints);
    }

    private void statButtonPressed(ActionEvent e) {

        StatFrame stats = new StatFrame(lastclusters, lastcomparer, lastNets, oldLimit);
        stats.setVisible(true);
    }

    public void updateStartButton() {
        if (networkSel.getSelected().size() > 0) {

            startMineButton.setEnabled(clusterListModel.requiresProteincomparers() ? protTable.isReady() : true);
        } else {
            startMineButton.setEnabled(false);
        }

    }

    private DfsTableModel resmodel;

    private ProteinCompare lastcomparer;

    private void clusteringListActionPerformed(java.awt.event.ActionEvent evt) {
        protTable.setEnabled(clusterListModel.requiresProteincomparers());
        clusteringLimitSlider.setEnabled(clusterListModel.requiresLimit());
        updateStartButton();
    }

    private int getOverlap() {
        Object o = overlapCombo.getSelectedItem();
        if (o instanceof Integer) {
            Integer i = (Integer) o;
            return i;
        }
        return Integer.MAX_VALUE;
    }

    private int getMaxSize() {
        Object o = maxsizeCombo.getSelectedItem();
        if (o instanceof Integer) {
            Integer i = (Integer) o;
            return i;
        }
        return Integer.MAX_VALUE;

    }

    private Collection<Cluster> lastclusters;

    private Collection<CyNetwork> lastNets;

    private GSpanMining lastGspan;

    @SuppressWarnings({"UnnecessaryLocalVariable"})
    private void startGspanButton(ActionEvent evt) {
        status.setIndeterminate(true);
        new Thread() {
            public void run() {

                DecimalFormat df = new DecimalFormat("#.##");
                List<CyNetwork> networks = networkSel.getSelected();
                lastNets = networks;
                //Vector<CyNetwork> nets = new Vector<CyNetwork>();
                //nets.add(network);
                if (networks.size() < 1) {
                    System.out.println("No networks selected");
                    status.setStatusInfo("No networks selected");
                    status.setIndeterminate(false);
                    return;
                }
                if (clusterListModel.requiresProteincomparers() && !protTable.isReady()) {
                    System.out.println("Protein comparer must be selected");
                    status.setStatusInfo("Protein comparer must be selected");
                    status.setIndeterminate(false);
                    return;
                }
                statisticsButton.setEnabled(false);
                startMineButton.setEnabled(false);
                status.setStatusInfo("Creating comparer...");
                ProteinCompare comp = protTable.getComparer(networks);
                //clusterTest(networks, comp);
               // if(true){
               // return;
               // }
                lastcomparer = comp;
                status.setStatusInfo("Clustering...");


                double limit = 1 - clusteringLimitSlider.getValue();
                oldLimit = limit;
                Collection<Cluster> clusters = clusterListModel.cluster(networks, comp,
                        QualityAssesser.CompareClusterNodeType.MIN_REP,
                        QualityAssesser.CompareClusterType.AVERAGE_REP, limit,
                        joinSingleClustersCheckbox.isSelected());
                Cluster single = null;
                if(clusters==null){
                    System.out.println("No clusters");
                    status.setStatusInfo("Error. No clusters created");
                    status.setIndeterminate(false);
                    return;

                }
                int size = 0;
                Set<Node> nodeset = new HashSet<Node>();
                for (Cluster cluster : clusters) {
                    size += cluster.size();
                    nodeset.addAll(cluster.getRepNodes());
                    if (cluster.id == '?') {
                        single = cluster;
                        //break;
                    }
                }
                int nodecount = nodeset.size();
                String clusterReusult = clusters.size() + " clusters. Nodes/cluster: " + df.format((double)size/(double)clusters.size()) + ". Clusters/node:  " + df.format(((double) size) / (double) nodecount) + ".";
                clusteringResultLabel.setText(clusterReusult);
                clusteringResultLabel.repaint();
                nodeset = null;
                //Collection<Cluster> clusters = IterativeClusterer.cluster(networks, comp,
                //        clusterNodeDistPanel.getClusterNodeDistanceType(), clusteringLimitSlider.getValue(), joinSingleClustersCheckbox.isSelected());
                lastclusters = clusters;
                statisticsButton.setEnabled(true);
                status.setStatusInfo("Running gSpan on " + networks.size() + " networks");
                int maxsize = getMaxSize();
                int overlaps = getOverlap();
                GSpanMining miner = new GSpanMining(status, overlaps);
                lastGspan = miner;

                cancelButton.setEnabled(true);
                int minsup = (int) minsupSlider.getValue();
                HashMap<CyNetwork, CyNetwork> netmap = new HashMap<CyNetwork, CyNetwork>();
                if (networks.size() >= 1) {
                    // comp = null;
                    // todo: testing
                    // multiminingTest(networks, miner, clusters, netmap);

                    System.gc();
                    resultTable.getSelectionModel().removeListSelectionListener(oldts);
                    Map<String, Map<CyNetwork, Node>> str2origNet2origNode = new HashMap<String, Map<CyNetwork, Node>>();
                    CyNetwork network = networks.iterator().next();

                    if (networks.size() == 1) {
                        results = miner.subGraphMining(clusters, network, minsup, netmap, str2origNet2origNode, maxsize);
                    } else {
                        results = miner.subGraphMiningMulti(clusters, networks, minsup, netmap, str2origNet2origNode, maxsize);

                        // aprioriGspanTiming();
                        // gSpanMotifFinderCompTest();
                    }

                    cancelButton.setEnabled(false);
                    status.setStatusInfo("gSpan done.");

                    if (results != null && results.size() > 0) {
                        saveButton.setEnabled(true);
                        if (joinSingleClustersCheckbox.isSelected() && single != null) {

                            clusterReusult += " Wild-card is '" + single.id + "'";
                        }
                        clusteringResultLabel.setText(clusterReusult);
                        clusteringResultLabel.repaint();
                        System.err.println("DFSNODES " + results.iterator().next());
                        resmodel = new DfsTableModel(results, netmap, minsup, networks.size() != 1, str2origNet2origNode, comp);

                        //aprioriTest(clusters, network, overlaps, minsup, maxsize, comp, results);

                        //testCorrectNess(str2origNet2origNode, networks, results);
                        resultTable.getSelectionModel().setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
                        resmodel.addActionListener(resultDetails);

                        //resultTable.getSelectionModel().addListSelectionListener(resmodel);
                        SwingUtilities.invokeLater(new Runnable() {
                            public void run() {
                                oldts = new TableSorter(resmodel);
                                oldts.setTableHeader(resultTable.getTableHeader());
                                resultTable.setModel(oldts);
                                //resultTable.setEditable(false);
                                //resultTable.setRootVisible(false);

                                updateColumWidths();
                                resultTable.getSelectionModel().addListSelectionListener(oldts);
                                resultTable.revalidate();
                                resultTable.repaint();
                            }


                        });
                    } else {
                        resmodel = new DfsTableModel();
                        resultDetails.clear();
                        resultDetails.repaint();
                        SwingUtilities.invokeLater(new Runnable() {
                            public void run() {
                                resultTable.setModel(resmodel);
                                resultTable.getSelectionModel().addListSelectionListener(resmodel);
                                resultTable.revalidate();
                                resultTable.repaint();
                                resultTable.revalidate();
                                resultTable.repaint();
                            }
                        });
                        status.setStatusInfo("gSpan return no results");
                        System.out.println("No results");
                    }

                } else {
                    System.out.println("WOW! You're good.");
                }
                System.runFinalization();
                System.gc();
                updateStartButton();
            }


        }.start();
    }

    private void multiminingTest(List<CyNetwork> networks, GSpanMining miner, Collection<Cluster> clusters, Map<CyNetwork, CyNetwork> netmap) {
        System.out.println("\n##############\nSUBGAPHMINING MULTI GSPAN TEST\n##############");
        CyNetwork nnet, motiff;
        if (networks.get(0).getNodeCount() > networks.get(1).getNodeCount()) {
            nnet = networks.get(0);
            motiff = networks.get(1);
        } else {
            motiff = networks.get(0);
            nnet = networks.get(1);
        }
        int subgraphsize = 20;
        int nsubgraphs = 6;
        int plantedin = 3;
        List<CyNetwork> randnets = miner.generateRandomNets(nnet, nsubgraphs, subgraphsize);
        randnets = miner.insertSubgraph(nnet, randnets, motiff, plantedin);
        Map<String, Map<CyNetwork, Node>> mmmap = new HashMap<String, Map<CyNetwork, Node>>();
        results = miner.subGraphMiningMulti(clusters, randnets, plantedin, netmap, mmmap, 100);
        StringBuilder gspanresults = new StringBuilder();
        for (DfsResult resultNode : results) {
            gspanresults.append(resultNode.toStringDetailed()).append("\n");
        }
        File saveto = new File("gspan_multiplenets");
        try {

            FileWriter out = new FileWriter(saveto);
            out.write("#running multiple nets gspan " +
                    nnet.getTitle() + "" +
                    " and " + motiff.getTitle() + "\n" +
                    ", subsize: " + subgraphsize + ", nsubs: " + nsubgraphs + "" +
                    ", plantedin: " + plantedin);
            out.write("\n\n#Gspan results\n\n");
            out.write(gspanresults.toString());
            out.close();
            System.out.println("#############\nSaved results to " + saveto + "\n##############");
        } catch (IOException e) {
            System.out.println("Couldn't write to " + saveto);
        }
    }

    private void clusterTest(Collection<CyNetwork> networks, ProteinCompare comp) {
        /* clusteringtest*/
        List<Cluster> clusters;
        IterativeClusterer cls = new IterativeClusterer();
        for(int i = 0; i <= 20; i ++){
            clusters = cls.cluster(networks, comp,
                QualityAssesser.CompareClusterNodeType.MIN_REP,
                i/20.0, true);
            Set<Node> nodeset = new HashSet<Node>();
            int size = 0;
            for (Cluster cluster : clusters) {
                size += cluster.size();
                nodeset.addAll(cluster.getRepNodes());
            }

            System.out.println("join, "+ (1.0- (i/20.0)) + ", "+ clusters.size() + ", " +  size / (double) nodeset.size() + ", " + nodeset.size()/(double)clusters.size());
        }
        for(int i = 0; i <= 20; i ++){

            clusters = cls.cluster(networks, comp,
                QualityAssesser.CompareClusterNodeType.MIN_REP,
                i/20.0, false);
            Set<Node> nodeset = new HashSet<Node>();
            int size = 0;
            for (Cluster cluster : clusters) {
                size += cluster.size();
                nodeset.addAll(cluster.getRepNodes());
            }

            System.out.println("notjoin, "+ (1.0-(i/20.0)) + ", "+ clusters.size() + ", " +  size / (double) nodeset.size() + ", " + nodeset.size()/(double)clusters.size());
        }

        /* end clusteringtest*/
    }

    private void aprioriGspanTiming() {
        /* Kristoffertest timing
        CyNetwork nnet, motiff;
        if (networks.get(0).getNodeCount() > networks.get(1).getNodeCount()) {
            nnet = networks.get(0);
            motiff = networks.get(1);
        } else {
            motiff = networks.get(0);
            nnet = networks.get(1);
        }
        new Tester(nnet, motiff, clusters, comp, status);
        return;
         //results = miner.subGraphMiningMulti(clusters, networks, minsup, netmap, str2origNet2origNode);
        */
    }

    private void gSpanMotifFinderCompTest() {
        /* START MOTIFFINDER GSPAN COMP */
/*

            CyNetwork nnet, motiff;
            if (networks.get(0).getNodeCount() > networks.get(1).getNodeCount()) {
                nnet = networks.get(0);
                motiff = networks.get(1);
            } else {
                motiff = networks.get(0);
                nnet = networks.get(1);
            }
            AssignmentResults motifFinderRes =
                    new AssignmentResults(9000,
                    resultDetails.getMessagePane());
            MotifFinder mf = new MotifFinder(nnet, motiff, comp, 0.0,
                    new MotifFinderPanel(status), motifFinderRes, status);

            while(mf.isRunning()){
             int a = 3;  // !!
            }

            // save results
            StringBuffer gspanresults = new StringBuffer();
            for (DfsResult resultNode : results) {
                gspanresults.append(resultNode.toStringDetailed()).append("\n");
            }
            File saveto = new File("gspan_motiffinder_comp");
            try {

                FileWriter out = new FileWriter(saveto);
                out.write("#running gspan and motiffinder on " +
                        nnet.getTitle() + "" +
                        " and " + motiff.getTitle() + "\n");
                out.write("\n\n#Gspan results\n\n");
                out.write(gspanresults.toString());
                out.write("\n\n#Motiffinder results\n\n");
                out.write(motifFinderRes.toString());
                out.close();
                System.out.println("Saved results to " + saveto);
            } catch (IOException e) {
                System.out.println("Couldn't write to " + saveto);
            }


/* END MOTIFFINDER GSPAN COMP */
    }

    private static void aprioriTest(Collection<Cluster> clusters, CyNetwork network, int overlaps, int minsup, int maxsize, ProteinCompare comp, List<DfsResult> results) {
        /*Apriori testing section*/

        try {
            Apriori apriori = new Apriori(clusters, network, overlaps);

            // todo: apriori should update a gui status bas
            System.out.println("Running Apriori");
            Map<GraphMotif, Vector<AprioriInstance>> aprioriresults = apriori.apply(maxsize, minsup);

            OutputStream os = new FileOutputStream("miningdata.dat");
            Writer osw = new OutputStreamWriter(os, "UTF-8")   ;
            BufferedWriter w = new BufferedWriter(osw);//new FileWriter("miningdata.dat"));
            w.write("#Results from gSpan and Apriori run \n");
            w.write("#Network: " + network.getIdentifier() + "\n");
            w.write("#Minsup: " + minsup + ", ");
            w.write("#Max motifsize: " + maxsize + "\n");
            w.write("Comparer: " + comp.toString());
            w.write("gSpan results: \n");

            for (DfsResult n : results) {
                // if(n.node.motif.labels.size()!=maxsize) continue;
                w.write(n.motif.toString() + "\n");
                //w.write(n.node.motif.labels.toString());
                w.newLine();
                int i = 0;
                for (ResultInstance gi : n.expFreqs) {
                    w.write("Instance " + i++ + ": ");
                    for (Node node : gi.nodes) {
                        w.write(node.getIdentifier() + " ,");
                    }
                    w.newLine();

                }
                w.newLine();
            }
            w.write("########\nApriori results:\n");
            System.out.println("Apirori results: " + aprioriresults.keySet().iterator().next().nodeCount());

            for (Map.Entry<GraphMotif, Vector<AprioriInstance>> entry : aprioriresults.entrySet()) {


                w.write("Motif: " + entry.getKey().toString());
                w.newLine();

                int i = 0;
                for (AprioriInstance sg : entry.getValue()) {

                    w.write("Instance " + i++ + ": ");
                    for (Node n : sg.subGraph) {
                        w.write(n.getIdentifier() + " ,");
                    }
                    w.newLine();
                }
                w.newLine();

            }
            w.write("######\nClusters\n");
            for (Cluster c : clusters) {
                w.write("Cluster " + c.id + ":");
                for (Node n : c.getRepNodes()) {
                    w.write(n.getIdentifier());
                    w.write(", ");
                }
                w.newLine();
            }
            w.close();
        } catch (Exception e) {
            e.printStackTrace();
        }

        /* END APRIORI SECTION*/
    }

    private static void testCorrectNess(Map<String, Map<CyNetwork, Node>> str2origNet2origNode, List<CyNetwork> networks, List<DfsResult> results) {
        boolean errors = false;
        for (DfsResult n : results) {
            // if(n.node.motif.labels.size()!=maxsize) continue;

            //w.write(n.node.motif.labels.toString());

            int i = 0;
            for (ResultInstance gi : n.expFreqs) {
                for (DfsCodeEdge edge : n.motif) {
                    Node expln1 = gi.nodes.get(edge.i);
                    Node expln2 = gi.nodes.get(edge.j);

                    Node origNode1 = str2origNet2origNode.get(
                            expln1.getIdentifier().substring(2)).get(networks.get(0));
                    Node origNode2 = str2origNet2origNode.get(
                            expln2.getIdentifier().substring(2)).get(networks.get(0));
                    Collection<Node> neighs = Toolkit.
                            getNodeNeighborMap(networks.get(0)).get(origNode1);
                    if (!neighs.contains(origNode2)) {
                        System.out.println("Error in " + n.motif.toString());
                        System.out.println(gi.toString());
                        System.out.println(edge.toString());
                        errors = true;
                    }

                }

            }

        }
        if (!errors) System.out.println("No errors");

    }

    private void updateColumWidths() {
        //resultScroller.repaint();
        int visibleWidth = (int) resultScroller.getViewport().getExtentSize().getWidth();
        int visibleHeight = (int) resultScroller.getViewport().getExtentSize().getHeight();
        int totalHeight = (int) resultScroller.getViewport().getViewSize().getHeight();
        if (visibleHeight < totalHeight) {
            if (resultScroller.getVerticalScrollBar() != null) {
                int i = resultScroller.getVerticalScrollBar().getWidth();
                if (i < 10) {
                    visibleWidth -= 20;
                } else {
                    visibleWidth -= i;
                }
            }
        }
        int unit = visibleWidth / (resultTable.getColumnModel().getColumnCount() + 2);
        for (int i = 0; i < resultTable.getColumnModel().getColumnCount(); i++) {
            TableColumn column = resultTable.getColumnModel().getColumn(i);
            int width = i == 0 ? unit * 3 : unit;
            column.setPreferredWidth(width);
        }
    }

    private void cancelButtonActionPerformed
            (ActionEvent
                    evt) {
        lastGspan.setRunning(false);
        cancelButton.setEnabled(false);
        status.setIndeterminate(false);
        status.setMinimum(0);
        status.setValue(0);
        status.setProgressText("");
        status.setStatusInfo("gSpan canceled");
        status.repaint();
    }


}
