package nemi.gspan;

import cytoscape.CyNetwork;
import giny.model.Node;
import nemi.gui.MessagePane;

import javax.swing.*;
import javax.swing.event.ListDataListener;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;
import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

/**
 * Created by IntelliJ IDEA.
 * User: kristost
 * Date: 24.mai.2006
 * Time: 12:04:07
 *
 * Show details for a gSpan motif. For a selected motif, show its instances, and when an instance
 * is clicked, show its nodes.
 *
 */
final class ResultDetailPanel extends JSplitPane implements ActionListener {
    private final JScrollPane instanceScroller = new JScrollPane();
    private final JList instanceList = new JList();
    private final MessagePane textPane = new MessagePane();

    public ResultDetailPanel() {
        this.setLeftComponent(instanceScroller);
        instanceScroller.setViewportView(instanceList);
        this.setRightComponent(textPane);
        this.setDividerLocation(150);
        instanceList.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);


        this.instanceList.setCellRenderer(new DefaultListCellRenderer() {
            //private JLabel label = new JLabel();
            public Component getListCellRendererComponent(JList list, Object value, int index, boolean isSelected, boolean cellHasFocus) {
                JLabel label = (JLabel) super.getListCellRendererComponent(list, value, index, isSelected, cellHasFocus);
                OrigInstance node = (OrigInstance) value;
                label.setText(node.toString());
                return label;
            }
        });
    }

    private static ListSelectionListener oldListener = null;

    public MessagePane getMessagePane(){
        return this.textPane;
    }

    public void actionPerformed(ActionEvent e) {
        if (e.getActionCommand().equals("Selection changed")) {
            if (e.getSource() instanceof gSpanMotif) {
                gSpanMotif node = (gSpanMotif) e.getSource();

                final InstanceModel m = new InstanceModel(node);
                instanceList.setModel(m);
                if (oldListener != null) {
                    instanceList.getSelectionModel().removeListSelectionListener(oldListener);
                }
                instanceList.getSelectionModel().addListSelectionListener(
                        oldListener = new ListSelectionListener() {
                            public void valueChanged(ListSelectionEvent e) {
                                if (!e.getValueIsAdjusting()) {
                                    ListSelectionModel lsm = (ListSelectionModel) e.getSource();
                                    int index = e.getFirstIndex();
                                    if (!lsm.isSelectedIndex(index)) {
                                        index = e.getLastIndex();
                                    }
                                    OrigInstance i = (OrigInstance) m.getElementAt(index);
                                    i.flagNet();
                                    textPane.print(i.toStringDetailed());
                                }
                            }
                        });

            } else System.out.println("unknown event source");
        } else System.out.println("unknown event");
    }


    public final void clear() {
        instanceList.setModel(new DefaultListModel());
        instanceList.repaint();

    }
}

/**
 *
 * Helper class linking an gSpan subgraph instance in the exploded, labeled network to an original instance
 * in the original network.
 *
 */
final class OrigInstance {

    private final CyNetwork net;
    CyNetwork expnet;
    private final List<Node> nodes;
    private final ResultInstance expInstance;


    public OrigInstance(CyNetwork net, List<Node> nodes, ResultInstance expInstance) {
        this.net = net;
        this.expInstance = expInstance;
        this.nodes = nodes;
    }

    public final String toString() {
        String netTitle = nemi.Toolkit.networkToString(net);
        return netTitle + ", " + nodes.size() + " nodes";
    }

    public final String toStringDetailed() {
        String ret = toString() + "\n";
        for (Node node : expInstance) {
            ret += node.getIdentifier() + "\n";
        }
        return ret + "\n";
    }

    public final void flagNet() {
        Set<Node> flagged = new HashSet<Node>(net.getFlaggedNodes());
        if (flagged.size() > 0)
            net.setFlaggedNodes(flagged, false);
        net.setFlaggedNodes(nodes, true);

        expInstance.flagNet();

    }

    public int getNodeCount(){
        return this.nodes.size();
    }

    public ResultInstance getExpInstance(){
        return this.expInstance;
    }

    public CyNetwork getNetwork() {
        return this.net;
    }
}

final class InstanceModel implements ListModel {

    private final List<OrigInstance> instances;

    public InstanceModel(gSpanMotif node) {
        this.instances = node.origInstances;
    }

    public final int getSize() {
        return instances.size();
    }

    public final Object getElementAt(int index) {
        return instances.get(index);
    }

    public final void addListDataListener(ListDataListener l) {}

    public final void removeListDataListener(ListDataListener l) {}
}
