package nemi.gspan;

import cytoscape.CyNetwork;
import giny.model.Node;

import java.util.*;

/**
 * An instance of a dfs node in an exploded/labeled network. Similar to SubgraphInstance,
 * only Resultinstance saves a complete list of nodes, not just the last as in Subgraph-
 * instance.
 *
 * Created by IntelliJ IDEA.
 * User: kristost
 * Date: 05.mai.2006
 * Time: 15:09:22
 * 
 */
@SuppressWarnings({"ForLoopReplaceableByForEach"})
final class ResultInstance implements Iterable<Node> {
    public final List<Node> nodes;
    public final CyNetwork net;

    public ResultInstance(SubgraphInstance parent) {
        this.net = parent.net;
        this.nodes = new Vector<Node>();
        SubgraphInstance temp = parent;
        while (temp != null) {
            nodes.add(0, temp.lastNode);
            temp = temp.parentInstance;
        }
        /*Collections.sort(nodes, new Comparator<Node>() {
            public int compare(Node o1, Node o2) {
                return o1.getIdentifier().compareTo(o2.getIdentifier());
            }
        });
        */
    }

    public String toString() {
        StringBuilder sb = new StringBuilder();
        String netTitle = net.getTitle();
        if (netTitle.indexOf('\\') > 0) netTitle = netTitle.substring(netTitle.indexOf('\\') + 1);
        if (netTitle.indexOf('/') > 0) netTitle = netTitle.substring(netTitle.indexOf('/') + 1);
        sb.append(netTitle);
        sb.append(": (");
        sb.append(nodes.size());
        sb.append(") :");
        if (nodes.size() <= 5) {
            for (Node node : nodes) {
                sb.append(node.getIdentifier());
                sb.append("->");
            }
            sb.deleteCharAt(sb.length() - 1);
            sb.deleteCharAt(sb.length() - 1);
        }
        return sb.toString();
    }

    public Iterator<Node> iterator() {
        return nodes.iterator();
    }

    public void flagNet() {
        Set<Node> flagged = new HashSet<Node>(net.getFlaggedNodes());
        if (flagged.size() > 0)
            net.setFlaggedNodes(flagged, false);
        net.setFlaggedNodes(nodes, true);
    }

    private int hashCode = 0;

    public int hashCode() {
        if (hashCode == 0) {

            for (int i = 0; i < nodes.size(); i++) {
                Node node = nodes.get(i);
                hashCode += i * node.hashCode();
            }
        }
        return this.hashCode + net.hashCode();
    }

    public boolean equals(Object g) {
        if (g instanceof ResultInstance) {
            ResultInstance gi = (ResultInstance) g;
            // return gi.edges.equals(this.edges);
            if(nodes.size()!= gi.nodes.size()) return false;

            for (int i = 0; i < nodes.size(); i++) {
                if (!nodes.get(i).equals(gi.nodes.get(i))) return false;
            }
            return true;
        }
        return false;
    }
}
