package nemi.gspan;

import cytoscape.CyNetwork;
import giny.model.Edge;
import giny.model.Node;
import nemi.NetNode;
import nemi.Toolkit;

import java.util.Collection;
import java.util.HashSet;
import java.util.Set;

/**
 * $Id: SubgraphInstance.java 441 2006-06-08 15:32:58Z kristost $
 * Created by IntelliJ IDEA.
 * User: kristost
 * Date: 30.mar.2006
 * Time: 11:55:18
 * <p/>
 * <p/>
 * Default DfsNode subgraphInstance. Does not save reference to all nodes, only the last; and its
 * corresponding subgraph parent instance.
 */
public final class SubgraphInstance {//implements Iterable<Node> {


    // final List<Node> nodes; // ordered as in the closedcode discovery order
    public final Node lastNode;
    public final SubgraphInstance parentInstance;
    public final CyNetwork net;
    private int hashCode = 0;

    public SubgraphInstance(Node lastNode, SubgraphInstance parentInstance, CyNetwork net) {
        this.lastNode = lastNode;
        this.parentInstance = parentInstance;
        this.net = net;
    }

    public String toString() {
        if(parentInstance==null) return "Bow before me for I am root. " + lastNode.getIdentifier();
        return "OrigInstance " + parentInstance.lastNode.getIdentifier() + ", " + lastNode.getIdentifier();
    }

    @SuppressWarnings({"CloneDoesntCallSuperClone"})
    public SubgraphInstance clone() {
        return new SubgraphInstance(lastNode, this, net);
    }

    public SubgraphInstance createChildForward(Node newNode, int fromIx) {
        return new SubgraphInstance(newNode, this, net);
    }

    public SubgraphInstance createChildBackward(Edge newEdge) {
        System.err.println("ikke bruke denne");
        SubgraphInstance newI = new SubgraphInstance(lastNode, this, net);
        if (newEdge == null) {
            throw new NullPointerException("Backward edge not found");
        }
        return newI;
    }

    public Collection<Node> getUndiscoveredNeighs(int ofNode) {
        ResultInstance f = new ResultInstance(this);
        Set<Node> neighs = new HashSet<Node>(Toolkit.getNeighbors(new NetNode(net, f.nodes.get(ofNode))));
        neighs.removeAll(f.nodes);
        return neighs;
    }

    public int hashCode() {
        if (hashCode == 0) {
            ResultInstance f = new ResultInstance(this);
            for (int i = 0; i < f.nodes.size(); i++) {
                Node node = f.nodes.get(i);
                hashCode += i * node.hashCode();
            }
        }
        return this.hashCode + net.hashCode();
    }


    public boolean equals(Object g) {
        if (!(g instanceof SubgraphInstance)) return false;
        SubgraphInstance other = (SubgraphInstance) g;
        //if (other.hashCode != hashCode) return false;
        if (!lastNode.equals(other.lastNode)) return false;
        //noinspection ForLoopReplaceableByForEach
        if (parentInstance == null || other.parentInstance == null)
            return parentInstance == null && other.parentInstance == null;
        if(parentInstance == other.parentInstance) return true;
        return parentInstance.equals(other.parentInstance);

    }
}
