package nemi.gspan;

import cytoscape.CyNetwork;
import giny.model.Node;

import java.util.*;
import java.io.IOException;
import java.io.FileWriter;
import java.io.File;

import nemi.NetNode;
import nemi.compare.ProteinCompare;

/**
 * NeMi - Neighborhood Mining
 * Distributed AS-IS - No Warranty
 * $Id: gSpanMotif.java 471 2006-06-12 16:40:56Z kristost $
 */
public class gSpanMotif {
    public Map<CyNetwork, Set<List<Node>>> matchset;
    public List<OrigInstance> origInstances;
    public DfsResult node;
    public double score;
    private Map<String, Map<CyNetwork, Node>> str2origNet2origNode;
    private ProteinCompare comparer;

    gSpanMotif(Map<CyNetwork, Set<List<Node>>> matchset, DfsResult node,
               List<OrigInstance> origInstances,
               Map<String, Map<CyNetwork, Node>> str2origNet2origNode,
               ProteinCompare comparer) {
        this.origInstances = origInstances;
        this.matchset = matchset;
        this.node = node;
        this.str2origNet2origNode = str2origNet2origNode;
        this.comparer = comparer;
        this.score = calcNode2NodeScore();
    }


    private double calcNode2NodeScore() {
        int nnodes = this.origInstances.iterator().next().getExpInstance().nodes.size();

        List<List<NetNode>> aligneds = new Vector<List<NetNode>>();

        // creating column arrays of nodes (instances placed under each other)
        // nodes
        for (int i = 0; i < nnodes; i++) {
            List<NetNode> aligned = new Vector<NetNode>();
            // orig instances
            for (OrigInstance origInstance : origInstances) {
                // exp instance
                // // CAUTION: EXP NODE; BUT ORIG NET
                Node node = origInstance.getExpInstance().nodes.get(i);
                CyNetwork net = origInstance.getNetwork();
                aligned.add(new NetNode(net, node));
            }
            aligneds.add(aligned);
        }

        double score = 0;

        for (List<NetNode> aligned : aligneds) {
            double avg = 0.0;
            int count = 0;
            for (int i = 0; i < aligned.size() - 1; i++) {
                NetNode node1 = aligned.get(i);

                // changing back to original node
                if (node1.node.getIdentifier().length() > 1 &&
                        node1.node.getIdentifier().charAt(1) == '.')
                    node1.node = str2origNet2origNode.get(
                            node1.node.getIdentifier().substring(2)).get(node1.net);

                for (int j = i + 1; j < aligned.size(); j++) {
                    NetNode node2 = aligned.get(j);
                    if (node2.node.getIdentifier().length() > 1 &&
                            node2.node.getIdentifier().charAt(1) == '.')
                        node2.node = str2origNet2origNode.get(
                                node2.node.getIdentifier().substring(2)).get(node2.net);
                    try {
                        avg += comparer.similarity(node1, node2);
                        count ++;
                    } catch (Exception e) {
                        System.out.print(".");
                    }
                }
            }
            score += (avg / count);
        }
        return (score / (double) aligneds.size());
    }

    public int countMatches() {
        int count = 0;
        for (Map.Entry<CyNetwork, Set<List<Node>>> entry : matchset.entrySet()) {
            count += entry.getValue().size();
        }
        return count;

    }
    /* writes results to file for plotting
    private static void checkResults(DfsTreeNode root, ProteinCompare comp) {
        final Map<String, Double> scoreMap = new HashMap<String, Double>();
        for (int i = 0; i < root.getChildCount(); i++) {
            DfsTreeNode dfsTreeNode = (DfsTreeNode) root.getChildAt(i);
            String s = dfsTreeNode.toString() + "\n";
            double score = checkDfsTreeNode(dfsTreeNode, comp);
            s += score + "\n\n";
            scoreMap.put(s, score);
        }
        Vector<String> sorted = new Vector<String>(scoreMap.keySet());
        Collections.sort(sorted, new Comparator<String>() {
            public int compare(String o1, String o2) {
                return scoreMap.get(o1).compareTo(scoreMap.get(o2));
            }
        });

        try {
            FileWriter fw = new FileWriter(new File("nodeToNodeCheck"));
            for (String s : sorted) {
                fw.write(s);
            }
            fw.close();
            System.out.println("!!WROTE RESULTS!!");
        } catch (IOException e) {
            e.printStackTrace();
        }

    }
    */


}