package nemi.gui;

import javax.swing.*;
import java.awt.*;
import java.text.DecimalFormat;
import java.util.Vector;

/**
 * NeMi - Neighborhood GSpanMining
 * Distributed AS-IS - No Warranty
 * $Id: GraphPlotPanel.java 407 2006-06-05 13:11:01Z kristost $
 *
 * Graph plotting component, used to plot comparer histograms.
 *
 */
public class GraphPlotPanel extends JPanel {
    double[] xvalues;
    double[] yvalues;
    String xlabel, ylabel;
    double xmin, xmax, ymin, ymax;
    int top = 10, bottom = 10, left = 10, right = 10;
    DecimalFormat scientificFormat = new DecimalFormat("#.#E0");
    DecimalFormat humanFormat = new DecimalFormat("#.#");
    float dash[] = {10.0f};
    Stroke dashed = new BasicStroke(1.0f, BasicStroke.CAP_BUTT,

            BasicStroke.JOIN_MITER, 10.0f, dash, 0.0f);
    Stroke regular = new BasicStroke();
    java.util.List<Double> xline = new Vector<Double>();
    java.util.List<Double> yline = new Vector<Double>();

    public GraphPlotPanel(double[] xvalues, double[] yvalues, String xlabel, String ylabel) {
        super();

        setValues(xvalues, yvalues, xlabel, ylabel);
        setMargins(10, 10, 30, 50);

    }

    public void setValues(double[] xvalues, double[] yvalues, String xlabel, String ylabel) {

        int min = Math.min(xvalues.length, yvalues.length);
        this.xvalues = new double[min];
        this.yvalues = new double[min];
        xmax = Double.MIN_VALUE;
        ymax = Double.MIN_VALUE;
        xmin = Double.MAX_VALUE;
        ymin = Double.MAX_VALUE;
        for (int i = 0; i < min; i++) {
            this.xvalues[i] = xvalues[i];
            this.yvalues[i] = yvalues[i];
            xmax = Math.max(xmax, xvalues[i]);
            ymax = Math.max(ymax, yvalues[i]);
            xmin = Math.min(xmin, xvalues[i]);
            ymin = Math.min(ymin, yvalues[i]);
        }
        this.xlabel = xlabel;
        this.ylabel = ylabel;


    }

    public void setMargins(int top, int right, int bottom, int left) {
        this.top = Math.max(top, 0);
        this.bottom = Math.max(bottom, 0);
        this.left = Math.max(left, 0);
        this.right = Math.max(right, 0);
        this.setMinimumSize(new Dimension(left + right, right + left));

    }


    public void paint(Graphics g) {
        super.paint(g);
        Graphics2D g2d = (Graphics2D) g;
        regular = g2d.getStroke();
        g2d.setRenderingHint(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON);

        int w = this.getWidth();
        int h = this.getHeight();
        drawLine(g2d, w, h);
        drawCoordinates(g2d, w, h, true);
        drawXandYLines(g2d, w, h);

    }

    private void drawXandYLines(Graphics2D g2d, int w, int h) {
        g2d.setColor(Color.RED);
        for (double d : xline) {
            PixelPos old = getPixel(d, 0, w, h);
            g2d.drawLine(old.x, top, old.x, h - bottom);
        }
        for (double d : yline) {
            PixelPos old = getPixel(0, d, w, h);
            g2d.drawLine(left,old.y, w-right, old.y);
        }

        g2d.setColor(Color.BLACK);
    }

    public void drawLine(Graphics2D g2d, int w, int h) {
        Polygon pol = new Polygon();
        PixelPos old = getPixel(xvalues[0], yvalues[0], w, h);
        g2d.setColor(Color.BLUE);
        for (int i = 1; i < xvalues.length; i++) {
            double x = xvalues[i];
            double y = yvalues[i];
            PixelPos p = getPixel(x, y, w, h);
            g2d.drawLine(old.x, old.y, p.x, p.y);
            old = p;
        }
    }

    public void drawXLineAt(double x) {
        xline.add(x);
    }
    public void drawYLineAt(double x) {
        yline.add(x);
    }

    public void drawCoordinates(Graphics2D g2d, int w, int h, boolean grid) {

        g2d.setColor(Color.BLACK);
        //g2d.setFont(Font.g);
        //X coordinate axis
        g2d.drawLine(left, h - bottom, w - right, h - bottom);
        g2d.drawString(xlabel, w / 2, h - 8);

        //X axis ticks
        int xexp = (int) Math.floor(Math.log10(xmax - xmin));
        double interval = Math.pow(10, (double) xexp);
        if (xmin + interval * 2 > xmax) {
            xexp--;
            interval = Math.pow(10, (double) xexp);
        }
        double value = xmin / interval;
        value = Math.floor(value);
        value *= interval;
        if (value <= xmin) value += interval;
        for (; value <= xmax; value += interval) {
            PixelPos p = getPixel(value, 0, w, h);
            if (grid) {
                g2d.setStroke(dashed);
                g2d.setColor(Color.GRAY);

                g2d.drawLine(p.x, h - bottom, p.x, top);
                g2d.setColor(Color.BLACK);
                g2d.setStroke(regular);
            }
            g2d.drawLine(p.x, h - bottom + 3, p.x, h - bottom - 7);
            double v = value;

            g2d.drawString(humanFormat.format(value), p.x - 10, h - bottom + 12);

        }

        //Y coordinate axis
        g2d.drawLine(left, top, left, h - bottom);
        g2d.rotate(-Math.PI / 2);
        g2d.drawString(ylabel, -h / 2, 10);
        g2d.rotate(Math.PI / 2);

        //draw Y axis ticks
        int yexp = (int) Math.floor(Math.log10(ymax - ymin));
        interval = Math.pow(10, (double) yexp);
        if (ymin + interval * 2 > ymax) {
            yexp--;
            interval = Math.pow(10, (double) yexp);
        }
        value = ymin / interval;
        value = Math.floor(value);
        value *= interval;
        if (value <= ymin) value += interval;
        for (; value <= ymax; value += interval) {
            PixelPos p = getPixel(0, value, w, h);
            if (grid) {
                g2d.setStroke(dashed);
                g2d.setColor(Color.GRAY);
                g2d.drawLine(left, p.y, w - right, p.y);
                g2d.setColor(Color.BLACK);
                g2d.setStroke(regular);


            }
            g2d.drawString(scientificFormat.format(value), left - 30, p.y + 5);
            g2d.drawLine(left - 3, p.y, left + 7, p.y);
        }


    }

    public PixelPos getPixel(double x, double y, int width, int height) {
        PixelPos p = new PixelPos();
        p.x = (int) (((x - xmin) / (xmax - xmin)) * (width - right - left) + left);
        double yt = (y - ymin) / (ymax - ymin);
        yt *= (height - top - bottom);
        //yt -=top;
        yt = height - yt - bottom;

        p.y = (int) yt;
        return p;
    }

    class PixelPos {
        int x;
        int y;
    }
}



