package nemi.gui;

import cytoscape.CyNetwork;
import cytoscape.Cytoscape;
import cytoscape.data.annotation.Annotation;
import cytoscape.data.annotation.AnnotationDescription;
import giny.model.Node;
import nemi.cluster.Cluster;
import nemi.compare.ProteinCompare;
import nemi.compare.gui.ProteinComparerTable;
import nemi.cytoscape.AnnotationList;
import nemi.gspan.DfsResult;

import javax.swing.*;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;
import javax.swing.table.AbstractTableModel;
import javax.swing.table.DefaultTableCellRenderer;
import java.awt.*;
import java.io.*;
import java.text.DecimalFormat;
import java.util.*;
import java.util.List;

/**
 * Created by IntelliJ IDEA.
 * User: kristost
 * Date: 03.apr.2006
 * Time: 14:47:52
 * Shows comparer & cluster statistics.
 */
public class StatFrame extends JFrame implements ListSelectionListener {

    JSplitPane splitter = new JSplitPane();
    JSplitPane clusterOtherSplitter = new JSplitPane();
    //JSplitPane bottomSplitter = new JSplitPane();
    JSplitPane nodesTFsplitter = new JSplitPane();
    List<DfsResult> results = null;
    NetworkSelector networkSel = new NetworkSelector();
    JLabel networkLabel = new JLabel();
    ProteinComparerTable protTable = new ProteinComparerTable();

    GraphPlotPanel plot;
    JLabel proteinCompLabel = new JLabel();
    JPanel controlPanel = new JPanel();
    JScrollPane allScroller = new JScrollPane();
    JScrollPane resultScroller = new JScrollPane();
    SliderWrapper clusteringLimitSlider;

    JScrollPane networkScroller = new JScrollPane();
    JLabel clusteringLimitLabel = new JLabel();
    // JTree resultTree = new JTree(new DefaultTreeModel(null));//new JTreeTable(new AbstractTreeTableModel());
    //ClusterNodeDistancePanel clusterNodeDistPanel = new ClusterNodeDistancePanel();
    // JLabel clusterNodeDistLabel = new JLabel();
    JTable clusterList = new JTable();
    JTable tfTable = new JTable();
    JTable nodeTable = new JTable();
    private JLabel clusterInfo;
    ProteinCompare comp;
    Collection<Cluster> clusters;
    Collection<CyNetwork> networks;

    double limit;
    private StringIntTableModel tabmod;
    //private Map<String, Map<String, Double>> nodeId2Fact2Prob;
    //private Map<Cluster, Map<String, Integer>> cluster2Fact2Count;

    public StatFrame(Collection<Cluster> clusters, ProteinCompare comparer, Collection<CyNetwork> networks, double oldLimit) {
        super("Statistics");
        limit = oldLimit;
        this.comp = comparer;
        this.clusters = clusters;
        this.networks = networks;
        initComponents();
        this.setDefaultCloseOperation(JFrame.DISPOSE_ON_CLOSE);
        this.setSize(500, 500);
        splitter.setDividerLocation(this.getHeight() / 2);
        clusterOtherSplitter.setDividerLocation(this.getWidth() / 3);
        nodesTFsplitter.setDividerLocation(this.getWidth() / 3);

    }


    private void initComponents() {
        splitter.setOrientation(JSplitPane.VERTICAL_SPLIT);
        this.add(splitter);


        splitter.setTopComponent(allScroller);
        allScroller.setViewportView(controlPanel);

        clusterOtherSplitter.setOrientation(JSplitPane.HORIZONTAL_SPLIT);
        //bottomSplitter.setDividerLocation(0.4);
        splitter.setBottomComponent(clusterOtherSplitter);
        JScrollPane clusterScroller = new JScrollPane(clusterList);
        clusterOtherSplitter.setLeftComponent(clusterScroller);
        JScrollPane nodeScroller = new JScrollPane(nodeTable);
        clusterOtherSplitter.setRightComponent(nodesTFsplitter);
        nodesTFsplitter.setOrientation(JSplitPane.HORIZONTAL_SPLIT);
        nodesTFsplitter.setLeftComponent(nodeScroller);
        JScrollPane tfScroller = new JScrollPane(tfTable);
        nodesTFsplitter.setRightComponent(tfScroller);

        makeHisto();
        updateClusterInfo();
        nodeTable.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);

        nodeTable.setDefaultRenderer(cytoscape.CyNode.class, new DefaultTableCellRenderer() {
            public Component getTableCellRendererComponent(JTable table, Object value, boolean isSelected, boolean hasFocus, int row, int column) {
                JLabel label = (JLabel) super.getTableCellRendererComponent(table, value, isSelected, hasFocus, row, column);
                Node n = (Node) value;
                label.setText(n.getIdentifier());
                return label;
            }
        });
        nodeTable.getSelectionModel().addListSelectionListener(new ListSelectionListener() {
            public void valueChanged(ListSelectionEvent e) {
                if (e.getValueIsAdjusting()) return;
                int i = e.getFirstIndex();
                ListSelectionModel lsm = (ListSelectionModel) e.getSource();
                if (!lsm.isSelectedIndex(i)) i = e.getLastIndex();
                clearSelection();
                if (nodeTable.getRowCount() <= i) return;
                Node n = (Node) nodeTable.getValueAt(i, -1);
                for (CyNetwork net : networks) {
                    selectNode(n, net);
                }

                tfTable.clearSelection();
                for (AnnotationDescription anno : new AnnotationList()) {
                    Annotation a = Cytoscape.getBioDataServer().getAnnotation(anno);
                    Vector<Integer> classIdVetor = a.getClassificationsVector(n.getIdentifier());
                    for (Integer ir : classIdVetor) {
                        try {
                            String id = a.getOntology().getTerm(ir).getName();
                            for (int l = 0; l < tfTable.getRowCount(); l++) {
                                String otherId = (String)tfTable.getValueAt(l, 0)  ;
                                if (otherId.equals(id)) {
                                    tfTable.addRowSelectionInterval(l, l);

                                }
                            }
                        } catch (NullPointerException npe) {
                        }

                    }
                }

                /*if (cluster2Fact2Count == null) {
                    updateid2fact2prob();
                }

                tfTable.clearSelection();

                Map<String, Double> fac2prob = nodeId2Fact2Prob.get(n.getIdentifier());
                if (fac2prob != null && fac2prob.size() > 0) {

                    for (Map.Entry<String, Double> entry : fac2prob.entrySet()) {
                        for (int l = 0; l < tfTable.getRowCount(); l++) {
                            if (tfTable.getValueAt(l, 0).equals(entry.getKey())) {
                                tfTable.addRowSelectionInterval(l, l);

                            }
                        }
                    }
                }
                */

            }


        });


    }

    private void clearSelection() {
        for (CyNetwork n : networks) {
            Set<Node> selectedNodes = new HashSet<Node>(n.getFlaggedNodes());
            n.setFlaggedNodes(selectedNodes, false);

        }

    }


    private void selectNode(Node node, CyNetwork network) {


        if (network.containsNode(node)) {
            network.setFlagged(node, true);

        }
    }


    private void updateClusterInfo() {
        clusterList.getSelectionModel().removeListSelectionListener(this);

        ClusterTableModel tableModel = new ClusterTableModel(new Vector<Cluster>(clusters));
        clusterList.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
        TableSorter sorter = new TableSorter(tableModel);
        clusterList.setModel(sorter);
        sorter.setTableHeader(clusterList.getTableHeader());
        clusterList.getSelectionModel().addListSelectionListener(this);
        System.out.println("Number of clusters " + clusters.size());
        int sum = 0;
        //char i = 'a';
        HashSet<Node> hashList = new HashSet<Node>();
        for (Cluster c : clusters) {
            sum += c.size();
            hashList.addAll(c.getRepNodes());
        }

        //clusterInfo.setText("Clustercount: "  + clusters.size() + ". One node is on average in " + (double)sum/(double)hashList.size() + " clusters.");
        //clusterInfo.repaint();


    }

    /*
    public void updateid2fact2prob() {
        //nodeId2Fact2Prob = nemi.Toolkit.readBindings();
        //nemi.Toolkit.id2fac2p = null;
        for (Map.Entry<String, Map<String, Double>> entry : nodeId2Fact2Prob.entrySet()) {
            for (Iterator<Map.Entry<String, Double>> fac2probEntryIter = entry.getValue().entrySet().iterator(); fac2probEntryIter.hasNext();)
            {
                Map.Entry<String, Double> fac2probEntry = fac2probEntryIter.next();
                if(fac2probEntry.getValue()<nemi.Toolkit.tfProbLimit){
                    fac2probEntryIter.remove();
                }


            }
        }

        cluster2Fact2Count = new HashMap<Cluster, Map<String, Integer>>();

        for (Cluster c : clusters) {
            if (!cluster2Fact2Count.containsKey(c))
                cluster2Fact2Count.put(c, new HashMap<String, Integer>());
            for (Node n : c.getRepNodes()) {
                if (!nodeId2Fact2Prob.containsKey(n.getIdentifier())) continue;
                Map<String, Integer> fact2count = cluster2Fact2Count.get(c);
                for (Map.Entry<String, Double> factProbentry : nodeId2Fact2Prob.get(n.getIdentifier()).entrySet())
                {
                    String fact = factProbentry.getKey();
                    if (!fact2count.containsKey(fact)) fact2count.put(fact, 0);
                    fact2count.put(fact, fact2count.get(fact) + 1);
                }
            }
        }


    }

     */
    private void makeHisto() {
        Map<Double, Integer> count = nemi.Toolkit.comparerStats(comp, networks);

        double[] xvalues = new double[count.size()];
        double[] yvalues = new double[count.size()];
        Vector<Double> keys = new Vector<Double>(count.keySet());
        Collections.sort(keys);
        Writer w = null;
        try {
            FileOutputStream fos = new FileOutputStream("out.dat");
            w =
                    new BufferedWriter(new OutputStreamWriter(fos));
        } catch (FileNotFoundException e1) {
            e1.printStackTrace();  //To change body of catch statement use File | Settings | File Templates.
        }
        for (int i = 0; i < keys.size(); i++) {
            Double d = keys.elementAt(i);
            xvalues[i] = d;
            yvalues[i] = count.get(d);
            System.out.println(d + " : " + count.get(d));
            if (w != null) {
                try {
                    w.write(d + " : " + count.get(d) + "\n");
                } catch (IOException e1) {
                    e1.printStackTrace();
                }
            }
        }
        try {
            w.flush();
            w.close();
        } catch (IOException e1) {
            e1.printStackTrace();  //To change body of catch statement use File | Settings | File Templates.
        }

        plot = new GraphPlotPanel(xvalues, yvalues, "Similarity", "Frequency");
        plot.drawXLineAt(limit);
        allScroller.setViewportView(plot);
        allScroller.repaint();

    }


    public static void main(String[] args) {
        double b = 234234;
        double d = 0.3432432;

        DecimalFormat form = new DecimalFormat("#.#E0");
        StringBuffer sb = new StringBuffer();
        String sb2 = form.format(b);
        String sb3 = form.format(d);
        System.out.println("sb2 " + sb2 + " sb3: " + sb3);

    }

    /**
     * Called whenever the clusterlist is slected
     *
     * @param e the event that characterizes the change.
     */
    public void valueChanged(ListSelectionEvent e) {
        if (e.getValueIsAdjusting()) return;
        int index = e.getFirstIndex();
        ListSelectionModel lsm = (ListSelectionModel) e.getSource();
        if (!lsm.isSelectedIndex(index)) index = e.getLastIndex();
        Cluster c = (Cluster) clusterList.getModel().getValueAt(index, -1);
        Map<String, Integer> fac2count = new HashMap<String, Integer>();
        List<String> idList = new Vector<String>();
        List<Integer> countList = new Vector<Integer>();
        /*
        if (nodeId2Fact2Prob == null) updateid2fact2prob();
        if (cluster2Fact2Count == null) updateid2fact2prob();


        for (Map.Entry<String, Integer> factCount : cluster2Fact2Count.get(c).entrySet()) {
            idList.add(factCount.getKey());
            countList.add(factCount.getValue());
        } */
        Map<String, Integer> annot2Count = new HashMap<String, Integer>();
        for (Node n : c.getRepNodes()) {
            for (AnnotationDescription anno : new AnnotationList()) {
                Annotation a = Cytoscape.getBioDataServer().getAnnotation(anno);
                Vector<Integer> classIdVetor = a.getClassificationsVector(n.getIdentifier());
                for (Integer ir : classIdVetor) {
                    try {
                        String id = a.getOntology().getTerm(ir).getName();
                        if (!annot2Count.containsKey(id)) annot2Count.put(id, 0);
                        annot2Count.put(id, annot2Count.get(id) + 1);

                    } catch (NullPointerException npe) {
                    }

                }
            }
        }
        for (Map.Entry<String, Integer> annoCount : annot2Count.entrySet()) {
            idList.add(annoCount.getKey());
            countList.add(annoCount.getValue());
        }
        tabmod = new StringIntTableModel(idList, countList);
        TableSorter tfCountSorter = new TableSorter(tabmod);
        tfTable.setModel(tfCountSorter);
        tfCountSorter.setTableHeader(tfTable.getTableHeader());
        tfTable.getSelectionModel().addListSelectionListener(new ListSelectionListener() {
            public void valueChanged(ListSelectionEvent e) {
                nodeTable.getValueAt(0, -1);
            }
        });
        NodeTableModel nodemod = new NodeTableModel(new Vector<Node>(c.getRepNodes()), clusters);
        TableSorter nodeSorter = new TableSorter(nodemod);
        nodeTable.setModel(nodeSorter);
        nodeSorter.setTableHeader(nodeTable.getTableHeader());
        clearSelection();
        for (CyNetwork net : networks) {
            for (Node n : c.getRepNodes()) {
                selectNode(n, net);
            }
        }
    }
}

class ClusterTableModel extends AbstractTableModel {
    private List<Cluster> list;

    ClusterTableModel(List<Cluster>vector) {
        list = vector;
    }


    /**
     * Returns the number of rows in the model. A
     * <code>JTable</code> uses this method to determine how many rows it
     * should display.  This method should be quick, as it
     * is called frequently during rendering.
     *
     * @return the number of rows in the model
     * @see #getColumnCount
     */
    public int getRowCount() {
        return list.size();  //To change body of implemented methods use File | Settings | File Templates.
    }

    /**
     * Returns the number of columns in the model. A
     * <code>JTable</code> uses this method to determine how many columns it
     * should create and display by default.
     *
     * @return the number of columns in the model
     * @see #getRowCount
     */
    public int getColumnCount() {
        return 2;  //To change body of implemented methods use File | Settings | File Templates.
    }


    /**
     * Returns a default name for the column using spreadsheet conventions:
     * A, B, C, ... Z, AA, AB, etc.  If <code>column</code> cannot be found,
     * returns an empty string.
     *
     * @param column the column being queried
     * @return a string containing the default name of <code>column</code>
     */
    public String getColumnName(int column) {
        if (column == 0) return "Cluster ID";
        if (column == 1) return "Nodecount";
        return "none";
    }

    /**
     * Returns <code>Object.class</code> regardless of <code>columnIndex</code>.
     *
     * @param columnIndex the column being queried
     * @return the Object.class
     */
    public Class<?> getColumnClass(int columnIndex) {
        if (columnIndex == 0)
            return String.class;    //To change body of overridden methods use File | Settings | File Templates.
        if (columnIndex == 1) return Integer.class;
        return Object.class;
    }

    /**
     * Returns the value for the cell at <code>columnIndex</code> and
     * <code>rowIndex</code>.
     *
     * @param rowIndex    the row whose value is to be queried
     * @param columnIndex the column whose value is to be queried
     * @return the value Object at the specified cell
     */
    public Object getValueAt(int rowIndex, int columnIndex) {
        Cluster c = list.get(rowIndex);

        if (columnIndex == 0) return "'" + c.id + "'";
        if (columnIndex == 1) return c.size();
        return c;  //To change body of implemented methods use File | Settings | File Templates.
    }
}

class StringIntTableModel extends AbstractTableModel {
    List<String> ids;
    List<Integer> counts;

    StringIntTableModel(List<String> ids, List<Integer> counts) {
        this.ids = ids;
        this.counts = counts;
    }

    /**
     * Returns the number of rows in the model. A
     * <code>JTable</code> uses this method to determine how many rows it
     * should display.  This method should be quick, as it
     * is called frequently during rendering.
     *
     * @return the number of rows in the model
     * @see #getColumnCount
     */
    public int getRowCount() {
        return ids.size();  //To change body of implemented methods use File | Settings | File Templates.
    }

    /**
     * Returns the number of columns in the model. A
     * <code>JTable</code> uses this method to determine how many columns it
     * should create and display by default.
     *
     * @return the number of columns in the model
     * @see #getRowCount
     */
    public int getColumnCount() {
        return 2;  //To change body of implemented methods use File | Settings | File Templates.
    }

    /**
     * Returns the value for the cell at <code>columnIndex</code> and
     * <code>rowIndex</code>.
     *
     * @param rowIndex    the row whose value is to be queried
     * @param columnIndex the column whose value is to be queried
     * @return the value Object at the specified cell
     */
    public Object getValueAt(int rowIndex, int columnIndex) {
        if (columnIndex == 0) return ids.get(rowIndex);
        if (columnIndex == 1) return counts.get(rowIndex);
        return null;
    }

    /**
     * Returns a default name for the column using spreadsheet conventions:
     * A, B, C, ... Z, AA, AB, etc.  If <code>column</code> cannot be found,
     * returns an empty string.
     *
     * @param column the column being queried
     * @return a string containing the default name of <code>column</code>
     */
    public String getColumnName(int column) {
        if (column == 0) return "Name";
        if (column == 1) return "Count";
        return "none";
    }

    public Class getColumnClass(int c) {
        return getValueAt(0, c).getClass();
    }
}

class NodeTableModel extends AbstractTableModel {
    List<Node> nodes;
    Map<Node, Set<Cluster>> node2Clusters;

    NodeTableModel(List<Node> ids, Collection<Cluster> clusters) {
        nodes = ids;
        node2Clusters = new HashMap<Node, Set<Cluster>>();

        for (Cluster cluster : clusters) {
            for (Node n : cluster.getRepNodes()) {
                if (!node2Clusters.containsKey(n)) node2Clusters.put(n, new HashSet<Cluster>());
                Set<Cluster> set = node2Clusters.get(n);
                set.add(cluster);
            }
        }

    }

    /**
     * Returns the number of rows in the model. A
     * <code>JTable</code> uses this method to determine how many rows it
     * should display.  This method should be quick, as it
     * is called frequently during rendering.
     *
     * @return the number of rows in the model
     * @see #getColumnCount
     */
    public int getRowCount() {
        return nodes.size();  //To change body of implemented methods use File | Settings | File Templates.
    }

    /**
     * Returns the number of columns in the model. A
     * <code>JTable</code> uses this method to determine how many columns it
     * should create and display by default.
     *
     * @return the number of columns in the model
     * @see #getRowCount
     */
    public int getColumnCount() {
        return 2;  //To change body of implemented methods use File | Settings | File Templates.1
    }

    /**
     * Returns the value for the cell at <code>columnIndex</code> and
     * <code>rowIndex</code>.
     *
     * @param rowIndex    the row whose value is to be queried
     * @param columnIndex the column whose value is to be queried
     * @return the value Object at the specified cell
     */
    public Object getValueAt(int rowIndex, int columnIndex) {
        Node n = nodes.get(rowIndex);
        if (columnIndex == 0) return n.getIdentifier();
        if (columnIndex == 1) {
            StringBuilder sb = new StringBuilder();
            sb.append("'");
            for (Cluster c : node2Clusters.get(n)) {
                sb.append(c.id);
                sb.append("', ");

            }
            sb.delete(sb.length() - 2, sb.length() - 1);
            return sb.toString();

        }
        return n;
    }

    /**
     * Returns a default name for the column using spreadsheet conventions:
     * A, B, C, ... Z, AA, AB, etc.  If <code>column</code> cannot be found,
     * returns an empty string.
     *
     * @param column the column being queried
     * @return a string containing the default name of <code>column</code>
     */
    public String getColumnName(int column) {
        if (column == 0) return "Node";
        if (column == 1) return "Cluster(s)";
        return "none";
    }

    public Class getColumnClass(int c) {
        if(c == 0) return String.class;
        if(c==1) return String.class  ;
        return getValueAt(0, c).getClass();
    }
}