package nemi.motif;

import giny.model.Node;

import java.util.*;


public class Assignment {
    public HashMap<Node, Node> motifToNetworkAssignment;
    public HashMap<Node, Node> networkToMotifAssignment;
    public double score;
    private String hashString = null;

    public Assignment(HashMap<Node, Node> assignment, double score) {
        this.motifToNetworkAssignment = assignment;
        this.networkToMotifAssignment = new HashMap<Node, Node>();
        for (Map.Entry<Node, Node> entry : assignment.entrySet()) {
            networkToMotifAssignment.put(entry.getValue(), entry.getKey());
        }
        this.score = score;

    }
    public Assignment(Vector<Node> motif, Vector<Node> networkVector, double score){
        for(int i = 0; i < motif.size();i++){
            networkToMotifAssignment.put(networkVector.get(i), motif.get(i));
            motifToNetworkAssignment.put(motif.get(i), networkVector.get(i));
        }
        this.score = score;
    }
    private Assignment(HashMap<Node, Node> mna, HashMap<Node, Node> nma, double score){
        this.motifToNetworkAssignment = mna;
        this.networkToMotifAssignment = nma;
        this.score = score;

    }
    public Assignment clone(){
        HashMap<Node, Node> mna = new HashMap<Node, Node>(motifToNetworkAssignment);
        HashMap<Node, Node> nma = new HashMap<Node, Node>(networkToMotifAssignment);
        return new Assignment(mna, nma, score);
    }

    public String getNodeHashString(){
        if (this.hashString == null) {
            Vector<Node> nnodes = new Vector<Node>(motifToNetworkAssignment.keySet());
            Vector<Node> mnodes = new Vector<Node>(networkToMotifAssignment.keySet());
            Comparator<Node> comp = new Comparator<Node>() {
                public int compare(Node o1, Node o2) {
                    return o1.getIdentifier().compareTo(o2.getIdentifier());
                }
            };
            Collections.sort(nnodes, comp);
            Collections.sort(mnodes, comp);

            StringBuilder s = new StringBuilder();
            for (Node node : nnodes)  s.append(node.getIdentifier());
            for (Node node : mnodes)  s.append(node.getIdentifier());
            this.hashString = s.toString();

        }
        return this.hashString;
    }
   /*
    private int hashCode = Integer.MAX_VALUE;
    public int hashCode(){
        if (hashCode == Integer.MAX_VALUE) {
            StringBuilder s = new StringBuilder();
            for (Map.Entry<Node, Node> entry : this.motifToNetworkAssignment.entrySet()) {
                s.append(entry.getKey().getIdentifier()).append(entry.getValue().getIdentifier());
            }
            hashCode = s.toString().hashCode();
        }
        return hashCode;
    }
    public boolean equals(Object o){
        if (o instanceof Assignment){
            return o.hashCode() == this.hashCode();
        }
        return false;
    }
    */
    public Assignment() {
        this.score = -1;
        this.motifToNetworkAssignment = null;
    }
    public int size(){
        return motifToNetworkAssignment.size();
    }

    public Collection<Node> getNetworkNodes() {
        return motifToNetworkAssignment.values();
    }
    public Collection<Node> getMotifNodes() {
        return motifToNetworkAssignment.keySet();
    }
    public boolean hasMotifNode(Node motifNode){
        return motifToNetworkAssignment.containsKey(motifNode);
    }
    public boolean hasNetworkNode(Node netNode){
        return networkToMotifAssignment.containsKey(netNode);
    }
    public void addAssignment(Node motifNode, Node networkNode){
        motifToNetworkAssignment.put(motifNode, networkNode);
        networkToMotifAssignment.put(networkNode, motifNode);
        score = -1;
    }
    public void addAssignment(Node motifNode, Node networkNode, double newScore){
        motifToNetworkAssignment.put(motifNode, networkNode);
        networkToMotifAssignment.put(networkNode, motifNode);
        score = newScore;
    }
    public Node getAssignedInNetwork(Node motifNode){
        return motifToNetworkAssignment.get(motifNode);
    }
    public Node getAssignedInMotif(Node networkNode){
            return networkToMotifAssignment.get(networkNode);
    }
    public String toString() {
        String s = "Assignment results:\nScore (sum similarity for each node): " + score + "\n";
        if (motifToNetworkAssignment != null) {
            s += "Number of assignments done: " + motifToNetworkAssignment.size() + "\n";
            for (Node mnode : motifToNetworkAssignment.keySet()) {
                s += mnode.getIdentifier() + " match " +
                        motifToNetworkAssignment.get(mnode).getIdentifier() + "\n";
            }
        } else
            s += "No assignments done\n";
        return s;
    }

    private String htmlVersion = null;
    public String toHtml() {
        if (htmlVersion == null)
            htmlVersion = "<html>" + toString().replace("\n", "<br>") + "</html>";
        return htmlVersion;
    }
}
