package nemi.motif;

import cytoscape.CyNetwork;
import giny.model.Node;
import nemi.NetNode;
import nemi.Toolkit;
import nemi.compare.ProteinCompare;

import java.util.*;

public class CleanedNet {

    private Random r = new Random();
    // private CyNetwork motif;
    private CyNetwork network;
    // maps motif node to map, which maps the network node to distance
    private HashMap<Node, HashMap<Node, Double>> mndistance = new HashMap<Node, HashMap<Node, Double>>();
    // the survived netnodes
    private ArrayList<Node> netnodes = new ArrayList<Node>();
    // reference to all survived nodes' neighbors
    // private HashMap<Node, Vector<Node>> netEdges = new HashMap<Node, Vector<Node>>();

    private int motifSize = -1;

    public CleanedNet(CyNetwork motif, CyNetwork network, double minSimilarity, ProteinCompare comparer) {
        this.network = network;

        // filter the network to create a list of 'interesting' nodes
        Node netNode = null, motifNode = null;
        int netcount = 0;
        this.motifSize = 0;
        for (Iterator ni = network.nodesIterator(); ni.hasNext(); netNode = (Node) ni.next()) {
            netcount ++;
            for (Iterator mi = motif.nodesIterator(); mi.hasNext(); motifNode = (Node) mi.next()) {
                motifSize ++;
                NetNode mnode = new NetNode(motif, motifNode);
                NetNode nnode = new NetNode(network, netNode);
                double sim = comparer.similarity(mnode, nnode);
                // all similarities calculated stored
                // addSimilarity(netNode, motifNode, sim);
                if (sim > minSimilarity) {
                    if (!netnodes.contains(netNode)) {
                        netnodes.add(netNode);
                    }
                }
            }
        }

        // referencing all survived nodes' edges
        // updateEdges();

        System.out.println("CleanedNet: Cleaned net for uninteresting nodes, was: "
                + netcount + ", now: " + this.getNetnodeSize());
    }

    public Vector<Node>[] generateRandomSubnets(int nnodes, int nnets) {
        Vector<Node>[] randomNets = new Vector[nnets];
        int interval = (int) ((double) netnodes.size() / (double) nnets);
        for (int i = 0; i < nnets; i++) {
            Vector<Node> subnet = new Vector<Node>(nnodes);
            randomNets[i] = subnet;
            // selecting a random node as start node for this subnet
            subnet.add(netnodes.get(i * interval + r.nextInt(interval) - (int) (interval * 0.5)));
            Vector<Node> neighbors = new Vector<Node>(Toolkit.getNeighbors(new NetNode(network, subnet.get(0))));

            for (int j = 1; j < nnodes; j++) {
                // adding a random neigbor
                addRandomNodeFromNeighborhood(neighbors, subnet);
            }
        }
        return randomNets;
    }

    public Node getNode(int index) {
        return this.netnodes.get(index);
    }

    public int size() {
        return this.netnodes.size();
    }

    private void addRandomNodeFromNeighborhood(Vector<Node> neighborhood, Vector<Node> toNet) {
        Node n = neighborhood.get(r.nextInt(neighborhood.size()));
        if (toNet.contains(n)) {
            addRandomNodeFromNeighborhood(neighborhood, toNet);
        } else {
            toNet.add(n);
            // growing the neighborhood with the new node's neighbors
            neighborhood.addAll(Toolkit.getNeighbors(new NetNode(network, n)));
        }
    }
    /*
    private void addSimilarity(Node netNode, Node toMotifNode, double distance) {
        if (!mndistance.containsKey(toMotifNode)) {
            mndistance.put(toMotifNode, new HashMap<Node, Double>());
        }
        mndistance.get(toMotifNode).put(netNode, distance);
    }

    public double getSimilarity(Node motifNode, Node netNode) {
        return mndistance.get(motifNode).get(netNode);
    }
    */

    /*
   private void updateEdges() {
       for (Node n : netnodes) {
           netEdges.put(n, nemi.Toolkit.getNeighbors(network.getIndex(n), network));
       }
   } */

    public List<Node> getMostSimilarNodes(Node mNode, int maxn) {
        HashMap<Node, Double> netNodesMap = mndistance.get(mNode);
        DoubleComparator comp = new DoubleComparator(netNodesMap);
        Vector<Node> nodes = new Vector<Node>(netNodesMap.keySet());
        Collections.sort(nodes, comp);
        if (maxn > nodes.size())
            return nodes;
        else
            return nodes.subList(0, maxn - 1);
    }

    public int getNetnodeSize() {
        return netnodes.size();
    }

    public int getMotifSize() {
        return this.motifSize;
    }

    class DoubleComparator implements Comparator<Node> {

        private HashMap<Node, Double> score;

        public DoubleComparator(HashMap<Node, Double> score) {
            this.score = score;
        }

        public int compare(Node o1, Node o2) {
            return (int) Math.round(score.get(o1) - score.get(o2));
        }
    }

}