package nemi.motif;

import cytoscape.CyNetwork;
import cytoscape.Cytoscape;
import nemi.compare.ProteinCompare;
import nemi.compare.gui.ProteinComparerTable;
import nemi.gui.MessagePane;
import nemi.gui.NeMiFinalView;
import nemi.gui.ViewableResults;

import javax.swing.*;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;
import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseEvent;
import java.util.Collection;
import java.util.HashMap;
import java.util.Set;
import java.util.Vector;

public class MotifFinderPanel extends JPanel implements ChangeListener {

    AssignmentResults results = null;
    ListSet netSet; // all networks open in cytoscape

    JPanel topPanel = new JPanel();
    //JPanel resultPanel = new JPanel();
    MessagePane messagePane = new MessagePane();
    JSplitPane topSplit = new JSplitPane();
    JSplitPane bottomSplit = new JSplitPane();
    MyTable table = new MyTable();
    GridBagLayout gridBagLayout = new GridBagLayout();
    JLabel jLabelMotif = new JLabel();
    JComboBox jComboMotif = new JComboBox();
    JLabel jLabelNetwork = new JLabel();
    JComboBox jComboNetwork = new JComboBox();
    JButton goButton = new JButton();
    JSlider jSliderBranchFactor = new JSlider();
    JLabel jLabelBranch = new JLabel();
    JLabel jLabelBranchVal = new JLabel();
    ProteinComparerTable protCompTable = new ProteinComparerTable();
    JButton jButtonSaveResults = new JButton("Save results");
    NeMiFinalView.StatusInfo status;

    public MotifFinderPanel(NeMiFinalView.StatusInfo status) {
        this.status = status;
        try {
            initComponents();
        } catch (Exception ex) {
            ex.printStackTrace();
        }

        // populate combos with all available networks
        //*
        netSet = new ListSet(Cytoscape.getNetworkSet());
        netSet.populateCombo(jComboMotif);
        netSet.populateCombo(jComboNetwork);
        //  */
        if (jComboMotif.getItemCount() > 1)
            jComboMotif.setSelectedIndex(1);

        jSliderBranchFactor.addChangeListener(this);
        jLabelBranchVal.setText("0.50");
        jButtonSaveResults.setEnabled(false);
        jButtonSaveResults.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                if (results != null) {
                     results.saveToFile();
                }
            }
        });
    }

    public void searchFinished() {
        goButton.setText("Run");
        jButtonSaveResults.setEnabled(true);
        goButton.setEnabled(true);
        status.setIndeterminate(false);
        if (results != null && results.getRowCount() == 0)
            status.setStatusInfo("No matches found ");
        else
            status.setStatusInfo("Motif finder search finished  ");
        //results.fireTableStructureChanged();
        if (results != null) results.fireTableDataChanged();
        //table.revalidate();
        //System.out.println(table.getValueAt(0,0));
        table.repaint();
        setProgress(0);
        //repaint();
    }

    public static void main(String[] args) {
        JFrame f = new JFrame();
        f.getContentPane().add(new MotifFinderPanel(null));
        f.setSize(500, 600);
        // f.pack();
        f.setVisible(true);
        f.setDefaultCloseOperation(WindowConstants.DISPOSE_ON_CLOSE);
    }

    private void initComponents() throws Exception {

        jLabelMotif.setText("Motif");
        jLabelNetwork.setText("Network");
        goButton.setToolTipText("");
        goButton.setText("Run");
        goButton.setEnabled(false);
        goButton.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                goButtonPressed(e);
            }
        });
        //jComboMotif.addActionListener(new MotifFinderPanel_jTextField1_actionAdapter(this));
        //jComboNetwork.addActionListener(new MotifFinderPanel_jComboNetwork_actionAdapter(this));
        jLabelBranch.setText("Branch factor");
        jLabelBranchVal.setText("0.5");


        initTopPanel();
        this.setLayout(new BorderLayout());
        // topSplit.setLayout(new BorderLayout());
        topSplit.setOrientation(JSplitPane.VERTICAL_SPLIT);
        bottomSplit.setOrientation(JSplitPane.HORIZONTAL_SPLIT);
        topSplit.setDividerLocation(210);
        topSplit.setTopComponent(topPanel);
        topSplit.setBottomComponent(bottomSplit);
        JScrollPane scroller = new JScrollPane();
        scroller.setViewportView(table);

        bottomSplit.setLeftComponent(scroller);
        bottomSplit.setRightComponent(messagePane);
        bottomSplit.setDividerLocation(250);
        this.add(topSplit);

    }

    private void initTopPanel() {
        topPanel.setLayout(gridBagLayout);
        // topPanel.setPreferredSize(new Dimension(500, 100));

        // int gridx, int gridy, int gridwidth, int gridheight, double weightx, double weighty,
        // int anchor, int fill, Insets insets, int ipadx, int ipady
        topPanel.add(jLabelMotif,
                new GridBagConstraints(0, 0, 1, 1, 1.0, 1.0,
                        GridBagConstraints.NORTHWEST, GridBagConstraints.NONE,
                        new Insets(5, 10, 5, 0), 30, 0));

        topPanel.add(jComboMotif,
                new GridBagConstraints(1, 0, 2, 1, 1.0, 1.0,
                        GridBagConstraints.WEST, GridBagConstraints.HORIZONTAL,
                        new Insets(5, 0, 5, 30), 0, 0));

        topPanel.add(jLabelNetwork,
                new GridBagConstraints(0, 1, 1, 1, 1.0, 1.0,
                        GridBagConstraints.NORTHWEST, GridBagConstraints.NONE,
                        new Insets(5, 10, 5, 0), 30, 0));

        topPanel.add(jComboNetwork,
                new GridBagConstraints(1, 1, 2, 1, 1.0, 1.0,
                        GridBagConstraints.NORTHWEST, GridBagConstraints.HORIZONTAL,
                        new Insets(5, 0, 5, 30), 0, 0));

        topPanel.add(new JLabel("Protein Comparer"), new GridBagConstraints(0, 2, 1, 1, 1.0, 1.0,
                GridBagConstraints.NORTHWEST, GridBagConstraints.HORIZONTAL,
                new Insets(5, 10, 5, 0), 0, 0));


        JScrollPane protcompscroll = new JScrollPane();
        protcompscroll.setViewportView(protCompTable);
        protcompscroll.setPreferredSize(new Dimension(400, 80));
        topPanel.add(protcompscroll, new GridBagConstraints(1, 2, 2, 1, 1.0, 1.0,
                GridBagConstraints.NORTHWEST, GridBagConstraints.HORIZONTAL,
                new Insets(5, 0, 5, 0), 0, 0));


        topPanel.add(jLabelBranch,
                new GridBagConstraints(0, 3, 1, 1, 1.0, 1.0,
                        GridBagConstraints.NORTHWEST, GridBagConstraints.NONE,
                        new Insets(5, 10, 5, 0), 0, 0));

        topPanel.add(jSliderBranchFactor,
                new GridBagConstraints(1, 3, 1, 1, 1.0, 1.0,
                        GridBagConstraints.NORTHWEST, GridBagConstraints.HORIZONTAL,
                        new Insets(5, 0, 5, 0), 0, 0));

        topPanel.add(jLabelBranchVal,
                new GridBagConstraints(2, 3, 1, 1, 1.0, 1.0,
                        GridBagConstraints.NORTHWEST, GridBagConstraints.EAST,
                        new Insets(5, 0, 5, 0), 0, 0));

        topPanel.add(goButton,
                new GridBagConstraints(0, 4, 1, 3, 1.0, 1.0,
                        GridBagConstraints.NORTHWEST, GridBagConstraints.WEST,
                        new Insets(5, 10, 5, 10), 40, 0));

        topPanel.add(jButtonSaveResults,
                new GridBagConstraints(1, 4, 1, 3, 1.0, 1.0,
                        GridBagConstraints.NORTHWEST, GridBagConstraints.WEST,
                        new Insets(5, 10, 5, 10), 40, 0));
        protCompTable.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                if (protCompTable.isReady()) {

                    goButton.setEnabled(true);
                } else {
                    goButton.setEnabled(false);
                }
            }
        });


    }


    public void goButtonPressed(ActionEvent e) {
        if (e.getSource() == this.goButton) {
            if (goButton.getText().equals("Abort") && results != null) {
                results.stopRunning();
                protCompTable.setEnabled(true);
                this.searchFinished();
            } else {
                CyNetwork network = netSet.getNetwork(jComboNetwork.getSelectedItem().toString());
                CyNetwork motif = netSet.getNetwork(jComboMotif.getSelectedItem().toString());
                Collection<CyNetwork> nets = new Vector<CyNetwork>();
                if(results!=null)  {
                table.getSelectionModel().removeListSelectionListener(results);
                }
                nets.add(network);
                nets.add(motif);
                ProteinCompare comp = protCompTable.getComparer(nets);

                //System.out.println("Creating new MotifFinder");
                results = null;
                results = new AssignmentResults(100, messagePane);
                table.clearSelection();

                table.setModel(results);
                double branchFactor = (double) jSliderBranchFactor.getValue() / 100.0;
                System.out.println("Starting new MotifFinder with branchFactor " + branchFactor);
                status.setIndeterminate(false);
                status.setStatusInfo("Status: Searching for motif...  ");

                new MotifFinder(network, motif, comp, branchFactor, this,
                         results, status);
                goButton.setText("Abort");

                table.getSelectionModel().addListSelectionListener(results);


            }

        }

        //results.fireTableDataChanged();
        table.repaint();
        //updateTable();
    }

    public void setProgress(int val) {
        status.setValue(val);
    }


    public void stateChanged(ChangeEvent e) {
        String val = Double.toString((double) jSliderBranchFactor.getValue() / 100.0) + "0000";
        jLabelBranchVal.setText(val.substring(0, 4));

    }

    private class MyTable extends JTable {
        public MyTable() {
            super();
        }

        public String getToolTipText(MouseEvent e) {
            String tip = null;
            java.awt.Point p = e.getPoint();
            int rowIndex = rowAtPoint(p);
            results.getToolTipText(rowIndex);
            return tip;
        }
    }



    private class ListSet {
        private HashMap<String, CyNetwork> string2net = new HashMap<String, CyNetwork>();

        public ListSet(Set<CyNetwork> nets) {
            if (nets == null) return;
            for (CyNetwork net : nets) {
                String name = net.getTitle();
                int lastSlash = -1;
                if (name.contains("/")) lastSlash = name.lastIndexOf('/');
                else if (name.contains("\\")) lastSlash = name.lastIndexOf("\\");
                name = name.substring(lastSlash + 1, name.length());
                string2net.put(name, net);
            }
        }

        public void populateCombo(JComboBox cb) {
            for (String name : string2net.keySet()) {
                cb.addItem(name);
            }
        }

        public CyNetwork getNetwork(String netName) {

            return string2net.get(netName);
        }

    }
}


