package nemi.protein;

import cytoscape.CyNetwork;
import giny.model.Node;
import nemi.compare.ProteinCompare;
import nemi.gui.MessagePane;
import nemi.gui.ViewableResults;

import javax.swing.*;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.TableModelListener;
import java.util.Collections;
import java.util.Comparator;
import java.util.List;
import java.util.Vector;

/**
 * Model for the assignment results table.
 */
class CSAResults extends ViewableResults {

    private Node sourceNode;
    private List<CSAProteinFinder.CsaMatch> matches;
    private CyNetwork net;
    private MessagePane messagePane;
    private CSAProteinFinder csa = new CSAProteinFinder();
    //private NeMiFinalView nemiView;

    public CSAResults(MessagePane messagePane) {
        this.messagePane = messagePane;
        //this.nemiView = nemiView;
        fireTableDataChanged();
    }

    public void setResults(List<CSAProteinFinder.CsaMatch> matches,
                           CyNetwork net, boolean weightResults, int nNeighs, Node sourceNode, ProteinCompare comp) {
        this.net = net;

        this.sourceNode = sourceNode;
        this.matches = matches;

        // todo: for graph plots only; remove
        CsaWeightingStatistics.addMatches(matches, weightResults);
        if(weightResults)
            CsaWeightingStatistics.print();

        Collections.sort(matches, new Comparator<CSAProteinFinder.CsaMatch>() {
            public int compare(CSAProteinFinder.CsaMatch o1, CSAProteinFinder.CsaMatch o2) {
                if (o1.score > o2.score) return -1;
                if (o2.score > o1.score) return 1;
                else return 0;
            }
        });
        fireTableDataChanged();

    }



    public void saveToFile() {
        StringBuffer sb = new StringBuffer();
        if (matches.size() == 0) sb.append("No matches found.");
        for (CSAProteinFinder.CsaMatch m : matches) {
            sb.append(m.toStringDetailed() + "\n");
        }
        nemi.Toolkit.saveToFile(sb);
    }

    // from AbstractTableModel
    public int getRowCount() {
        if (matches == null) {
            return 0;
        }
        return matches.size();
    }

    /*public int compare(Object o1, Object o2) {
        if (o1 instanceof Node && o2 instanceof Node) {
            Node n1 = (Node) o1;
            Node n2 = (Node) o2;
            return scores.get(n2).compareTo(scores.get(n1));
        }
        return 0;  //To change body of implemented methods use File | Settings | File Templates.
    } */

    public int getColumnCount() {
        return 2;  //To change body of implemented methods use File | Settings | File Templates.
    }

    public Object getValueAt(int rowIndex, int columnIndex) {
        if (matches == null) return "Degbug: (matches == null)";
        CSAProteinFinder.CsaMatch match = this.matches.get(rowIndex);
        if (columnIndex == 0) {
            return match.toString();
        } else if (columnIndex == 1) {
            return match.scoreToString();
        }
        return "Errorentry in CsaProteinFinderPanel row " + rowIndex + " column " + columnIndex;  //To change body of implemented methods use File | Settings | File Templates.
    }

    public String getColumnName(int columnIndex) {
        //System.out.println("getting column name for column " + columnIndex);
        switch (columnIndex) {
            case 0:
                return "Node";
            case 1:
                return "Score";
        }
        return "unknown column";

    }

    public Class<?> getColumnClass(int columnIndex) {
        return String.class;  //To change body of implemented methods use File | Settings | File Templates.
    }

    public boolean isCellEditable(int rowIndex, int columnIndex) {
        return false;  //To change body of implemented methods use File | Settings | File Templates.
    }

    public void setValueAt(Object aValue, int rowIndex, int columnIndex) {
        //To change body of implemented methods use File | Settings | File Templates.
    }

    public void addTableModelListener(TableModelListener l) {
    }

    public void removeTableModelListener(TableModelListener l) {
    }

    int lastSelectedIndex;

    // selection changed
    public void valueChanged(ListSelectionEvent e) {
        if (e.getValueIsAdjusting()) return;
        ListSelectionModel lsm = (ListSelectionModel) e.getSource();
        int i =e.getFirstIndex();
                if(!lsm.isSelectedIndex(i)){
                    i = e.getLastIndex();
                }
        //System.out.println("first:" + e.getFirstIndex() + " last: " + e.getLastIndex());

        CSAProteinFinder.CsaMatch match = matches.get(i);
        messagePane.print(match.toStringDetailed());
        List<Node> nodelist = new Vector<Node>();
        nodelist.add(match.target.node);
        List<Node> flaggedNodes = new Vector<Node>(net.getFlaggedNodes());
        net.setFlaggedNodes(flaggedNodes, false);
        net.setFlaggedNodes(nodelist, true);
    }

    public void stopRunning() {
        System.out.println("stop running in csaresults");
    }

    public String getToolTipText(int rowIndex) {
        System.out.println("getting tooltip for csaresults");

        return matches.get(rowIndex).toString();
    }


}
