package nemi.protein;

import nemi.Toolkit;

import java.io.File;
import java.io.FileWriter;
import java.util.Random;
import java.util.Vector;

/**
 * Created by IntelliJ IDEA.
 * User: kristost
 * Date: 02.mar.2006
 * Time: 14:57:03
 * To change this template use File | Settings | File Templates.
 */
public class CsaHungarianComparison {
    public static void main(String[] args) {
        int maxnodes = 15;
        int inc = 1;
        int iters = 1;
        Vector<int[]> results = new Vector<int[]>();
        Vector<int[][]> tables = runCsa(maxnodes, iters, inc, results);
        runHun(tables, iters, inc, results);
    }

    private static void runHun(Vector<int[][]> tables, int iters, int inc, Vector<int[]> results) {
        StringBuilder hunres = new StringBuilder("");
        int nodes = 10, iter = 0;
        long elapsed = 0;
        for (int[][] ints : tables) {
            HungarianMethod m = new HungarianMethod(int2double(ints));

            long start = System.nanoTime();
            byte[][] temphunres = m.munkres();
            elapsed += (System.nanoTime() - start) / 1E6;
            Toolkit.printTable(convertHun2Csa(temphunres), "hunres");
            if ((++iter % iters) == 0) {
                // System.out.println("new hung table " + iter);
                elapsed /= iters;
                hunres.append(nodes + " " + elapsed + " \n");
                nodes += inc;
            }
        }
        System.out.println(hunres);
        writeToFile(hunres.toString(), "c:\\diplom\\data\\results\\100hun_0_500_results.txt");
    }

    private static int[] convertHun2Csa(byte[][] hunres) {
        int[] csares = new int[hunres.length];
        int row = 0;
        for (byte[] bytes : hunres) {
            int col = 0;
            for (byte b : bytes) {
                if (b == 1) csares[row] = col;
                col ++;
            }
            row ++;
        }
        return csares;
    }

    private static Vector<int[][]> runCsa(int maxnodes, int iters, int inc, Vector<int[]> results) {

        StringBuilder csares = new StringBuilder("");
        Vector<int[][]> tables = new Vector<int[][]>();

        for (int nodes = 10, i = 0; nodes <= maxnodes; nodes += inc, i++) {
            System.out.println("csa nodes " + nodes);
            long elapsed = 0;
            for (int iter = 0; iter < iters; iter++) {
                int[][] table = generateTable(nodes, 100, -50);
                // Toolkit.printTable(table, "");
                tables.add(table);
                Csa s = new Csa(table);
                // Toolkit.printTable(s.getMinCost(), "csa : " + nodes);

                long start = System.nanoTime();
                int[] minCost = s.getMinCost();
                elapsed += (System.nanoTime() - start) / 1E6;
                Toolkit.printTable(minCost, "csa");
                results.add(minCost);
                System.gc();
            }

            elapsed /= iters;
            csares.append(nodes + " " + elapsed + " \n");

        }
        System.out.println("Csa results: \n" + csares);
        writeToFile(csares.toString(), "c:\\diplom\\data\\results\\100csa_0_500_results.txt");
        return tables;
    }

    private static double[][] int2double(int[][] ints) {
        double[][] ret = new double[ints.length][ints.length];
        for (int i = 0; i < ints.length; i++) {
            for (int j = 0; j < ints.length; j++) {
                ret[i][j] = ints[i][j];
            }
        }
        return ret;
    }

    public static int[][] generateTable(int size, int range, int minimum) {
        Random r = new Random();
        int[][] table = new int[size][size];
        for (int[] ints : table) {
            for (int i = 0; i < ints.length; i++) {
                ints[i] = r.nextInt(range) + minimum;
            }
        }
        return table;
    }

    public static boolean writeToFile(String sb, String filename) {

        try {
            File f = new File(filename);
            FileWriter fw = new FileWriter(f, false);
            fw.write(sb);
            fw.flush();


        } catch (Exception e) {
            System.err.println(e);
            return false;
        }
        System.out.println("Wrote " + filename);
        return true;
    }


}
